/*****************************************************************************/
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* File: "2link_cylindrical.c"                                               */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*
  File generated by Mathematica on 11/13/2001.
*/
/*****************************************************************************/

/*****************************************************************************/
/*
Idealized DH parameters:

0.  0.   0.  th1  0.
0.  1.   0.  th2  0.
0.  0.5  0.  0.  0.

Idealized inertia tensors:

0.005  0.      0.
0.     0.5025  0.
0.     0.      0.5025

0.0025  0.       0.
0.      0.06375  0.
0.      0.       0.06375


Idealized masses:

1.
0.5

Idealized Pc vectors:

0.5   0.  0.

0.25  0.  0.

*/
/*****************************************************************************/

/*****************************************************************************/
/*
Real DH parameters:

0.  0.   0.  th1  0.
0.  1.   0.  th2  0.
0.  0.5  0.  0.  0.

Real inertia tensors:

0.0055  0.       0.
0.      0.55275  0.
0.      0.       0.55275

0.00225  0.        0.
0.       0.057375  0.
0.       0.        0.057375


Real masses:

1.1
0.45

Real Pc vectors:

0.5   0.  0.

0.25  0.  0.


*/
/*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "manip.h"
#include "robots.h"

/*****************************************************************************/

int robot_assumed_2link_cylindrical(double *f, double *th, double *dth, 
				    double *tau)
{
  double th1;
  double th2;
  double dth1;
  double dth2;
  double tau1;
  double tau2;
  double c1;
  double c2;
  double c3;
  double s1;
  th1 = th[0];
  th2 = th[1];
  dth1 = dth[0];
  dth2 = dth[1];
  tau1 = tau[0];
  tau2 = tau[1];
  c1 = Cos(th1);
  c2 = Cos(th2);
  c3 = Cos(th1 + th2);
  s1 = Sin(th2);
  f[0] = dth1;
  f[1] = dth2;
  f[2] = (0.095*(-1.*c1*9.81 - 0.125*c3*9.81 + 0.25*dth1*dth2*s1 + 0.125*Power(dth2,2)*s1 + tau1))/(0.1189875 - 0.015625*Power(c2,2)) + ((-0.095 - 0.125*c2)*(-0.125*c3*9.81 - 0.125*Power(dth1,2)*s1 + tau2))/(0.1189875 - 0.015625*Power(c2,2));
  f[3] = ((-0.095 - 0.125*c2)*(-1.*c1*9.81 - 0.125*c3*9.81 + 0.25*dth1*dth2*s1 + 0.125*Power(dth2,2)*s1 + tau1))/(0.1189875 - 0.015625*Power(c2,2)) + ((1.3475000000000001 + 0.25*c2)*(-0.125*c3*9.81 - 0.125*Power(dth1,2)*s1 + tau2))/(0.1189875 - 0.015625*Power(c2,2));

  return(1);
}

/*****************************************************************************/

int model_assumed_2link_cylindrical(double *m, double *v, double *g, 
				    double *th, double *dth)
{
  double th1;
  double th2;
  double dth1;
  double dth2;
  double c1;
  double c2;
  double c3;
  double s1;
  th1 = th[0];
  th2 = th[1];
  dth1 = dth[0];
  dth2 = dth[1];
  c1 = Cos(th1);
  c2 = Cos(th2);
  c3 = Cos(th1 + th2);
  s1 = Sin(th2);
  if(m != NULL) {
   m[0] = 0.25*(5.390000000000001 + 1.*c2);
   m[1] = 0.25*(0.38 + 0.5*c2);
   m[2] = 0.25*(0.38 + 0.5*c2);
   m[3] = 0.095;
  }
  if(v != NULL) {
   v[0] = -0.25*dth1*dth2*s1 - 0.125*Power(dth2,2)*s1;
   v[1] = 0.125*Power(dth1,2)*s1;
  }
  if(g != NULL) {
   g[0] = 2.4525*(4.*c1 + 0.5*c3);
   g[1] = 1.22625*c3;
  }

  return(1);
}

/*****************************************************************************/

int robot_actual_2link_cylindrical(double *f, double *th, double *dth, 
				   double *tau)
{
  double th1;
  double th2;
  double dth1;
  double dth2;
  double tau1;
  double tau2;
  double c1;
  double c2;
  double c3;
  double s1;
  th1 = th[0];
  th2 = th[1];
  dth1 = dth[0];
  dth2 = dth[1];
  tau1 = tau[0];
  tau2 = tau[1];
  c1 = Cos(th1);
  c2 = Cos(th2);
  c3 = Cos(th1 + th2);
  s1 = Sin(th2);
  f[0] = dth1;
  f[1] = dth2;
  f[2] = (0.0855*(-1.*c1*9.81 - 0.1125*c3*9.81 + 0.225*dth1*dth2*s1 + 0.1125*Power(dth2,2)*s1 + tau1))/(0.10924762500000001 - 0.01265625*Power(c2,2)) + ((-0.0855 - 0.1125*c2)*(-0.1125*c3*9.81 - 0.1125*Power(dth1,2)*s1 + tau2))/(0.10924762500000001 - 0.01265625*Power(c2,2));
  f[3] = ((-0.0855 - 0.1125*c2)*(-1.*c1*9.81 - 0.1125*c3*9.81 + 0.225*dth1*dth2*s1 + 0.1125*Power(dth2,2)*s1 + tau1))/(0.10924762500000001 - 0.01265625*Power(c2,2)) + ((1.36325 + 0.225*c2)*(-0.1125*c3*9.81 - 0.1125*Power(dth1,2)*s1 + tau2))/(0.10924762500000001 - 0.01265625*Power(c2,2));

  return(1);
}

/*****************************************************************************/

int model_actual_2link_cylindrical(double *m, double *v, double *g, 
				   double *th, double *dth)
{
  double th1;
  double th2;
  double dth1;
  double dth2;
  double c1;
  double c2;
  double c3;
  double s1;
  th1 = th[0];
  th2 = th[1];
  dth1 = dth[0];
  dth2 = dth[1];
  c1 = Cos(th1);
  c2 = Cos(th2);
  c3 = Cos(th1 + th2);
  s1 = Sin(th2);
  if(m != NULL) {
   m[0] = 0.25*(5.453 + 0.9*c2);
   m[1] = 0.25*(0.342 + 0.45*c2);
   m[2] = 0.25*(0.342 + 0.45*c2);
   m[3] = 0.0855;
  }
  if(v != NULL) {
   v[0] = -0.225*dth1*dth2*s1 - 0.1125*Power(dth2,2)*s1;
   v[1] = 0.1125*Power(dth1,2)*s1;
  }
  if(g != NULL) {
   g[0] = 2.4525*(4.*c1 + 0.45*c3);
   g[1] = 1.103625*c3;
  }

  return(1);
}

/*****************************************************************************/
