/*****************************************************************************/
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* File: "2link_pointmass.c"                                                 */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*
  File generated by Mathematica on 11/13/2001.
*/
/*****************************************************************************/

/*****************************************************************************/
/*
Idealized DH parameters:

0.  0.   0.  th1  0.
0.  1.   0.  th2  0.
0.  0.5  0.  0.  0.

Idealized inertia tensors:

0.  0.  0.
0.  0.  0.
0.  0.  0.

0.  0.  0.
0.  0.  0.
0.  0.  0.


Idealized masses:

1.
0.5

Idealized Pc vectors:

1.   0.  0.

0.5  0.  0.

*/
/*****************************************************************************/

/*****************************************************************************/
/*
Real DH parameters:

0.  0.   0.  th1  0.
0.  1.   0.  th2  0.
0.  0.5  0.  0.  0.

Real inertia tensors:

0.  0.  0.
0.  0.  0.
0.  0.  0.

0.  0.  0.
0.  0.  0.
0.  0.  0.


Real masses:

1.1
0.45

Real Pc vectors:

1.   0.  0.

0.5  0.  0.


*/
/*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "manip.h"
#include "robots.h"


/*****************************************************************************/

int robot_assumed_2link_pointmass(double *f, double *th, double *dth, 
				  double *tau)
{
  double th1;
  double th2;
  double dth1;
  double dth2;
  double tau1;
  double tau2;
  double c1;
  double c2;
  double c3;
  double s1;
  th1 = th[0];
  th2 = th[1];
  dth1 = dth[0];
  dth2 = dth[1];
  tau1 = tau[0];
  tau2 = tau[1];
  c1 = Cos(th1);
  c2 = Cos(th2);
  c3 = Cos(th1 + th2);
  s1 = Sin(th2);
  f[0] = dth1;
  f[1] = dth2;
  f[2] = (0.125*(-1.5*c1*9.81 - 0.25*c3*9.81 + 0.5*dth1*dth2*s1 + 0.25*Power(dth2,2)*s1 + tau1))/(0.1875 - 0.0625*Power(c2,2)) + ((-0.125 - 0.25*c2)*(-0.25*c3*9.81 - 0.25*Power(dth1,2)*s1 + tau2))/(0.1875 - 0.0625*Power(c2,2));
  f[3] = ((-0.125 - 0.25*c2)*(-1.5*c1*9.81 - 0.25*c3*9.81 + 0.5*dth1*dth2*s1 + 0.25*Power(dth2,2)*s1 + tau1))/(0.1875 - 0.0625*Power(c2,2)) + ((1.625 + 0.5*c2)*(-0.25*c3*9.81 - 0.25*Power(dth1,2)*s1 + tau2))/(0.1875 - 0.0625*Power(c2,2));

  return(1);
}

/*****************************************************************************/

int model_assumed_2link_pointmass(double *m, double *v, double *g, 
				  double *th, double *dth)
{
  double th1;
  double th2;
  double dth1;
  double dth2;
  double c1;
  double c2;
  double c3;
  double s1;
  th1 = th[0];
  th2 = th[1];
  dth1 = dth[0];
  dth2 = dth[1];
  c1 = Cos(th1);
  c2 = Cos(th2);
  c3 = Cos(th1 + th2);
  s1 = Sin(th2);
  if(m != NULL) {
   m[0] = 1.625 + 0.5*c2;
   m[1] = 0.125 + 0.25*c2;
   m[2] = 0.125 + 0.25*c2;
   m[3] = 0.125;
  }
  if(v != NULL) {
   v[0] = -0.5*dth1*dth2*s1 - 0.25*Power(dth2,2)*s1;
   v[1] = 0.25*Power(dth1,2)*s1;
  }
  if(g != NULL) {
   g[0] = 9.81*(1.5*c1 + 0.25*c3);
   g[1] = 2.4525*c3;
  }

  return(1);
}

/*****************************************************************************/

int robot_actual_2link_pointmass(double *f, double *th, double *dth, 
				 double *tau)
{
  double th1;
  double th2;
  double dth1;
  double dth2;
  double tau1;
  double tau2;
  double c1;
  double c2;
  double c3;
  double s1;
  th1 = th[0];
  th2 = th[1];
  dth1 = dth[0];
  dth2 = dth[1];
  tau1 = tau[0];
  tau2 = tau[1];
  c1 = Cos(th1);
  c2 = Cos(th2);
  c3 = Cos(th1 + th2);
  s1 = Sin(th2);
  f[0] = dth1;
  f[1] = dth2;
  f[2] = (0.1125*(-1.55*c1*9.81 - 0.225*c3*9.81 + 0.45*dth1*dth2*s1 + 0.225*Power(dth2,2)*s1 + tau1))/(0.174375 - 0.050625*Power(c2,2)) + ((-0.1125 - 0.225*c2)*(-0.225*c3*9.81 - 0.225*Power(dth1,2)*s1 + tau2))/(0.174375 - 0.050625*Power(c2,2));
  f[3] = ((-0.1125 - 0.225*c2)*(-1.55*c1*9.81 - 0.225*c3*9.81 + 0.45*dth1*dth2*s1 + 0.225*Power(dth2,2)*s1 + tau1))/(0.174375 - 0.050625*Power(c2,2)) + ((1.6625 + 0.45*c2)*(-0.225*c3*9.81 - 0.225*Power(dth1,2)*s1 + tau2))/(0.174375 - 0.050625*Power(c2,2));

  return(1);
}

/*****************************************************************************/

int model_actual_2link_pointmass(double *m, double *v, double *g, 
				 double *th, double *dth)
{
  double th1;
  double th2;
  double dth1;
  double dth2;
  double c1;
  double c2;
  double c3;
  double s1;
  th1 = th[0];
  th2 = th[1];
  dth1 = dth[0];
  dth2 = dth[1];
  c1 = Cos(th1);
  c2 = Cos(th2);
  c3 = Cos(th1 + th2);
  s1 = Sin(th2);
  if(m != NULL) {
   m[0] = 1.6625 + 0.45*c2;
   m[1] = 0.1125 + 0.225*c2;
   m[2] = 0.1125 + 0.225*c2;
   m[3] = 0.1125;
  }
  if(v != NULL) {
   v[0] = -0.45*dth1*dth2*s1 - 0.225*Power(dth2,2)*s1;
   v[1] = 0.225*Power(dth1,2)*s1;
  }
  if(g != NULL) {
   g[0] = 9.81*(1.55*c1 + 0.225*c3);
   g[1] = 2.20725*c3;
  }

  return(1);
}

/*****************************************************************************/
