/*****************************************************************************/
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* File: "4link_pointmass.c"                                                 */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*
  File generated by Mathematica on 11/13/2001.
*/
/*****************************************************************************/

/*****************************************************************************/
/*
Idealized DH parameters:

0.      0.    0.  th1  0.
1.5708  0.    0.  th2  0.
0.      1.    0.  th3  0.
0.      0.75  0.  th4  0.
0.      0.5   0.  0.  0.

Idealized inertia tensors:

0.  0.  0.
0.  0.  0.
0.  0.  0.

0.  0.  0.
0.  0.  0.
0.  0.  0.

0.  0.  0.
0.  0.  0.
0.  0.  0.

0.  0.  0.
0.  0.  0.
0.  0.  0.


Idealized masses:

1.
2.
1.5
1.

Idealized Pc vectors:

0.    0.  0.

1.    0.  0.

0.75  0.  0.

0.5   0.  0.

*/
/*****************************************************************************/

/*****************************************************************************/
/*
Real DH parameters:

0.      0.    0.  th1  0.
1.5708  0.    0.  th2  0.
0.      1.    0.  th3  0.
0.      0.75  0.  th4  0.
0.      0.5   0.  0.  0.

Real inertia tensors:

0.  0.  0.
0.  0.  0.
0.  0.  0.

0.  0.  0.
0.  0.  0.
0.  0.  0.

0.  0.  0.
0.  0.  0.
0.  0.  0.

0.  0.  0.
0.  0.  0.
0.  0.  0.


Real masses:

1.
1.8
1.575
1.1

Real Pc vectors:

0.    0.  0.

1.    0.  0.

0.75  0.  0.

0.5   0.  0.


*/
/*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "manip.h"
#include "robots.h"

/*****************************************************************************/

int robot_assumed_4link_pointmass(double *f, double *th, double *dth, 
				  double *tau)
{
  double th1;
  double th2;
  double th3;
  double th4;
  double dth1;
  double dth2;
  double dth3;
  double dth4;
  double tau1;
  double tau2;
  double tau3;
  double tau4;
  double c1;
  double c2;
  double c3;
  double c4;
  double c5;
  double c6;
  double c7;
  double c8;
  double c9;
  double c10;
  double c11;
  double c12;
  double s1;
  double s2;
  double s3;
  double s4;
  double s5;
  double s6;
  double s7;
  double s8;
  double s9;
  th1 = th[0];
  th2 = th[1];
  th3 = th[2];
  th4 = th[3];
  dth1 = dth[0];
  dth2 = dth[1];
  dth3 = dth[2];
  dth4 = dth[3];
  tau1 = tau[0];
  tau2 = tau[1];
  tau3 = tau[2];
  tau4 = tau[3];
  c1 = Cos(th2);
  c2 = Cos(2.*th2);
  c3 = Cos(th3);
  c4 = Cos(th2 + th3);
  c5 = Cos(2.*th2 + th3);
  c6 = Cos(2.*th2 + 2.*th3);
  c7 = Cos(th4);
  c8 = Cos(th3 + th4);
  c9 = Cos(th2 + th3 + th4);
  c10 = Cos(2.*th2 + th3 + th4);
  c11 = Cos(2.*th2 + 2.*th3 + th4);
  c12 = Cos(2.*th2 + 2.*th3 + 2.*th4);
  s1 = Sin(2.*th2);
  s2 = Sin(th3);
  s3 = Sin(2.*th2 + th3);
  s4 = Sin(2.*th2 + 2.*th3);
  s5 = Sin(th4);
  s6 = Sin(th3 + th4);
  s7 = Sin(2.*th2 + th3 + th4);
  s8 = Sin(2.*th2 + 2.*th3 + th4);
  s9 = Sin(2.*th2 + 2.*th3 + 2.*th4);
  f[0] = dth1;
  f[1] = dth2;
  f[2] = dth3;
  f[3] = dth4;
  f[4] = (-1.*dth1*(dth2*(-4.5*s1 - 3.75*s3 - 1.40625*s4 - 1.*s7 - 0.75*s8 - 0.25*s9) + dth3*(-1.875*s2 - 1.875*s3 - 1.40625*s4 - 0.5*s6 - 0.5*s7 - 0.75*s8 - 0.25*s9) + dth4*(-0.375*s5 - 0.5*s6 - 0.5*s7 - 0.375*s8 - 0.25*s9)) + tau1)/(3.078125 + 0.5*c10 + 0.375*c11 + 0.125*c12 + 2.25*c2 + 1.875*c3 + 1.875*c5 + 0.703125*c6 + 0.375*c7 + 0.5*c8);
  f[5] = ((0.3515625 - 0.140625*Power(c7,2))*(-4.5*c1*9.81 - 1.875*c4*9.81 - 0.5*c9*9.81 - 2.25*Power(dth1,2)*s1 + 3.75*dth2*dth3*s2 + 1.875*Power(dth3,2)*s2 - 1.875*Power(dth1,2)*s3 - 0.703125*Power(dth1,2)*s4 + 0.75*dth2*dth4*s5 + 0.75*dth3*dth4*s5 + 0.375*Power(dth4,2)*s5 + 1.*dth2*dth3*s6 + 0.5*Power(dth3,2)*s6 + 1.*dth2*dth4*s6 + 1.*dth3*dth4*s6 + 0.5*Power(dth4,2)*s6 - 0.5*Power(dth1,2)*s7 - 0.375*Power(dth1,2)*s8 - 0.125*Power(dth1,2)*s9 + tau2))/(1.58203125 - 0.87890625*Power(c3,2) - 0.6328125*Power(c7,2) + 0.703125*c3*c7*c8 - 0.3515625*Power(c8,2)) - (1.*(0.3515625 + 0.46875*c3 - 0.140625*Power(c7,2) - 0.1875*c7*c8)*(-1.875*c4*9.81 - 0.5*c9*9.81 - 0.9375*Power(dth1,2)*s2 - 1.875*Power(dth2,2)*s2 - 0.9375*Power(dth1,2)*s3 - 0.703125*Power(dth1,2)*s4 + 0.75*dth2*dth4*s5 + 0.75*dth3*dth4*s5 + 0.375*Power(dth4,2)*s5 - 0.25*Power(dth1,2)*s6 - 0.5*Power(dth2,2)*s6 - 0.25*Power(dth1,2)*s7 - 0.375*Power(dth1,2)*s8 - 0.125*Power(dth1,2)*s9 + tau3))/(1.58203125 - 0.87890625*Power(c3,2) - 0.6328125*Power(c7,2) + 0.703125*c3*c7*c8 - 0.3515625*Power(c8,2)) + ((0.46875*c3 + 0.703125*c3*c7 - 0.703125*c8 - 0.1875*c7*c8)*(-0.5*c9*9.81 - 0.1875*Power(dth1,2)*s5 - 0.375*Power(dth2,2)*s5 - 0.75*dth2*dth3*s5 - 0.375*Power(dth3,2)*s5 - 0.25*Power(dth1,2)*s6 - 0.5*Power(dth2,2)*s6 - 0.25*Power(dth1,2)*s7 - 0.1875*Power(dth1,2)*s8 - 0.125*Power(dth1,2)*s9 + tau4))/(1.58203125 - 0.87890625*Power(c3,2) - 0.6328125*Power(c7,2) + 0.703125*c3*c7*c8 - 0.3515625*Power(c8,2));
  f[6] = (-1.*(0.3515625 + 0.46875*c3 - 0.140625*Power(c7,2) - 0.1875*c7*c8)*(-4.5*c1*9.81 - 1.875*c4*9.81 - 0.5*c9*9.81 - 2.25*Power(dth1,2)*s1 + 3.75*dth2*dth3*s2 + 1.875*Power(dth3,2)*s2 - 1.875*Power(dth1,2)*s3 - 0.703125*Power(dth1,2)*s4 + 0.75*dth2*dth4*s5 + 0.75*dth3*dth4*s5 + 0.375*Power(dth4,2)*s5 + 1.*dth2*dth3*s6 + 0.5*Power(dth3,2)*s6 + 1.*dth2*dth4*s6 + 1.*dth3*dth4*s6 + 0.5*Power(dth4,2)*s6 - 0.5*Power(dth1,2)*s7 - 0.375*Power(dth1,2)*s8 - 0.125*Power(dth1,2)*s9 + tau2))/(1.58203125 - 0.87890625*Power(c3,2) - 0.6328125*Power(c7,2) + 0.703125*c3*c7*c8 - 0.3515625*Power(c8,2)) + ((1.4765625 + 0.9375*c3 - 0.140625*Power(c7,2) - 0.375*c7*c8 - 0.25*Power(c8,2))*(-1.875*c4*9.81 - 0.5*c9*9.81 - 0.9375*Power(dth1,2)*s2 - 1.875*Power(dth2,2)*s2 - 0.9375*Power(dth1,2)*s3 - 0.703125*Power(dth1,2)*s4 + 0.75*dth2*dth4*s5 + 0.75*dth3*dth4*s5 + 0.375*Power(dth4,2)*s5 - 0.25*Power(dth1,2)*s6 - 0.5*Power(dth2,2)*s6 - 0.25*Power(dth1,2)*s7 - 0.375*Power(dth1,2)*s8 - 0.125*Power(dth1,2)*s9 + tau3))/(1.58203125 - 0.87890625*Power(c3,2) - 0.6328125*Power(c7,2) + 0.703125*c3*c7*c8 - 0.3515625*Power(c8,2)) - (1.*(1.125 + 0.46875*c3 + 1.6875*c7 + 0.703125*c3*c7 - 0.703125*c8 - 0.9375*c3*c8 - 0.1875*c7*c8 - 0.25*Power(c8,2))*(-0.5*c9*9.81 - 0.1875*Power(dth1,2)*s5 - 0.375*Power(dth2,2)*s5 - 0.75*dth2*dth3*s5 - 0.375*Power(dth3,2)*s5 - 0.25*Power(dth1,2)*s6 - 0.5*Power(dth2,2)*s6 - 0.25*Power(dth1,2)*s7 - 0.1875*Power(dth1,2)*s8 - 0.125*Power(dth1,2)*s9 + tau4))/(1.58203125 - 0.87890625*Power(c3,2) - 0.6328125*Power(c7,2) + 0.703125*c3*c7*c8 - 0.3515625*Power(c8,2));
  f[7] = ((0.46875*c3 + 0.703125*c3*c7 - 0.703125*c8 - 0.1875*c7*c8)*(-4.5*c1*9.81 - 1.875*c4*9.81 - 0.5*c9*9.81 - 2.25*Power(dth1,2)*s1 + 3.75*dth2*dth3*s2 + 1.875*Power(dth3,2)*s2 - 1.875*Power(dth1,2)*s3 - 0.703125*Power(dth1,2)*s4 + 0.75*dth2*dth4*s5 + 0.75*dth3*dth4*s5 + 0.375*Power(dth4,2)*s5 + 1.*dth2*dth3*s6 + 0.5*Power(dth3,2)*s6 + 1.*dth2*dth4*s6 + 1.*dth3*dth4*s6 + 0.5*Power(dth4,2)*s6 - 0.5*Power(dth1,2)*s7 - 0.375*Power(dth1,2)*s8 - 0.125*Power(dth1,2)*s9 + tau2))/(1.58203125 - 0.87890625*Power(c3,2) - 0.6328125*Power(c7,2) + 0.703125*c3*c7*c8 - 0.3515625*Power(c8,2)) - (1.*(1.125 + 0.46875*c3 + 1.6875*c7 + 0.703125*c3*c7 - 0.703125*c8 - 0.9375*c3*c8 - 0.1875*c7*c8 - 0.25*Power(c8,2))*(-1.875*c4*9.81 - 0.5*c9*9.81 - 0.9375*Power(dth1,2)*s2 - 1.875*Power(dth2,2)*s2 - 0.9375*Power(dth1,2)*s3 - 0.703125*Power(dth1,2)*s4 + 0.75*dth2*dth4*s5 + 0.75*dth3*dth4*s5 + 0.375*Power(dth4,2)*s5 - 0.25*Power(dth1,2)*s6 - 0.5*Power(dth2,2)*s6 - 0.25*Power(dth1,2)*s7 - 0.375*Power(dth1,2)*s8 - 0.125*Power(dth1,2)*s9 + tau3))/(1.58203125 - 0.87890625*Power(c3,2) - 0.6328125*Power(c7,2) + 0.703125*c3*c7*c8 - 0.3515625*Power(c8,2)) + ((7.453125 - 3.515625*Power(c3,2) + 3.375*c7 - 1.875*c3*c8 - 0.25*Power(c8,2))*(-0.5*c9*9.81 - 0.1875*Power(dth1,2)*s5 - 0.375*Power(dth2,2)*s5 - 0.75*dth2*dth3*s5 - 0.375*Power(dth3,2)*s5 - 0.25*Power(dth1,2)*s6 - 0.5*Power(dth2,2)*s6 - 0.25*Power(dth1,2)*s7 - 0.1875*Power(dth1,2)*s8 - 0.125*Power(dth1,2)*s9 + tau4))/(1.58203125 - 0.87890625*Power(c3,2) - 0.6328125*Power(c7,2) + 0.703125*c3*c7*c8 - 0.3515625*Power(c8,2));

  return(1);
}

/*****************************************************************************/

int model_assumed_4link_pointmass(double *m, double *v, double *g, 
				  double *th, double *dth)
{
  double th1;
  double th2;
  double th3;
  double th4;
  double dth1;
  double dth2;
  double dth3;
  double dth4;
  double c1;
  double c2;
  double c3;
  double c4;
  double c5;
  double c6;
  double c7;
  double c8;
  double c9;
  double c10;
  double c11;
  double c12;
  double s1;
  double s2;
  double s3;
  double s4;
  double s5;
  double s6;
  double s7;
  double s8;
  double s9;
  th1 = th[0];
  th2 = th[1];
  th3 = th[2];
  th4 = th[3];
  dth1 = dth[0];
  dth2 = dth[1];
  dth3 = dth[2];
  dth4 = dth[3];
  c1 = Cos(th2);
  c2 = Cos(2.*th2);
  c3 = Cos(th3);
  c4 = Cos(th2 + th3);
  c5 = Cos(2.*th2 + th3);
  c6 = Cos(2.*th2 + 2.*th3);
  c7 = Cos(th4);
  c8 = Cos(th3 + th4);
  c9 = Cos(th2 + th3 + th4);
  c10 = Cos(2.*th2 + th3 + th4);
  c11 = Cos(2.*th2 + 2.*th3 + th4);
  c12 = Cos(2.*th2 + 2.*th3 + 2.*th4);
  s1 = Sin(2.*th2);
  s2 = Sin(th3);
  s3 = Sin(2.*th2 + th3);
  s4 = Sin(2.*th2 + 2.*th3);
  s5 = Sin(th4);
  s6 = Sin(th3 + th4);
  s7 = Sin(2.*th2 + th3 + th4);
  s8 = Sin(2.*th2 + 2.*th3 + th4);
  s9 = Sin(2.*th2 + 2.*th3 + 2.*th4);
  if(m != NULL) {
   m[0] = 0.5*(6.15625 + 1.*c10 + 0.75*c11 + 0.25*c12 + 4.5*c2 + 3.75*c3 + 3.75*c5 + 1.40625*c6 + 0.75*c7 + 1.*c8);
   m[1] = 0;
   m[2] = 0;
   m[3] = 0;
   m[4] = 0;
   m[5] = 0.5*(12.3125 + 7.5*c3 + 1.5*c7 + 2.*c8);
   m[6] = 0.5*(3.3125 + 3.75*c3 + 1.5*c7 + 1.*c8);
   m[7] = 0.5*(0.5 + 0.75*c7 + 1.*c8);
   m[8] = 0;
   m[9] = 0.5*(3.3125 + 3.75*c3 + 1.5*c7 + 1.*c8);
   m[10] = 0.5*(3.3125 + 1.5*c7);
   m[11] = 0.5*(0.5 + 0.75*c7);
   m[12] = 0;
   m[13] = 0.5*(0.5 + 0.75*c7 + 1.*c8);
   m[14] = 0.5*(0.5 + 0.75*c7);
   m[15] = 0.25;
  }
  if(v != NULL) {
   v[0] = dth1*(dth2*(-4.5*s1 - 3.75*s3 - 1.40625*s4 - 1.*s7 - 0.75*s8 - 0.25*s9) + dth3*(-1.875*s2 - 1.875*s3 - 1.40625*s4 - 0.5*s6 - 0.5*s7 - 0.75*s8 - 0.25*s9) + dth4*(-0.375*s5 - 0.5*s6 - 0.5*s7 - 0.375*s8 - 0.25*s9));
   v[1] = 2.25*Power(dth1,2)*s1 - 3.75*dth2*dth3*s2 - 1.875*Power(dth3,2)*s2 + 1.875*Power(dth1,2)*s3 + 0.703125*Power(dth1,2)*s4 - 0.75*dth2*dth4*s5 - 0.75*dth3*dth4*s5 - 0.375*Power(dth4,2)*s5 - 1.*dth2*dth3*s6 - 0.5*Power(dth3,2)*s6 - 1.*dth2*dth4*s6 - 1.*dth3*dth4*s6 - 0.5*Power(dth4,2)*s6 + 0.5*Power(dth1,2)*s7 + 0.375*Power(dth1,2)*s8 + 0.125*Power(dth1,2)*s9;
   v[2] = 0.9375*Power(dth1,2)*s2 + 1.875*Power(dth2,2)*s2 + 0.9375*Power(dth1,2)*s3 + 0.703125*Power(dth1,2)*s4 - 0.75*dth2*dth4*s5 - 0.75*dth3*dth4*s5 - 0.375*Power(dth4,2)*s5 + 0.25*Power(dth1,2)*s6 + 0.5*Power(dth2,2)*s6 + 0.25*Power(dth1,2)*s7 + 0.375*Power(dth1,2)*s8 + 0.125*Power(dth1,2)*s9;
   v[3] = 0.1875*Power(dth1,2)*s5 + 0.375*Power(dth2,2)*s5 + 0.75*dth2*dth3*s5 + 0.375*Power(dth3,2)*s5 + 0.25*Power(dth1,2)*s6 + 0.5*Power(dth2,2)*s6 + 0.25*Power(dth1,2)*s7 + 0.1875*Power(dth1,2)*s8 + 0.125*Power(dth1,2)*s9;
  }
  if(g != NULL) {
   g[0] = 0;
   g[1] = 4.905*(9.*c1 + 3.75*c4 + 1.*c9);
   g[2] = 4.905*(3.75*c4 + 1.*c9);
   g[3] = 4.905*c9;
  }

  return(1);
}

/*****************************************************************************/

int robot_actual_4link_pointmass(double *f, double *th, double *dth, 
				 double *tau)
{
  double th1;
  double th2;
  double th3;
  double th4;
  double dth1;
  double dth2;
  double dth3;
  double dth4;
  double tau1;
  double tau2;
  double tau3;
  double tau4;
  double c1;
  double c2;
  double c3;
  double c4;
  double c5;
  double c6;
  double c7;
  double c8;
  double c9;
  double c10;
  double c11;
  double c12;
  double c13;
  double c14;
  double c15;
  double c16;
  double c17;
  double c18;
  double s1;
  double s2;
  double s3;
  double s4;
  double s5;
  double s6;
  double s7;
  double s8;
  double s9;
  double s10;
  double s11;
  double s12;
  double s13;
  double s14;
  double s15;
  th1 = th[0];
  th2 = th[1];
  th3 = th[2];
  th4 = th[3];
  dth1 = dth[0];
  dth2 = dth[1];
  dth3 = dth[2];
  dth4 = dth[3];
  tau1 = tau[0];
  tau2 = tau[1];
  tau3 = tau[2];
  tau4 = tau[3];
  c1 = Cos(th1 - 1.*th2);
  c2 = Cos(th2);
  c3 = Cos(2.*th2);
  c4 = Cos(th1 + th2);
  c5 = Cos(th1 - 1.*th2 - 1.*th3);
  c6 = Cos(th3);
  c7 = Cos(th2 + th3);
  c8 = Cos(th1 + th2 + th3);
  c9 = Cos(2.*th2 + th3);
  c10 = Cos(2.*th2 + 2.*th3);
  c11 = Cos(th1 - 1.*th2 - 1.*th3 - 1.*th4);
  c12 = Cos(th4);
  c13 = Cos(th3 + th4);
  c14 = Cos(th2 + th3 + th4);
  c15 = Cos(th1 + th2 + th3 + th4);
  c16 = Cos(2.*th2 + th3 + th4);
  c17 = Cos(2.*th2 + 2.*th3 + th4);
  c18 = Cos(2.*th2 + 2.*th3 + 2.*th4);
  s1 = Sin(th1 - 1.*th2);
  s2 = Sin(2.*th2);
  s3 = Sin(th1 + th2);
  s4 = Sin(th1 - 1.*th2 - 1.*th3);
  s5 = Sin(th3);
  s6 = Sin(th1 + th2 + th3);
  s7 = Sin(2.*th2 + th3);
  s8 = Sin(2.*th2 + 2.*th3);
  s9 = Sin(th1 - 1.*th2 - 1.*th3 - 1.*th4);
  s10 = Sin(th4);
  s11 = Sin(th3 + th4);
  s12 = Sin(th1 + th2 + th3 + th4);
  s13 = Sin(2.*th2 + th3 + th4);
  s14 = Sin(2.*th2 + 2.*th3 + th4);
  s15 = Sin(2.*th2 + 2.*th3 + 2.*th4);
  f[0] = dth1;
  f[1] = dth2;
  f[2] = dth3;
  f[3] = dth4;
  f[4] = (-1.*dth1*(dth4*(-0.4125*s10 - 0.55*s11 - 0.55*s13 - 0.4125*s14 - 0.275*s15) + dth2*(-1.1*s13 - 0.825*s14 - 0.275*s15 - 4.475*s2 - 4.0125*s7 - 1.5046875*s8) + dth3*(-0.55*s11 - 0.55*s13 - 0.825*s14 - 0.275*s15 - 2.00625*s5 - 2.00625*s7 - 1.5046875*s8)) - 1.*9.81*(0.02259870446365638*c1 + 0.0027774944033544155*c11 + 0.0027774944033544155*c15 + 0.02259870446365638*c4 + 0.010131542084963268*c5 + 0.010131542084963268*c8 - 0.022598704463656385*s1 - 0.0027774944033544155*s12 - 0.022598704463656385*s3 - 0.010131542084963268*s4 - 0.010131542084963268*s6 - 0.0027774944033544155*s9) + tau1)/(3.12734375 + 0.75234375*c10 + 0.4125*c12 + 0.55*c13 + 0.55*c16 + 0.4125*c17 + 0.1375*c18 + 2.2375*c3 + 2.00625*c6 + 2.00625*c9);
  f[5] = ((0.4137890625000001 - 0.17015624999999998*Power(c12,2))*(0.022598704463656385*c1*9.81 + 0.0027774944033544155*c11*9.81 - 0.5499438918641743*c14*9.81 - 0.0027774944033544155*c15*9.81 - 4.474543483803964*c2*9.81 - 0.02259870446365638*c4*9.81 + 0.010131542084963268*c5*9.81 - 2.0060453328227266*c7*9.81 - 0.010131542084963268*c8*9.81 - 0.02259870446365638*9.81*s1 + 0.825*dth2*dth4*s10 + 0.825*dth3*dth4*s10 + 0.4125*Power(dth4,2)*s10 + 1.1*dth2*dth3*s11 + 0.55*Power(dth3,2)*s11 + 1.1*dth2*dth4*s11 + 1.1*dth3*dth4*s11 + 0.55*Power(dth4,2)*s11 + 0.0027774944033544155*9.81*s12 - 0.55*Power(dth1,2)*s13 - 0.4125*Power(dth1,2)*s14 - 0.1375*Power(dth1,2)*s15 - 2.2375*Power(dth1,2)*s2 + 0.022598704463656385*9.81*s3 - 0.010131542084963268*9.81*s4 + 4.0125*dth2*dth3*s5 + 2.00625*Power(dth3,2)*s5 + 0.010131542084963268*9.81*s6 - 2.00625*Power(dth1,2)*s7 - 0.75234375*Power(dth1,2)*s8 - 0.0027774944033544155*9.81*s9 + tau2))/(1.8517060546875006 - 0.76144921875*Power(c12,2) - 0.45516796875000015*Power(c13,2) + 0.9103359375*c12*c13*c6 - 1.1068857421875002*Power(c6,2)) - (1.*(0.4137890625000001 - 0.17015624999999998*Power(c12,2) - 0.226875*c12*c13 + 0.55171875*c6)*(0.0027774944033544155*c11*9.81 - 0.5499438918641743*c14*9.81 - 0.0027774944033544155*c15*9.81 + 0.010131542084963268*c5*9.81 - 2.0060453328227266*c7*9.81 - 0.010131542084963268*c8*9.81 + 0.825*dth2*dth4*s10 + 0.825*dth3*dth4*s10 + 0.4125*Power(dth4,2)*s10 - 0.275*Power(dth1,2)*s11 - 0.55*Power(dth2,2)*s11 + 0.0027774944033544155*9.81*s12 - 0.275*Power(dth1,2)*s13 - 0.4125*Power(dth1,2)*s14 - 0.1375*Power(dth1,2)*s15 - 0.010131542084963268*9.81*s4 - 1.003125*Power(dth1,2)*s5 - 2.00625*Power(dth2,2)*s5 + 0.010131542084963268*9.81*s6 - 1.003125*Power(dth1,2)*s7 - 0.75234375*Power(dth1,2)*s8 - 0.0027774944033544155*9.81*s9 + tau3))/(1.8517060546875006 - 0.76144921875*Power(c12,2) - 0.45516796875000015*Power(c13,2) + 0.9103359375*c12*c13*c6 - 1.1068857421875002*Power(c6,2)) + ((-0.8275781250000002*c13 - 0.226875*c12*c13 + 0.55171875*c6 + 0.8275781249999999*c12*c6)*(0.0027774944033544155*c11*9.81 - 0.5499438918641743*c14*9.81 - 0.0027774944033544155*c15*9.81 - 0.20625*Power(dth1,2)*s10 - 0.4125*Power(dth2,2)*s10 - 0.825*dth2*dth3*s10 - 0.4125*Power(dth3,2)*s10 - 0.275*Power(dth1,2)*s11 - 0.55*Power(dth2,2)*s11 + 0.0027774944033544155*9.81*s12 - 0.275*Power(dth1,2)*s13 - 0.20625*Power(dth1,2)*s14 - 0.1375*Power(dth1,2)*s15 - 0.0027774944033544155*9.81*s9 + tau4))/(1.8517060546875006 - 0.76144921875*Power(c12,2) - 0.45516796875000015*Power(c13,2) + 0.9103359375*c12*c13*c6 - 1.1068857421875002*Power(c6,2));
  f[6] = (-1.*(0.4137890625000001 - 0.17015624999999998*Power(c12,2) - 0.226875*c12*c13 + 0.55171875*c6)*(0.022598704463656385*c1*9.81 + 0.0027774944033544155*c11*9.81 - 0.5499438918641743*c14*9.81 - 0.0027774944033544155*c15*9.81 - 4.474543483803964*c2*9.81 - 0.02259870446365638*c4*9.81 + 0.010131542084963268*c5*9.81 - 2.0060453328227266*c7*9.81 - 0.010131542084963268*c8*9.81 - 0.02259870446365638*9.81*s1 + 0.825*dth2*dth4*s10 + 0.825*dth3*dth4*s10 + 0.4125*Power(dth4,2)*s10 + 1.1*dth2*dth3*s11 + 0.55*Power(dth3,2)*s11 + 1.1*dth2*dth4*s11 + 1.1*dth3*dth4*s11 + 0.55*Power(dth4,2)*s11 + 0.0027774944033544155*9.81*s12 - 0.55*Power(dth1,2)*s13 - 0.4125*Power(dth1,2)*s14 - 0.1375*Power(dth1,2)*s15 - 2.2375*Power(dth1,2)*s2 + 0.022598704463656385*9.81*s3 - 0.010131542084963268*9.81*s4 + 4.0125*dth2*dth3*s5 + 2.00625*Power(dth3,2)*s5 + 0.010131542084963268*9.81*s6 - 2.00625*Power(dth1,2)*s7 - 0.75234375*Power(dth1,2)*s8 - 0.0027774944033544155*9.81*s9 + tau2))/(1.8517060546875006 - 0.76144921875*Power(c12,2) - 0.45516796875000015*Power(c13,2) + 0.9103359375*c12*c13*c6 - 1.1068857421875002*Power(c6,2)) + ((1.6444140625 - 0.17015624999999998*Power(c12,2) - 0.45375*c12*c13 - 0.30250000000000005*Power(c13,2) + 1.1034375*c6)*(0.0027774944033544155*c11*9.81 - 0.5499438918641743*c14*9.81 - 0.0027774944033544155*c15*9.81 + 0.010131542084963268*c5*9.81 - 2.0060453328227266*c7*9.81 - 0.010131542084963268*c8*9.81 + 0.825*dth2*dth4*s10 + 0.825*dth3*dth4*s10 + 0.4125*Power(dth4,2)*s10 - 0.275*Power(dth1,2)*s11 - 0.55*Power(dth2,2)*s11 + 0.0027774944033544155*9.81*s12 - 0.275*Power(dth1,2)*s13 - 0.4125*Power(dth1,2)*s14 - 0.1375*Power(dth1,2)*s15 - 0.010131542084963268*9.81*s4 - 1.003125*Power(dth1,2)*s5 - 2.00625*Power(dth2,2)*s5 + 0.010131542084963268*9.81*s6 - 1.003125*Power(dth1,2)*s7 - 0.75234375*Power(dth1,2)*s8 - 0.0027774944033544155*9.81*s9 + tau3))/(1.8517060546875006 - 0.76144921875*Power(c12,2) - 0.45516796875000015*Power(c13,2) + 0.9103359375*c12*c13*c6 - 1.1068857421875002*Power(c6,2)) - (1.*(1.230625 + 1.8459375000000002*c12 - 0.8275781250000003*c13 - 0.22687500000000005*c12*c13 - 0.30250000000000005*Power(c13,2) + 0.55171875*c6 + 0.8275781249999999*c12*c6 - 1.1034375*c13*c6)*(0.0027774944033544155*c11*9.81 - 0.5499438918641743*c14*9.81 - 0.0027774944033544155*c15*9.81 - 0.20625*Power(dth1,2)*s10 - 0.4125*Power(dth2,2)*s10 - 0.825*dth2*dth3*s10 - 0.4125*Power(dth3,2)*s10 - 0.275*Power(dth1,2)*s11 - 0.55*Power(dth2,2)*s11 + 0.0027774944033544155*9.81*s12 - 0.275*Power(dth1,2)*s13 - 0.20625*Power(dth1,2)*s14 - 0.1375*Power(dth1,2)*s15 - 0.0027774944033544155*9.81*s9 + tau4))/(1.8517060546875006 - 0.76144921875*Power(c12,2) - 0.45516796875000015*Power(c13,2) + 0.9103359375*c12*c13*c6 - 1.1068857421875002*Power(c6,2));
  f[7] = ((-0.8275781250000002*c13 - 0.226875*c12*c13 + 0.55171875*c6 + 0.8275781249999999*c12*c6)*(0.022598704463656385*c1*9.81 + 0.0027774944033544155*c11*9.81 - 0.5499438918641743*c14*9.81 - 0.0027774944033544155*c15*9.81 - 4.474543483803964*c2*9.81 - 0.02259870446365638*c4*9.81 + 0.010131542084963268*c5*9.81 - 2.0060453328227266*c7*9.81 - 0.010131542084963268*c8*9.81 - 0.02259870446365638*9.81*s1 + 0.825*dth2*dth4*s10 + 0.825*dth3*dth4*s10 + 0.4125*Power(dth4,2)*s10 + 1.1*dth2*dth3*s11 + 0.55*Power(dth3,2)*s11 + 1.1*dth2*dth4*s11 + 1.1*dth3*dth4*s11 + 0.55*Power(dth4,2)*s11 + 0.0027774944033544155*9.81*s12 - 0.55*Power(dth1,2)*s13 - 0.4125*Power(dth1,2)*s14 - 0.1375*Power(dth1,2)*s15 - 2.2375*Power(dth1,2)*s2 + 0.022598704463656385*9.81*s3 - 0.010131542084963268*9.81*s4 + 4.0125*dth2*dth3*s5 + 2.00625*Power(dth3,2)*s5 + 0.010131542084963268*9.81*s6 - 2.00625*Power(dth1,2)*s7 - 0.75234375*Power(dth1,2)*s8 - 0.0027774944033544155*9.81*s9 + tau2))/(1.8517060546875006 - 0.76144921875*Power(c12,2) - 0.45516796875000015*Power(c13,2) + 0.9103359375*c12*c13*c6 - 1.1068857421875002*Power(c6,2)) - (1.*(1.230625 + 1.8459375000000002*c12 - 0.8275781250000003*c13 - 0.22687500000000005*c12*c13 - 0.30250000000000005*Power(c13,2) + 0.55171875*c6 + 0.8275781249999999*c12*c6 - 1.1034375*c13*c6)*(0.0027774944033544155*c11*9.81 - 0.5499438918641743*c14*9.81 - 0.0027774944033544155*c15*9.81 + 0.010131542084963268*c5*9.81 - 2.0060453328227266*c7*9.81 - 0.010131542084963268*c8*9.81 + 0.825*dth2*dth4*s10 + 0.825*dth3*dth4*s10 + 0.4125*Power(dth4,2)*s10 - 0.275*Power(dth1,2)*s11 - 0.55*Power(dth2,2)*s11 + 0.0027774944033544155*9.81*s12 - 0.275*Power(dth1,2)*s13 - 0.4125*Power(dth1,2)*s14 - 0.1375*Power(dth1,2)*s15 - 0.010131542084963268*9.81*s4 - 1.003125*Power(dth1,2)*s5 - 2.00625*Power(dth2,2)*s5 + 0.010131542084963268*9.81*s6 - 1.003125*Power(dth1,2)*s7 - 0.75234375*Power(dth1,2)*s8 - 0.0027774944033544155*9.81*s9 + tau3))/(1.8517060546875006 - 0.76144921875*Power(c12,2) - 0.45516796875000015*Power(c13,2) + 0.9103359375*c12*c13*c6 - 1.1068857421875002*Power(c6,2)) + ((7.964101562500002 + 3.6918750000000005*c12 - 0.30250000000000005*Power(c13,2) - 2.206875*c13*c6 - 4.0250390625*Power(c6,2))*(0.0027774944033544155*c11*9.81 - 0.5499438918641743*c14*9.81 - 0.0027774944033544155*c15*9.81 - 0.20625*Power(dth1,2)*s10 - 0.4125*Power(dth2,2)*s10 - 0.825*dth2*dth3*s10 - 0.4125*Power(dth3,2)*s10 - 0.275*Power(dth1,2)*s11 - 0.55*Power(dth2,2)*s11 + 0.0027774944033544155*9.81*s12 - 0.275*Power(dth1,2)*s13 - 0.20625*Power(dth1,2)*s14 - 0.1375*Power(dth1,2)*s15 - 0.0027774944033544155*9.81*s9 + tau4))/(1.8517060546875006 - 0.76144921875*Power(c12,2) - 0.45516796875000015*Power(c13,2) + 0.9103359375*c12*c13*c6 - 1.1068857421875002*Power(c6,2));

  return(1);
}

/*****************************************************************************/

int model_actual_4link_pointmass(double *m, double *v, double *g, 
				 double *th, double *dth)
{
  double th1;
  double th2;
  double th3;
  double th4;
  double dth1;
  double dth2;
  double dth3;
  double dth4;
  double c1;
  double c2;
  double c3;
  double c4;
  double c5;
  double c6;
  double c7;
  double c8;
  double c9;
  double c10;
  double c11;
  double c12;
  double c13;
  double c14;
  double c15;
  double c16;
  double c17;
  double c18;
  double s1;
  double s2;
  double s3;
  double s4;
  double s5;
  double s6;
  double s7;
  double s8;
  double s9;
  double s10;
  double s11;
  double s12;
  double s13;
  double s14;
  double s15;
  th1 = th[0];
  th2 = th[1];
  th3 = th[2];
  th4 = th[3];
  dth1 = dth[0];
  dth2 = dth[1];
  dth3 = dth[2];
  dth4 = dth[3];
  c1 = Cos(th1 - 1.*th2);
  c2 = Cos(th2);
  c3 = Cos(2.*th2);
  c4 = Cos(th1 + th2);
  c5 = Cos(th1 - 1.*th2 - 1.*th3);
  c6 = Cos(th3);
  c7 = Cos(th2 + th3);
  c8 = Cos(th1 + th2 + th3);
  c9 = Cos(2.*th2 + th3);
  c10 = Cos(2.*th2 + 2.*th3);
  c11 = Cos(th1 - 1.*th2 - 1.*th3 - 1.*th4);
  c12 = Cos(th4);
  c13 = Cos(th3 + th4);
  c14 = Cos(th2 + th3 + th4);
  c15 = Cos(th1 + th2 + th3 + th4);
  c16 = Cos(2.*th2 + th3 + th4);
  c17 = Cos(2.*th2 + 2.*th3 + th4);
  c18 = Cos(2.*th2 + 2.*th3 + 2.*th4);
  s1 = Sin(th1 - 1.*th2);
  s2 = Sin(2.*th2);
  s3 = Sin(th1 + th2);
  s4 = Sin(th1 - 1.*th2 - 1.*th3);
  s5 = Sin(th3);
  s6 = Sin(th1 + th2 + th3);
  s7 = Sin(2.*th2 + th3);
  s8 = Sin(2.*th2 + 2.*th3);
  s9 = Sin(th1 - 1.*th2 - 1.*th3 - 1.*th4);
  s10 = Sin(th4);
  s11 = Sin(th3 + th4);
  s12 = Sin(th1 + th2 + th3 + th4);
  s13 = Sin(2.*th2 + th3 + th4);
  s14 = Sin(2.*th2 + 2.*th3 + th4);
  s15 = Sin(2.*th2 + 2.*th3 + 2.*th4);
  if(m != NULL) {
   m[0] = 0.00005100479445067836*(61314.701562500006 + 14750.4515625*c10 + 8087.475*c12 + 10783.300000000001*c13 + 10783.300000000001*c16 + 8087.475*c17 + 2695.8250000000003*c18 + 43868.425*c3 + 39334.537500000006*c6 + 39334.537500000006*c9);
   m[1] = 0;
   m[2] = 0;
   m[3] = 0;
   m[4] = 0;
   m[5] = 0.00005100479445067836*(122629.40312500001 + 16174.95*c12 + 21566.600000000002*c13 + 78669.07500000001*c6);
   m[6] = 0.00005100479445067836*(34892.553125000006 + 16174.95*c12 + 10783.300000000001*c13 + 39334.537500000006*c6);
   m[7] = 0.00005100479445067836*(5391.650000000001 + 8087.475*c12 + 10783.300000000001*c13);
   m[8] = 0;
   m[9] = 0.00005100479445067836*(34892.553125000006 + 16174.95*c12 + 10783.300000000001*c13 + 39334.537500000006*c6);
   m[10] = 0.00005100479445067836*(34892.553125000006 + 16174.95*c12);
   m[11] = 0.00005100479445067836*(5391.650000000001 + 8087.475*c12);
   m[12] = 0;
   m[13] = 0.00005100479445067836*(5391.650000000001 + 8087.475*c12 + 10783.300000000001*c13);
   m[14] = 0.00005100479445067836*(5391.650000000001 + 8087.475*c12);
   m[15] = 0.275;
  }
  if(v != NULL) {
   v[0] = dth1*(dth4*(-0.4125*s10 - 0.55*s11 - 0.55*s13 - 0.4125*s14 - 0.275*s15) + dth2*(-1.1*s13 - 0.825*s14 - 0.275*s15 - 4.475*s2 - 4.0125*s7 - 1.5046875*s8) + dth3*(-0.55*s11 - 0.55*s13 - 0.825*s14 - 0.275*s15 - 2.00625*s5 - 2.00625*s7 - 1.5046875*s8));
   v[1] = -0.825*dth2*dth4*s10 - 0.825*dth3*dth4*s10 - 0.4125*Power(dth4,2)*s10 - 1.1*dth2*dth3*s11 - 0.55*Power(dth3,2)*s11 - 1.1*dth2*dth4*s11 - 1.1*dth3*dth4*s11 - 0.55*Power(dth4,2)*s11 + 0.55*Power(dth1,2)*s13 + 0.4125*Power(dth1,2)*s14 + 0.1375*Power(dth1,2)*s15 + 2.2375*Power(dth1,2)*s2 - 4.0125*dth2*dth3*s5 - 2.00625*Power(dth3,2)*s5 + 2.00625*Power(dth1,2)*s7 + 0.75234375*Power(dth1,2)*s8;
   v[2] = -0.825*dth2*dth4*s10 - 0.825*dth3*dth4*s10 - 0.4125*Power(dth4,2)*s10 + 0.275*Power(dth1,2)*s11 + 0.55*Power(dth2,2)*s11 + 0.275*Power(dth1,2)*s13 + 0.4125*Power(dth1,2)*s14 + 0.1375*Power(dth1,2)*s15 + 1.003125*Power(dth1,2)*s5 + 2.00625*Power(dth2,2)*s5 + 1.003125*Power(dth1,2)*s7 + 0.75234375*Power(dth1,2)*s8;
   v[3] = 0.20625*Power(dth1,2)*s10 + 0.4125*Power(dth2,2)*s10 + 0.825*dth2*dth3*s10 + 0.4125*Power(dth3,2)*s10 + 0.275*Power(dth1,2)*s11 + 0.55*Power(dth2,2)*s11 + 0.275*Power(dth1,2)*s13 + 0.20625*Power(dth1,2)*s14 + 0.1375*Power(dth1,2)*s15;
  }
  if(g != NULL) {
   g[0] = 0.0005003570335611547*(443.070199714447*c1 + 54.45555527216668*c11 + 54.45555527216668*c15 + 443.070199714447*c4 + 198.63901411778983*c5 + 198.63901411778983*c8 - 443.0701997144471*s1 - 54.45555527216668*s12 - 443.0701997144471*s3 - 198.63901411778983*s4 - 198.63901411778983*s6 - 54.45555527216668*s9);
   g[1] = 0.0005003570335611547*(-443.0701997144471*c1 - 54.45555527216668*c11 + 10782.199943889002*c14 + 54.45555527216668*c15 + 87727.89954346052*c2 + 443.070199714447*c4 - 198.63901411778983*c5 + 39330.52479532238*c7 + 198.63901411778983*c8 + 443.070199714447*s1 - 54.45555527216668*s12 - 443.0701997144471*s3 + 198.63901411778983*s4 - 198.63901411778983*s6 + 54.45555527216668*s9);
   g[2] = 0.0005003570335611547*(-54.45555527216668*c11 + 10782.199943889002*c14 + 54.45555527216668*c15 - 198.63901411778983*c5 + 39330.52479532238*c7 + 198.63901411778983*c8 - 54.45555527216668*s12 + 198.63901411778983*s4 - 198.63901411778983*s6 + 54.45555527216668*s9);
   g[3] = 0.0005003570335611547*(-54.45555527216668*c11 + 10782.199943889002*c14 + 54.45555527216668*c15 - 54.45555527216668*s12 + 54.45555527216668*s9);
  }

  return(1);
}

/*****************************************************************************/
