/*****************************************************************************/
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* Last modified: 11/13/2001                                                 */
/*                                                                           */
/* File: "manip.h"                                                           */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/

/* Mathematica CForm defines */

#define Cos(a) cos(a)
#define Sin(a) sin(a)
#define Power(a,b) pow(a, ((double) b))

/*****************************************************************************/

#define JOINT_LIMIT 100.
#define NMAX 20
#define BUFFER_SIZE 1000000

#ifndef M_PI
#define M_PI 3.14159265358979323846264338328
#endif

/*****************************************************************************/

typedef struct database_struct {
  int (*robot)();
  int (*assumed_model)();
  int (*actual_model)();
  int n_links;
} Database;

/*****************************************************************************/

typedef struct via_point_struct {
  double time;
  double joints[NMAX];
} VIA_POINT;

/*****************************************************************************/

typedef struct manip_parameters_struct {
  int n_links;
  double KP[NMAX];
  double KV[NMAX];
  double KI[NMAX];
  int controller_type;
  int robot_type;
  double total_time;
  double frequency;
  double step_size;
  VIA_POINT start;
  VIA_POINT goal;
  int (*robot)();
  int (*control)();
  int (*model)();
} MANIP_PARAMETERS;

/*****************************************************************************/

typedef struct controller_struct {
  int n;
  double h;
  double *kp;
  double *kv;
  double *ki;
  double thd[NMAX];
  double dthd[NMAX];
  double ddthd[NMAX];
  double th[NMAX];
  double dth[NMAX];
  double err[NMAX];
  double derr[NMAX];
  double ierr[NMAX];
  int (*model)();
} CONTROLLER;

/*****************************************************************************/

int file_exist(char *file_name);
int file_no_exist(char *file_name, char *type);
int read_value_one(double *value, char *separators);
int read_all_line_values(double *values, char *separators);

/*****************************************************************************/

void get_manip_parameters(char *file_name, MANIP_PARAMETERS *p);
int process_manip_line(char *line, MANIP_PARAMETERS *p, int alert_flag);
int read_via_point(VIA_POINT *via, char *separators);
void display_manip_parameters(FILE *fp, MANIP_PARAMETERS *p);
void initialize_manip_parameters(MANIP_PARAMETERS *p);

/*****************************************************************************/

int cubic_path(double time, VIA_POINT *start, VIA_POINT *goal,
	       double *thd, double *dthd, double *ddthd, int n);
int runge_kutta_new_values(double *th_out, double *dth_out,
			   double *th, double *dth, 
			   double *k, int n, double h, double factor);
int runge_kutta(double *th, double *dth, double *tau, int n,
		int (*func)(), double h);

int simulate_controlled_trajectory(double *results, MANIP_PARAMETERS *p);
int display_results(FILE *fp, double *results, int n_res, int n);

/*****************************************************************************/
