/*****************************************************************************/
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* Last modified: 11/13/2001                                                 */
/*                                                                           */
/* File: "manip_main.c"                                                      */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#ifdef MacOS
#include <console.h>
#endif

#include "manip.h"
#include "control.h"
#include "robots.h"

/******************************* Globals *************************************/

Database Table[]= {
  {
    &robot_actual_2link_pointmass, 
    &model_assumed_2link_pointmass,
    &model_actual_2link_pointmass,
    2
  },
  {
    &robot_actual_2link_cylindrical,
    &model_assumed_2link_cylindrical,
    &model_actual_2link_cylindrical,
    2
  },
  {
    &robot_actual_4link_pointmass, 
    &model_assumed_4link_pointmass,
    &model_actual_4link_pointmass,
    4
  }
};


/*****************************************************************************/

double GlobalResults[BUFFER_SIZE];

/*****************************************************************************/

int main(int argc, char **argv)
{
  int n;
  FILE *fp;
  double *results;
  char output_file[256];
  MANIP_PARAMETERS p;

#ifdef MacOS
  argc = ccommand(&argv);
#endif

  results = &GlobalResults[0];

  initialize_manip_parameters(&p);
  get_manip_parameters(argv[1], &p);
  if(argc >= 3) {
    sprintf(&output_file[0], "%s", argv[2]);
  } else {
    sprintf(&output_file[0], "results.dat");
  }
  set_up_controller(&p);
  display_manip_parameters(stdout, &p);
  n = simulate_controlled_trajectory(results, &p);

  fprintf(stdout, "Writing output file %s.\n", &output_file[0]);
  fp = fopen(&output_file[0], "w");
  display_results(fp, results, n, p.n_links);
  fclose(fp);

  return(1);
}

/*****************************************************************************/
