(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    104143,       2820]*)
(*NotebookOutlinePosition[    104986,       2849]*)
(*  CellTagsIndexPosition[    104942,       2845]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["Complete manipulator definitions", "Title"],

Cell["\<\
This notebook supercedes all previous definition notebooks, \
including all previous as well as new definitions.\
\>", "Text"],

Cell[CellGroupData[{

Cell["General definitions (not manipulator specific)", "Section"],

Cell[CellGroupData[{

Cell["Generic definitions", "Subsection"],

Cell[CellGroupData[{

Cell["Turn off annoying warnings messages", "Subsubsection"],

Cell[BoxData[{
    \(\(Off[General::spell1];\)\), "\n", 
    \(\(Off[General::spell];\)\), "\n", 
    \(\(Off[ParametricPlot::ppcom];\)\), "\n", 
    \(\(Off[ParametricPlot3D::ppcom];\)\), "\[IndentingNewLine]", 
    \(\(Off[D::dvar];\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Load in necessary packages", "Subsubsection"],

Cell[BoxData[{
    \(\(<< LinearAlgebra`MatrixManipulation`;\)\), "\n", 
    \(\(<< Graphics`Shapes`;\)\), "\n", 
    \(\(<< Graphics`Colors`;\)\), "\[IndentingNewLine]", 
    \(\(<< Graphics`Graphics3D`;\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Nice view of matrices and vectors", "Subsubsection"],

Cell[BoxData[
    \(DisplayMatrixOrVector[m_] := \(m // MatrixForm\)\  // \ 
        TraditionalForm\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Conversion of table to rules list", "Subsubsection"],

Cell[BoxData[
    \(\(ConvertTableToRules[tbl_] := \(\((Rule\  @@ \ #)\) &\)\  /@ \ 
          tbl;\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Elementary rotation and transform definitions", "Subsection"],

Cell[CellGroupData[{

Cell["Definition of elementary rotation matrices", "Subsubsection"],

Cell[BoxData[{
    \(\(rX[
          a_] := {{1, 0, 0}, {0, \ Cos[a], \ \(-Sin[a]\)}, {0, Sin[a], \ 
            Cos[a]}};\)\), "\n", 
    \(\(rY[
          a_] := {{Cos[a], 0, Sin[a]}, {0, 1, 0}, \ {\(-Sin[a]\), 0, 
            Cos[a]}};\)\), "\n", 
    \(\(rZ[
          a_] := {{Cos[a], \ \(-Sin[a]\), 0}, {Sin[a], Cos[a], 0}, {0, 0, 
            1}};\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Convert a 3x3 rotation matrix and a 3x1 position vector to a 4x4 \
homogeneous operator\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(RAndPToT[r_, \ p_] := \[IndentingNewLine]Join[
          Transpose[Join[Transpose[r], \ {p}]], {{0, 0, 0, 1}}];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Convert a 3x3 rotation matrix to 4x4 homogeneous rotational \
operator\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(RToT[r_] := RAndPToT[r, \ {0, 0, 0}];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Elementary homogeneous rotational operators", "Subsubsection"],

Cell[BoxData[{
    \(\(RX[a_] := RToT[rX[a]];\)\), "\n", 
    \(\(RY[a_] := RToT[rY[a]];\)\), "\n", 
    \(\(RZ[a_] := RToT[rZ[a]];\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Angle-axis rotation [",
  Cell[BoxData[
      \(TraditionalForm\`\(R\_k\)(\[Theta])\)]],
  "]"
}], "Subsubsection"],

Cell[BoxData[
    \(\(RK[\[Theta]_, \ 
          k_] := \[IndentingNewLine]Module[{kx, \ ky, kz, \ v, c, 
            s}, \[IndentingNewLine]{kx, \ ky, \ kz}\  = \ 
            k/Sqrt[k . k]; {{kx^2\ v\  + \ c, kx\ ky\ v\  - \ kz\ s, \ 
                kx\ kz\ v\  + \ ky\ s}, {kx\ ky\ v\  + \ kz\ s, \ 
                ky^2\ v + \ c, \ 
                ky\ kz\ v\  - \ kx\ s}, {kx\ kz\ v\  - \ ky\ s, \ 
                ky\ kz\ v\  + \ kx\ s, \ 
                kz^2\ v\  + \ c}}\  /. \ {v -> \((1 - Cos[\[Theta]])\), \ 
              c -> Cos[\[Theta]], \ s -> Sin[\[Theta]]}];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Convert a 3x1 position vector to a 4x4 homogeneous translational \
operator\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(PToT[p_] := RAndPToT[IdentityMatrix[3], \ p];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Elementary homogeneous translational operators", "Subsubsection"],

Cell[BoxData[{
    \(\(DX[x_] := PToT[{x, 0, 0}];\)\), "\n", 
    \(\(DY[y_] := PToT[{0, y, 0}];\)\), "\n", 
    \(\(DZ[z_] := PToT[{0, 0, z}];\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Extract 3x3 rotation matrix from 4x4 homogeneous transform", 
  "Subsubsection"],

Cell[BoxData[
    \(\(RFromT[t_] := TakeMatrix[t, {1, 1}, {3, 3}];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Extract 3x1 position vector from 4x4 homogeneous transform", 
  "Subsubsection"],

Cell[BoxData[
    \(\(PFromT[t_] := 
        TakeMatrix[t, {1, 4}, {3, 4}]\  // \ Flatten;\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compute inverse of a 4x4 homogeneous transform", "Subsubsection"],

Cell[BoxData[
    \(\(InverseT[
          t_] := \[IndentingNewLine]Module[{r, \ 
            p}, \[IndentingNewLine]r\  = \ 
            RFromT[t]; \[IndentingNewLine]p\  = \ 
            PFromT[t]; \[IndentingNewLine]RAndPToT[
            Transpose[
              r], \ \(-Transpose[r] . p\)]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Cubic trajectory generation (continuous acceleration at via points)\
\
\>", "Subsection"],

Cell[CellGroupData[{

Cell["Nice output of vectors", "Subsubsection"],

Cell[BoxData[
    \(ViewSolution[
        v_] := \ \(v\  // 
          TableForm[#, TableAlignments \[Rule] {Left}] &\)\  // \ 
        TraditionalForm\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cubic polynomial definition", "Subsubsection"],

Cell[BoxData[
    \(\(Cubic[
          coeff_, {t_, \ ts_}] := \[IndentingNewLine]Module[{a0, a1, a2, 
            a3}, \[IndentingNewLine]{a0, a1, a2, a3}\  = \ 
            coeff; \[IndentingNewLine]a0\  + \ a1\ \((t - ts)\)\  + \ 
            a2\ \((t - ts)\)\^2 + 
            a3\ \((t - ts)\)\^3\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cubic short-cut functions", "Subsubsection"],

Cell[BoxData[
    \(\(cubicVia[a_, n_, teval_] := \[IndentingNewLine]Cubic[
          Table[a[n, i], {i, 0, 3}], {teval, \ 0}];\)\)], "Input"],

Cell[BoxData[
    \(\(DcubicVia[a_, \ n_, \ teval_] := 
        D[cubicVia[a, n, t], t]\  /. \ t \[Rule] teval;\)\)], "Input"],

Cell[BoxData[
    \(\(DDcubicVia[a_, \ n_, \ teval_] := 
        D[cubicVia[a, n, t], {t, 2}]\  /. \ t \[Rule] teval;\)\)], "Input"],

Cell[BoxData[
    \(\(eqFormat[lhs_, \ rhs_, flag_:  False] := \[IndentingNewLine]If[
          flag\  \[Equal] \ False, \[IndentingNewLine]lhs\  - \ 
            rhs, \[IndentingNewLine]lhs\  \[Equal] \ rhs];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Set up cubic solution equations given point path specification \
(with via points)\
\>", "Subsubsection"],

Cell["\<\
The functions below assume that the first point in the path \
description occurs at time = 0.\
\>", "Text"],

Cell[BoxData[
    \(\(CubicEquations[pts_, \ flag_:  False, \ d\[Theta]0_:  0, 
          d\[Theta]f_:  0] := \[IndentingNewLine]Module[{\ 
            tlist, \ \[Theta], \ tintervals, n, 
            eqs}, \[IndentingNewLine]{tlist, \[Theta]}\  = \ 
            Transpose[
              pts]; \[IndentingNewLine]tintervals\  = \ \(\((#[\([2]\)]\  - \ \
#[\([1]\)])\) &\)\  /@ \ 
              Partition[tlist, \ 2, \ 1]; \[IndentingNewLine]n\  = \ 
            Length[pts]; \  (*\ 
            number\ of\ points\  = \ 
              via\ points\  + \ 
                2\ end\ points\ *) \[IndentingNewLine]eqs\  = \ 
            Table[0, {4 \((n - 1)\)}]; \[IndentingNewLine] (*\ 
            end\ point\ constraints\ *) \[IndentingNewLine]eqs[\([1]\)]\  = \ 
            eqFormat[\[Theta][\([1]\)]\ , cubicVia[a, \ 1, \ 0], \ 
              flag]; \[IndentingNewLine]eqs[\([2]\)]\  = \ 
            eqFormat[d\[Theta]0\ , \ DcubicVia[a, \ 1, \ 0], \ 
              flag]; \[IndentingNewLine]eqs[\([4 \((n - 1)\) - 1]\)]\  = \ 
            eqFormat[Last[\[Theta]]\ , \ 
              cubicVia[a, n - 1, \ tintervals[\([n - 1]\)]], 
              flag]; \[IndentingNewLine]eqs[\([4 \((n - 1)\)]\)]\  = \ 
            eqFormat[d\[Theta]f\ , \ 
              DcubicVia[a, n - 1, \ tintervals[\([n - 1]\)]], 
              flag]; \[IndentingNewLine] (*\ position, \ 
            velocity\ and\ acceleration\ continuity\ constraints\ \
*) \[IndentingNewLine]Do[\[IndentingNewLine]eqs[\([4 \((i - 1)\) + 
                    3]\)]\  = \ 
              eqFormat[\[Theta][\([i + 1]\)]\ , 
                cubicVia[a, i, tintervals[\([i]\)]], \ 
                flag]; \ \[IndentingNewLine]eqs[\([4 \((i - 1)\) + 
                    4]\)]\  = \ 
              eqFormat[\[Theta][\([i + 1]\)]\ , cubicVia[a, \ i + 1, \ 0], \ 
                flag]; \[IndentingNewLine]eqs[\([4 \((i - 1)\) + 5]\)]\  = \ 
              eqFormat[DcubicVia[a, i, tintervals[\([i]\)]]\ , 
                DcubicVia[a, i + 1, 0], 
                flag]; \[IndentingNewLine]eqs[\([4 \((i - 1)\) + 6\ ]\)]\  = 
              eqFormat[DDcubicVia[a, i, tintervals[\([i]\)]]\ , 
                DDcubicVia[a, i + 1, 0], flag], \ {i, 1, 
              n - 2}\[IndentingNewLine]]; \[IndentingNewLine]eqs\
\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Set up cubic solution equations with subscripted coefficients (for \
display)\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(CubicEquationsDisplay[pts_, d\[Theta]0_:  0, d\[Theta]f_:  0] := 
        CubicEquations[pts, \ True, d\[Theta]0, \ d\[Theta]f]\  /. \ 
          a[i_, \ j_] \[Rule] Subscript[a, i*10 + j];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solve for coefficients of cubic trajectory with via points", \
"Subsubsection"],

Cell[BoxData[
    \(\(CubicCoefficients[pts_, \ d\[Theta]0_:  0, 
          d\[Theta]f_:  0] := \[IndentingNewLine]Module[{eqs, \ 
            aList}, \[IndentingNewLine]eqs\  = \ 
            CubicEquations[pts, \ True, \ d\[Theta]0, \ 
              d\[Theta]f]; \[IndentingNewLine]aVec\  = \ 
            Table[a[i, j], {i, 1, Length[pts] - 1}, {j, 0, 3}]\  // \ 
              Flatten; \[IndentingNewLine]Solve[eqs, \ aVec]\  // \ 
            Flatten\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Solve for coefficients of cubic trajectory with subscripted \
coefficients (for display)\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(CubicCoefficientsDisplay[pts_, \ d\[Theta]0_:  0, d\[Theta]f_:  0] := 
        CubicCoefficients[pts, d\[Theta]0, \ d\[Theta]f]\  /. \ 
          a[i_, \ j_] \[Rule] Subscript[a, i*10 + j];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Extract matrix representation of solution equations", "Subsubsection"],

Cell[BoxData[
    \(\(CubicCoefficientMatrices[pts_, d\[Theta]0_:  0, 
          d\[Theta]f_:  0] := \[IndentingNewLine]Module[{eqs, aMat, \ bMat, \ 
            aVec, mat}, \[IndentingNewLine]eqs\  = \ 
            CubicEquations[pts, \ False, d\[Theta]0, \ 
              d\[Theta]f]; \[IndentingNewLine]aVec\  = \ 
            Table[a[i, j], {i, 1, Length[pts] - 1}, {j, 0, 3}]\  // \ 
              Flatten; \[IndentingNewLine]aMat\  = \ \(Coefficient[#, 
                  aVec] &\)\  /@ \ 
              eqs; \[IndentingNewLine]bMat\  = \ \((eqs\  - \ 
                  aMat . aVec)\)\  // \ 
              Simplify; \[IndentingNewLine]mat = {aMat, 
              aVec\  /. a[i_, \ j_] \[Rule] Subscript[a, i*10 + j], 
              bMat}; \[IndentingNewLine]\((mat[\([1]\)]\  // \ 
                  TraditionalForm)\) . \((\ \((\({#} &\) /@ \ 
                      mat[\([2]\)])\)\  // \ 
                  TraditionalForm)\) \[Equal] \((\((\({#} &\) /@ \ 
                    mat[\([3]\)])\)\  // \ 
                TraditionalForm)\)\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cubic polynomial trajectory", "Subsubsection"],

Cell[BoxData[
    \(\(u[t_, {t0_, tf_}] := 
        If[t \[GreaterEqual] \ t0\  && \ t\  < \ tf, 1, 0];\)\)], "Input"],

Cell[BoxData[
    \(\(CubicPolyTrajectory[pts_, d\[Theta]0_:  0, 
          d\[Theta]f_:  0] := \[IndentingNewLine]Module[{tintervals, 
            coeff}, \[IndentingNewLine]tintervals\  = \ 
            Partition[\((pts\  // \ Transpose)\)[\([1]\)], 2, 
              1]; \[IndentingNewLine]coeff\  = \ 
            CubicCoefficients[pts, d\[Theta]0, \ 
              d\[Theta]f]; \[IndentingNewLine]\((Plus\  @@ \ 
                Table[Cubic[
                        Table[a[i, j], {j, 0, 3}], {t, 
                          tintervals[\([i, 1]\)]}] 
                      u[t, tintervals[\([i]\)]]\  /. \ coeff, {i, 1, 
                    Length[pts] - 1}])\) + 
            If[t \[GreaterEqual] \(Last[pts]\)[\([1]\)], 
              Evaluate[\(Last[pts]\)[\([2]\)]], 
              0]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cubic polynomial trajectories in multiple dimensions", "Subsubsection"],

Cell[BoxData[
    \(\(MultipleCubicPolyTrajectories[
          pts_] := \[IndentingNewLine]Module[{tvec, \ \[Theta]s, \ 
            pts2}, \[IndentingNewLine]tvec\  = \ 
            First\  /@ \ 
              pts; \[IndentingNewLine]pts2\  = \ \(Transpose[{tvec, #}] &\)\  \
/@ \ Drop[Transpose[pts], 1]; \[IndentingNewLine]CubicPolyTrajectory\  /@ \ 
            pts2\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simple cubic trajectory generation", "Subsection"],

Cell["\<\
The functions below assume that the first point in the cubic path \
description occurs at time = 0.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Set up  simple cubic solution equations", "Subsubsection"],

Cell[BoxData[
    \(\(SimpleCubicEquations[{t0_, \[Theta]0_, 
            d\[Theta]0_}, {tf_, \[Theta]f_, 
            d\[Theta]f_}] := \[IndentingNewLine]CubicEquations[{{t0, \
\[Theta]0}, {tf, \[Theta]f}}, True, d\[Theta]0, d\[Theta]f];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Set up cubic solution equations with subscripted coefficients (for \
display)\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(SimpleCubicEquationsDisplay[p0_, pf_] := 
        SimpleCubicEquations[p0, pf]\  /. \ 
          a[i_, \ j_] \[Rule] Subscript[a, j];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solve for simple cubic coefficients", "Subsubsection"],

Cell[BoxData[
    \(\(SimpleCubicCoefficients[{t0_, \[Theta]0_, 
            d\[Theta]0_}, {tf_, \[Theta]f_, 
            d\[Theta]f_}] := \[IndentingNewLine]CubicCoefficients[{{t0, \
\[Theta]0}, {tf, \[Theta]f}}, d\[Theta]0, d\[Theta]f];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Solve for coefficients of cubic trajectory with subscripted \
coefficients (for display)\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(SimpleCubicCoefficientsDisplay[p0_, pf_] := 
        SimpleCubicCoefficients[p0, pf]\  /. \ 
          a[i_, \ j_] \[Rule] Subscript[a, j];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simple cubic polynomial trajectory", "Subsubsection"],

Cell[BoxData[
    \(SimpleCubicPolyTrajectory[{t0_, \[Theta]0_, 
          d\[Theta]0_}, {tf_, \[Theta]f_, 
          d\[Theta]f_}] := \[IndentingNewLine]CubicPolyTrajectory[{{t0, \
\[Theta]0}, {tf, \[Theta]f}}, d\[Theta]0, d\[Theta]f]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Cubic trajectory generation (velocity heuristic at via \
points)\
\>", "Subsection"],

Cell[CellGroupData[{

Cell["Slope computation", "Subsubsection"],

Cell[BoxData[
    \(\(DetermineSlope[{{t0_, \[Theta]0_}, {tf_, \[Theta]f_}}] \
:= \[IndentingNewLine]\((\[Theta]f\  - \ \[Theta]0)\)/\((tf\  - \ 
              t0)\);\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compute velocity at via points", "Subsubsection"],

Cell[BoxData[
    \(\(VelocityAtViaPoint[{m1_, m2_}] := \[IndentingNewLine]If[
          Sign[m1]\  \[Equal] \ Sign[m2], m1 + m2/2, \ 0];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Generate velocities at via points", "Subsubsection"],

Cell[BoxData[
    \(\(VelocityAugmentedPoints[pts_, \ d\[Theta]0_:  0, 
          d\[Theta]f_:  0] := \[IndentingNewLine]Module[{dVias, \ 
            vel}, \[IndentingNewLine]dVias\  = \ 
            VelocityAtViaPoint\  /@ \ 
              Partition[DetermineSlope\  /@ \ Partition[pts, 2, 1], 2, 
                1]; \[IndentingNewLine]vel = {d\[Theta]0}~Join~dVias~
              Join~{d\[Theta]f}; \[IndentingNewLine]Transpose[
            Transpose[pts]~Join~{vel}]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cubic polynomial trajectory with velocity heuristic", "Subsubsection"],

Cell[BoxData[
    \(\(CubicPolyTrajectoryWithVelocityHeuristic[pts_, \ d\[Theta]0_:  0, 
          d\[Theta]f_:  0] := \[IndentingNewLine]Module[{pts2, traj1, 
            traj2}, \[IndentingNewLine]pts2\  = \ 
            Partition[VelocityAugmentedPoints[pts, d\[Theta]0, d\[Theta]f], 
              2, 1]; \[IndentingNewLine]traj1 = \
\(SimpleCubicPolyTrajectory[#[\([1]\)], #[\([2]\)]] &\)\  /@ \ 
              pts2; \[IndentingNewLine]Plus\  @@ \ \((First\  /@ \ 
                  traj1)\)\  + \ 
            Last[Last[traj1]]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cubic polynomial trajectories in multiple dimensions", "Subsubsection"],

Cell[BoxData[
    \(\(MultipleCubicPolyTrajectoriesWithVelocityHeuristic[
          pts_] := \[IndentingNewLine]Module[{tvec, \ \[Theta]s, \ 
            pts2}, \[IndentingNewLine]tvec\  = \ 
            First\  /@ \ 
              pts; \[IndentingNewLine]pts2\  = \ \(Transpose[{tvec, #}] &\)\  \
/@ \ Drop[Transpose[pts], 
                1]; \[IndentingNewLine]\
CubicPolyTrajectoryWithVelocityHeuristic\  /@ \ 
            pts2\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cubic trajectory functional conversion routines", "Subsection"],

Cell[CellGroupData[{

Cell["Convert point-specified trajectory to table", "Subsubsection"],

Cell[BoxData[
    \(\(MultiplePointsToTable[
          pts_, {tmin_, tmax_, dt_}] := \[IndentingNewLine]Module[{poly, \ 
            tTbl, trajTbl}, \[IndentingNewLine]poly\  = \ 
            MultipleCubicPolyTrajectories[pts]; \[IndentingNewLine]tTbl\  = \ 
            Table[t, {t, tmin, tmax, dt}]\  // \ 
              N; \[IndentingNewLine]trajTbl\  = \ \((\(\((poly\  /. \ 
                        t \[Rule] #)\) &\)\  /@ \ tTbl)\)\  // \ 
              Transpose; \[IndentingNewLine]\(Transpose[{tTbl, \ #}] &\)\  \
/@ \ trajTbl\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Convert table trajectory to interpolation function", "Subsubsection"],

Cell[BoxData[
    \(\(MultipleTableToInterpolationFunction[
          tbl_] := \[IndentingNewLine]\(Interpolation[#, 
              InterpolationOrder \[Rule] 3] &\)\  /@ tbl;\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Convert point-specified trajectory to interpolation function", \
"Subsubsection"],

Cell[BoxData[
    \(\(MultiplePointsToInterpolationFunction[
          pts_, {tmin_, tmax_, 
            dt_}] := \[IndentingNewLine]MultipleTableToInterpolationFunction[
          MultiplePointsToTable[pts, {tmin, tmax, dt}]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Convert polynomial trajectory to table", "Subsubsection"],

Cell[BoxData[
    \(\(MultiplePolyToTable[
          poly_, {tmin_, tmax_, dt_}] := \[IndentingNewLine]Module[{tTbl, 
            trajTbl}, \[IndentingNewLine]tTbl\  = \ 
            Table[t, {t, tmin, tmax, dt}]\  // \ 
              N; \[IndentingNewLine]trajTbl\  = \ \((\(\((poly\  /. \ 
                        t \[Rule] #)\) &\)\  /@ \ tTbl)\)\  // \ 
              Transpose; \[IndentingNewLine]\(Transpose[{tTbl, \ #}] &\)\  \
/@ \ trajTbl\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Convert polynomial trajectory to interpolation function", \
"Subsubsection"],

Cell[BoxData[
    \(\(MultiplePolyToInterpolationFunction[
          poly_, {tmin_, tmax_, 
            dt_}] := \[IndentingNewLine]MultipleTableToInterpolationFunction[
          MultiplePolyToTable[poly, {tmin, tmax, dt}]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cubic trajectory plotting routines", "Subsection"],

Cell[CellGroupData[{

Cell["Plot polynomial trajectory with points", "Subsubsection"],

Cell[BoxData[
    \(\(PlotPolyTrajectory[pts_, {tmin_, tmax_}, color_:  Black, 
          fnc_:  CubicPolyTrajectory, d\[Theta]0_:  0, d\[Theta]f_:  0, \ 
          opts___] := \[IndentingNewLine]Module[{poly, g1, 
            g2}, \[IndentingNewLine]poly\  = \ 
            fnc[pts, d\[Theta]0, d\[Theta]f]; \[IndentingNewLine]g1 = 
            Plot[Evaluate[poly], {t, tmin, tmax}, \ 
              DisplayFunction \[Rule] Identity, 
              PlotStyle \[Rule] color]; \[IndentingNewLine]g2 = 
            Show[Graphics[{RGBColor[1, 0, 0], PointSize[ .02], 
                  Point /@ pts}], 
              DisplayFunction \[Rule] Identity]; \[IndentingNewLine]Show[g1, 
            g2, PlotRange \[Rule] All, Frame \[Rule] True, 
            Axes \[Rule] False, DisplayFunction \[Rule] $DisplayFunction, 
            opts]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plot derivative of polynomial trajectory", "Subsubsection"],

Cell[BoxData[
    \(\(PlotDPolyTrajectory[pts_, {tmin_, tmax_}, color_:  Black, 
          fnc_:  CubicPolyTrajectory, d\[Theta]0_:  0, d\[Theta]f_:  0, \ 
          opts___] := \[IndentingNewLine]Module[{poly, 
            g1}, \[IndentingNewLine]poly\  = \ 
            fnc[pts, d\[Theta]0, d\[Theta]f]; \[IndentingNewLine]g1 = 
            Plot[Evaluate[D[poly, t]], {t, tmin, tmax}, \ 
              DisplayFunction \[Rule] Identity, 
              PlotStyle \[Rule] color]; \[IndentingNewLine]Show[g1, 
            PlotRange \[Rule] All, Frame \[Rule] True, Axes \[Rule] False, 
            DisplayFunction \[Rule] $DisplayFunction, 
            opts]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plot 2nd derivative of polynomial trajectory", "Subsubsection"],

Cell[BoxData[
    \(\(PlotDDPolyTrajectory[pts_, {tmin_, tmax_}, color_:  Black, 
          fnc_:  CubicPolyTrajectory, d\[Theta]0_:  0, d\[Theta]f_:  0, 
          opts___] := \[IndentingNewLine]Module[{poly, 
            g1}, \[IndentingNewLine]poly\  = \ 
            fnc[pts, d\[Theta]0, d\[Theta]f]; \[IndentingNewLine]g1 = 
            Plot[Evaluate[D[poly, {t, 2}]], {t, tmin, tmax}, \ 
              DisplayFunction \[Rule] Identity, 
              PlotStyle \[Rule] color]; \[IndentingNewLine]Show[g1, 
            PlotRange \[Rule] All, Frame \[Rule] True, Axes \[Rule] False, 
            DisplayFunction \[Rule] $DisplayFunction, 
            opts]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Multi-dimensional path plotting", "Subsubsection"],

Cell[BoxData[
    \(\(PlotMultiplePointsTrajectories[pts_, {tmin_, tmax_}, colors_, 
          fnc_:  MultipleCubicPolyTrajectories, \ 
          opts___] := \[IndentingNewLine]Module[{poly, gpts, tlist, g1, 
            g2}, \[IndentingNewLine]poly\  = \ 
            fnc[pts]; \[IndentingNewLine]tlist\  = \ 
            First\  /@ \ pts; \[IndentingNewLine]gpts\  = \ 
            Partition[
              Flatten[\(Transpose[{tlist, #}] &\)\  /@ \ 
                  Drop[Transpose[pts], 1]], 2]; \[IndentingNewLine]g1 = 
            Plot[Evaluate[poly], {t, tmin, tmax}, \ 
              DisplayFunction \[Rule] Identity, 
              PlotStyle \[Rule] colors]; \[IndentingNewLine]g2 = 
            Show[Graphics[{Black, PointSize[ .02], Point /@ gpts}], 
              DisplayFunction \[Rule] Identity]; \[IndentingNewLine]Show[g1, 
            g2, PlotRange \[Rule] All, Frame \[Rule] True, 
            Axes \[Rule] False, DisplayFunction \[Rule] $DisplayFunction, 
            opts]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["General differential equations definitions", "Subsection"],

Cell[CellGroupData[{

Cell["Differential equations conversion routines", "Subsubsection"],

Cell[BoxData[
    \(\(IsolateSecondDerivatives[eqsAll_, 
          vars_] := \[IndentingNewLine]Module[{ddvars, \ ddcoeff, \ eqs, \ 
            fvec}, \[IndentingNewLine]ddvars\  = \ \(Dt[#, {t, 2}] &\)\  /@ \ 
              vars; \[IndentingNewLine]eqs\  = \ \(#[\([1]\)] &\)\  /@ \ 
              eqsAll; \[IndentingNewLine]fvec\  = \ \(#[\([2]\)] &\)\  /@ \ 
              eqsAll; \[IndentingNewLine]ddcoeff\  = 
            Outer[Coefficient[#1, \ #2] &, \ eqs, ddvars]\  // \ 
              Simplify; \[IndentingNewLine]remainder\  = \ \((eqs\  - \ 
                  ddcoeff . ddvars)\)\  // \ Simplify; \n\t\t\tInverse[
              ddcoeff] . \((fvec - 
                remainder)\)\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeGeneralDiffEq[eqsAll_, \ 
          vars_] := \[IndentingNewLine]Module[{vt, eqs, \ peq1, \ 
            eq1}, \[IndentingNewLine]vt\  = \ \(\((#[t])\) &\)\  /@ \ 
              vars; \[IndentingNewLine]eqs\  = 
            IsolateSecondDerivatives[eqsAll, \ 
              vars]; \[IndentingNewLine]peq1\  = \ 
            eqs\  /. \ \((\(\((# -> #[t])\) &\)\  /@ \ 
                  vars)\); \[IndentingNewLine]eq1\  = \ \(\((#[\([1]\)] == #[\
\([2]\)])\) &\)\  /@ \ 
              Transpose[{\(D[#, {t, 2}] &\)\  /@ \ vt, \ 
                  peq1}]; \[IndentingNewLine]eq1\  // \ 
            Chop\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeInitialCondDiffEq[vars_, \ xinit_, \ dxinit_, \ 
          tcurr_] := \[IndentingNewLine]Module[{vt, \ eq2, \ 
            eq3}, \[IndentingNewLine]vt\  = \ \(\((#[t])\) &\)\  /@ \ 
              vars; \[IndentingNewLine]eq2\  = \ \(\((#[\([1]\)]\  == \ \
#[\([2]\)])\) &\)\  /@ \ Transpose[{vt\  /. \ t \[Rule] tcurr, xinit}]; 
          eq3\  = \ \(\((#[\([1]\)]\  == \ #[\([2]\)])\) &\)\  /@ \ 
              Transpose[{D[vt, t]\  /. \ t \[Rule] tcurr, \ dxinit}]; 
          Flatten[{eq2, eq3}]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Static differential equation solver", "Subsubsection"],

Cell[BoxData[
    \(\(SimulateDiffEqStatic[deqA_, \ vars_, \ 
          h_, {xinit_, \ dxinit_, tmin_}, 
          opts___] := \[IndentingNewLine]Module[{deqB, \ deqs, \ tmax, \ 
            sol, \ xinitNew, \ dxinitNew}, \[IndentingNewLine]deqB\  = \ 
            MakeInitialCondDiffEq[vars, \ xinit, \ dxinit, \ 
              tmin]; \[IndentingNewLine]deqs\  = \ 
            deqA~Join~deqB; \[IndentingNewLine]tmax\  = \ 
            tmin + h; \[IndentingNewLine]sol\  = \ 
            Flatten[vars\  /. \ 
                NDSolve[deqs, \ vars, \ {t, tmin, tmax}, 
                  opts]]; \[IndentingNewLine]sol\[IndentingNewLine]];\)\)], \
"Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solve one period", "Subsubsection"],

Cell[BoxData[
    \(\(SimulateDiffEqOne[deqA_, \ vars_, \ h_, {xinit_, \ dxinit_, tmin_}, 
          opts___] := \[IndentingNewLine]Module[{deqB, \ deqs, \ tmax, \ 
            sol, \ xinitNew, \ dxinitNew}, \[IndentingNewLine]deqB\  = \ 
            MakeInitialCondDiffEq[vars, \ xinit, \ dxinit, \ 
              tmin]; \[IndentingNewLine]deqs\  = \ 
            deqA~Join~deqB; \[IndentingNewLine]tmax\  = \ 
            tmin + h; \[IndentingNewLine]sol\  = \ 
            Flatten[vars\  /. \ 
                NDSolve[deqs, \ vars, \ {t, tmin, tmax}, 
                  opts]]; \[IndentingNewLine]xinitNew\  = \ \(\((#[t]\  /. \ 
                    t \[Rule] \ tmax)\) &\)\  /@ \ 
              sol; \[IndentingNewLine]dxinitNew\  = \ \(\((D[#[t], \ 
                      t]\  /. \ t \[Rule] tmax)\) &\)\  /@ \ 
              sol; \[IndentingNewLine]{xinitNew, \ dxinitNew, 
            tmax}\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solve all", "Subsubsection"],

Cell[BoxData[
    \(\(SimulateDiffEqAll[eqs_, \ vars_, \ {xinit_, \ dxinit_}, \ tmax_, \ 
          step_, \ opts___] := \[IndentingNewLine]Module[{deqA, traj, \ 
            ttraj, \ xtraj}, \[IndentingNewLine]deqA\  = \ 
            MakeGeneralDiffEq[eqs, \ vars]; \[IndentingNewLine]traj\  = \ 
            NestList[
              SimulateDiffEqOne[deqA, \ vars, \ step, \ #, \ 
                  opts] &, \ {xinit, \ dxinit, 0}, \ 
              Ceiling[tmax/step]]; \[IndentingNewLine]ttraj\  = \ \(Transpose[
                traj]\)[\([3]\)]; \[IndentingNewLine]xtraj\  = \ 
            Transpose[\(Transpose[Drop[#, \(-1\)]] &\)\  /@ \ 
                traj]; \[IndentingNewLine]\(Interpolation[
                Transpose[{ttraj, #}]] &\)\  /@ \ 
            xtraj\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["General controller definitions", "Subsection"],

Cell[CellGroupData[{

Cell["Generate desired and error values for current time", "Subsubsection"],

Cell[BoxData[
    \(\(GenerateControllerInputs[
          trajd_, \ {x_, \ dx_, \ 
            tcurr_}] := \[IndentingNewLine]Module[{xd, \ dxd, \ ddxd, err, 
            derr}, \[IndentingNewLine]xd\  = \ \(#[tcurr] &\)\  /@ \ 
              trajd\ [\([1]\)]; \[IndentingNewLine]dxd\  = \ \(#[
                  tcurr] &\)\  /@ \ 
              trajd[\([2]\)]; \[IndentingNewLine]ddxd\  = \ \(#[
                  tcurr] &\)\  /@ \ 
              trajd[\([3]\)]; \[IndentingNewLine]err\  = \ 
            xd\  - \ x; \[IndentingNewLine]derr\  = \ 
            dxd\  - \ dx; \[IndentingNewLine]{xd, \ dxd, \ ddxd, \ err, \ 
            derr}\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simulate one controller step", "Subsubsection"],

Cell[BoxData[
    \( (*\ 
      This\ function\ uses\ the\ global\ variable\ IntegralError\ *) \)], \
"Input"],

Cell[BoxData[
    \(\(ControlOne[deqA_, \ vars_, \ fvars_, \ 
          h_, {x_, \ dx_, f_, counter_, tcurr_}, {controlLaw_, desired_, \ 
            params___}, 
          opts___] := \[IndentingNewLine]Module[{deqB, \ deqs, \ tmax, \ 
            sol, \ xNew, \ dxNew, \ fNew, \ frule, \ xd, dxd, ddxd, err, 
            derr}, \[IndentingNewLine]deqB\  = \ 
            MakeInitialCondDiffEq[vars, \ x, \ dx, \ 
              tcurr]; \[IndentingNewLine]frule\  = \ \(\((#[\([1]\)] \[Rule] \
#[\([2]\)])\) &\)\  /@ \ 
              Transpose[{fvars, \ f}]; \[IndentingNewLine]deqs\  = \ 
            deqA~Join~deqB\  /. \ frule; \[IndentingNewLine]tNew = \ 
            tcurr + h; \[IndentingNewLine]sol\  = \[IndentingNewLine]Flatten[
              vars\  /. \ NDSolve[deqs, \ vars, \ {t, tcurr, tNew}, 
                  opts]]; \[IndentingNewLine]xNew\  = \ \(\((#[t]\  /. \ 
                    t \[Rule] \ tNew)\) &\)\  /@ \ 
              sol; \[IndentingNewLine]dxNew\  = \ \(\((D[#[t], \ t]\  /. \ 
                    t \[Rule] tNew)\) &\)\  /@ \ sol; \[IndentingNewLine]{xd, 
              dxd, \ ddxd, \ err, \ derr}\  = \ 
            GenerateControllerInputs[
              desired, {xNew, \ dxNew, \ 
                tNew}]; \[IndentingNewLine]IntegralError\  += \ 
            err*h; \[IndentingNewLine]fNew\  = \ 
            controlLaw[x, \ dx, \ ddxd, \ err, \ derr, 
              params]; \[IndentingNewLine]{xNew, \ dxNew, fNew, \ 
            counter + 1, tNew}\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simulate entire controller", "Subsubsection"],

Cell[BoxData[
    \( (*\ 
      This\ function\ uses\ the\ global\ variable\ IntegralError\ *) \)], \
"Input"],

Cell[BoxData[
    \(\(ControlAll[eqs_, \ vars_, \ fvars_, {xinit_, \ dxinit_}, \ tmax_, \ 
          step_, \ {controlLaw_, \ desired_, \ params___}, 
          opts___] := \[IndentingNewLine]Module[{deqA, traj, \ ttraj, \ 
            xtraj, \ finit, xtraj2, \ ftraj, ftraj2, xd, dxd, \ ddxd, \ 
            err, \ derr, \ nSteps}, \[IndentingNewLine]nSteps\  = \ 
            Ceiling[tmax/step]; \[IndentingNewLine]deqA\  = \ 
            MakeGeneralDiffEq[eqs, \ vars]; \[IndentingNewLine]{xd, dxd, \ 
              ddxd, \ err, \ derr}\  = \ 
            GenerateControllerInputs[
              desired, {xinit, \ dxinit, \ 
                0}]; \[IndentingNewLine]IntegralError\  = \ 
            err; \[IndentingNewLine]finit\  = \ 
            controlLaw[xinit, dxinit, \ ddxd, \ err, \ derr, 
              params]; \[IndentingNewLine]traj\  = \ 
            NestList[
              ControlOne[deqA, \ vars, \ fvars, \ 
                  step, \ #, \ {controlLaw, \ desired, \ params}, 
                  opts] &, \ {xinit, \ dxinit, finit, 2, 0}, 
              nSteps]; \[IndentingNewLine]ttraj\  = \ 
            Last[Transpose[traj]]; \[IndentingNewLine]xtraj\  = \ 
            Transpose[\(Transpose[Drop[#, \(-3\)]] &\)\  /@ \ 
                traj]; \[IndentingNewLine]xtraj2\  = \ \(Interpolation[
                  Transpose[{ttraj, #}]] &\)\  /@ \ 
              xtraj; \[IndentingNewLine]ftraj\  = \ \ Transpose[\(#[\([3]\)] \
&\)\ \  /@ \ traj]; \[IndentingNewLine]ftraj2\  = \ \(Interpolation[
                  Transpose[{ttraj, \ #}]] &\)\  /@ \ \
\[IndentingNewLine]ftraj; \[IndentingNewLine]{xtraj2, \ 
            ftraj2}\ \[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["General plotting routines", "Subsection"],

Cell[CellGroupData[{

Cell["Short cuts", "Subsubsection"],

Cell[BoxData[{
    \(\(nS = DisplayFunction \[Rule] Identity;\)\), "\[IndentingNewLine]", 
    \(\(yS = 
        DisplayFunction \[Rule] $DisplayFunction;\)\), "\[IndentingNewLine]", \

    \(\(dsh\  = \ Dashing[{0.015, 0.015}];\)\), "\[IndentingNewLine]", 
    \(\(imgSize\  = \ ImageSize \[Rule] 288;\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Combine interpolation functions", "Subsubsection"],

Cell[BoxData[
    \(\(CombineInterpolationFunctions[traj1_, traj2_, {tmin_, tmax_}, 
          fnc_] := \[IndentingNewLine]Module[{f1, 
            f2}, \[IndentingNewLine]f1\  = \ \(#[t] &\)\  /@ \ 
              traj1; \[IndentingNewLine]f2\  = \ \(#[t] &\)\  /@ \ 
              traj2; \[IndentingNewLine]\(FunctionInterpolation[#, {t, tmin, 
                  tmax}] &\)\  /@ \ 
            fnc[f1, f2]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Plot a multiple dimensional trajectory (from a list of \
interpolation)\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(PlotMultipleTrajectory[traj_, {tmin_, tmax_}, plStyleList_, 
          opts___] := \[IndentingNewLine]Plot[
          Evaluate[\(#[t] &\)\  /@ \ traj], {t, \ tmin, tmax}, 
          PlotStyle \[Rule] plStyleList, Frame \[Rule] True, 
          PlotRange \[Rule] All, opts];\)\)], "Input"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Manipulator specific definitions", "Section"],

Cell[CellGroupData[{

Cell["Static forward kinematic definitions", "Subsection"],

Cell[TextData[{
  "In this notebook, DH parameters should be speficied as table, with one \
entry per link in the form ",
  Cell[BoxData[
      \(TraditionalForm\`{\[Alpha]\_\(i - 1\)\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`a\_\(i - 1\)\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`d\_i\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\[Theta]\_i\)]],
  ", type}, where \"type\" is 0 for revolute joints and 1 for prismatic \
joints."
}], "Text"],

Cell[CellGroupData[{

Cell["Degree to radian conversion (for DH parameter list)", "Subsubsection"],

Cell[BoxData[
    \(\(ConvertFromDegreesOne[{\[Alpha]_, \ a_, \ d_, \ \[Theta]_, \ 
            type___}] := {If[NumericQ[\[Alpha]], \ \[Alpha]\ Degree, \ 
              alpha], \ a, \ d, \ 
            If[NumericQ[\[Theta]], \ \[Theta]\ Degree, \ \[Theta]], \ 
            type}\  // \ FullSimplify;\)\)], "Input"],

Cell[BoxData[
    \(\(ConvertFromDegrees[l_] := 
        ConvertFromDegreesOne\  /@ \ l;\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Basic one link homogeneous transform (based on DH parameters)\
\>", 
  "Subsubsection"],

Cell[BoxData[
    \(\(TLink[{\[Alpha]_, \ a_, \ \ d_, \[Theta]_, type___}] := 
        RX[\[Alpha]] . DX[a] . RZ[\[Theta]] . DZ[d];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Compound transform for multiple links (for DH parameter list)\
\>", 
  "Subsubsection"],

Cell[BoxData[
    \(\(TAll[l_] := \ Dot\  @@ \((TLink\  /@ \ l)\);\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Homogeneous transform for robot  from frame {j} to frame {i}  ",
  "(",
  Cell[BoxData[
      \(TraditionalForm\`\[InvisiblePrefixScriptBase]\)]],
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^j\)\(\(\
\[InvisiblePrefixScriptBase]\_i\)\(\[InvisiblePrefixScriptBase]T\)\)\)]],
  ")"
}], "Subsubsection"],

Cell[BoxData[
    \(\(T[dh_, \ i_, \ 
          j_] := \[IndentingNewLine]Which[\[IndentingNewLine]i\  == \ j, 
          IdentityMatrix[4], \[IndentingNewLine]i\  < \ j, 
          TAll[Take[dh, \ {i + 1, \ j}]], \[IndentingNewLine]i\  > \ j, 
          InverseT[
            TAll[Take[dh, {j + 1, i}]]]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Rotation matrix for robot from frame {j} to frame {i} (",
  Cell[BoxData[
      \(TraditionalForm\`\[InvisiblePrefixScriptBase]\)]],
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^j\)\(\(\
\[InvisiblePrefixScriptBase]\_i\)\(\[InvisiblePrefixScriptBase]R\)\)\)]],
  ")"
}], "Subsubsection"],

Cell[BoxData[
    \(\(R[dh_, i_, j_] := RFromT[T[dh, \ i, \ j]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Position vector for robot from frame {j} to frame {i} (",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^i\)\(P\_\(j, 
            ORG\)\)\)]],
  ")"
}], "Subsubsection"],

Cell[BoxData[
    \(\(P[dh_, i_, j_] := PFromT[T[dh, \ i, \ j]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Velocity forward kinematics definitions", "Subsection"],

Cell[CellGroupData[{

Cell[TextData[{
  "Angular velocity of link {i} in frame {j} coordinates ",
  "(",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^i\)\(\[Omega]\_i\)\)]\
],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^j\)\(\[Omega]\_i\)\)]\
],
  ")"
}], "Subsubsection"],

Cell[BoxData[
    \(\(\[Omega][dh_, \ 0, \ 0] := \ {0, 0, 0};\)\)], "Input"],

Cell[BoxData[
    \(\(\[Omega][dh_, \ i_, \ i_] := \ 
        R[dh, \ i, \ i - 1] . \[Omega][dh, \ i - 1, i - 1] + 
          If[dh[\([i, 5]\)]\  == \ 0, \ 
            Dt[dh[\([i, 4]\)], t] {0, 0, 1}, {0, 0, 0}];\)\)], "Input"],

Cell[BoxData[
    \(\(\[Omega][dh_, \ j_, \ i_] := 
        R[dh, \ j, \ i] . \[Omega][dh, \ i, i];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Linear velocity of link {i} in frame {j} coordinates ",
  "(",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^i\)\(v\_i\)\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^j\)\(v\_i\)\)]],
  ")"
}], "Subsubsection"],

Cell[BoxData[
    \(\(v[dh_, \ 0, \ 0] := \ {0, 0, 0};\)\)], "Input"],

Cell[BoxData[
    \(v[dh_, \ i_, \ i_] := 
      R[dh, \ i, \ 
            i - 1] . \((v[dh, \ i - 1, \ i - 1]\  + \ 
              Cross[\[Omega][dh, \ i - 1, i - 1], \ \ P[dh, i - 1, i]])\) + 
        If[dh[\([i, 5]\)] == 1, 
          Dt[dh[\([i, 3]\)], t] {0, 0, 1}, {0, 0, 0}]\)], "Input"],

Cell[BoxData[
    \(\(v[dh_, \ j_, \ i_] := R[dh, \ j, \ i] . v[dh, \ i, i];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Jacobian (from recursive equations)", "Subsection"],

Cell[CellGroupData[{

Cell["\<\
Variable extraction and creation routines (from DH parameters)\
\>",
   "Subsubsection"],

Cell[BoxData[
    \(\(ExtractVariables[
          dh_] := \[IndentingNewLine]Module[{vars}, \[IndentingNewLine]vars\  \
= \ \(If[#[\([5]\)]\  == \ 0, \ #[\([4]\)], \ #[\([3]\)]] &\)\  /@ \ 
              dh; \(Take[vars, \ #] &\)\  /@ \ 
              Position[NumberQ\  /@ \ vars, \ False]\  // \ 
            Flatten\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(ExtractFirstDerivatives[
          dh_] := \[IndentingNewLine]Module[{vars}, \[IndentingNewLine]vars\  \
= \ ExtractVariables[dh]; \[IndentingNewLine]\(Dt[#, t] &\)\  /@ \ 
            vars\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeVelocityVariables[
          dh_] := \[IndentingNewLine]Module[{lvars}, \[IndentingNewLine]lvars\
\  = \ Length[ExtractVariables[dh]]; 
          Table[ToExpression["\<V\>" <> ToString[i]], {i, 1, 
              lvars}]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compute linear velocity Jacobian from recursive equations", 
  "Subsubsection"],

Cell[BoxData[
    \(\(LinearJacobian1[dh_, \ j_, \ 
          i_] := \[IndentingNewLine]Module[{vars, \ vel, \ 
            j1}, \[IndentingNewLine]vars\  = \ 
            ExtractVariables[dh]; \[IndentingNewLine]vel\  = \ 
            v[dh, \ j, \ i]; \[IndentingNewLine]j1\  = \ 
            Transpose[\(Coefficient[vel, \ Dt[#, \ t]] &\)\  /@ \ 
                vars]; \[IndentingNewLine]j1\[IndentingNewLine]];\)\)], \
"Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compute angular velocity Jacobian from recursive equations", 
  "Subsubsection"],

Cell[BoxData[
    \(\(AngularJacobian1[dh_, \ j_, \ 
          i_] := \[IndentingNewLine]Module[{vars, \ vel, \ 
            j1}, \[IndentingNewLine]vars\  = \ 
            ExtractVariables[dh]; \[IndentingNewLine]vel\  = \ \[Omega][dh, \ 
              j, \ i]; \[IndentingNewLine]j1\  = \ 
            Transpose[\(Coefficient[vel, \ Dt[#, \ t]] &\)\  /@ \ 
                vars]; \[IndentingNewLine]j1\[IndentingNewLine]];\)\)], \
"Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compute full Jacobian from recursive equations", "Subsubsection"],

Cell[BoxData[
    \(\(Jacobian1[dh_, \ j_, \ i_] := \[IndentingNewLine]Join[
          LinearJacobian1[dh, \ j, \ i], \ 
          AngularJacobian1[dh, \ j, \ i]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Jacobian (from forward kinematic homogeneous transform)", "Subsection"],

Cell[CellGroupData[{

Cell["\<\
Compute linear velocity Jacobian from forward kinematic homogeneous \
transform\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(LinearJacobian2[dh_, \ j_, \ 
          i_] := \[IndentingNewLine]Module[{vars, \ p, \ 
            j2}, \[IndentingNewLine]vars\  = \ 
            ExtractVariables[dh]; \[IndentingNewLine]p\  = \ 
            P[dh, \ j, \ i]; \[IndentingNewLine]j2\  = \ 
            Outer[D, \ p, \ 
              vars]; \[IndentingNewLine]j2\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Compute angular velocity Jacobian from forward kinematic \
homogeneous transform\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(AngularJacobian2[dh_, \ j_, \ 
          i_] := \[IndentingNewLine]Module[{vars, r, \ r2, \[CapitalOmega], \ 
            j2}, \[IndentingNewLine]vars\  = \ 
            ExtractVariables[dh]; \[IndentingNewLine]r\  = \ 
            R[dh, \ j, \ i]; \[IndentingNewLine]r2\  = \ 
            Dt[r, \ t] . 
              Transpose[
                r]; \[IndentingNewLine]\[CapitalOmega]\  = \ {r2[\([3, 
                  2]\)], \ r2[\([1, 3]\)], \ r2[\([2, 1]\)]}; 
          j2\  = \ Transpose[\(Coefficient[\[CapitalOmega], \ 
                    Dt[#, \ t]] &\)\  /@ \ 
                vars]; \[IndentingNewLine]j2\[IndentingNewLine]];\)\)], \
"Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Compute full Jacobian from forward kinematic homogeneous \
transform\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(Jacobian2[dh_, \ j_, \ i_] := 
        Join[LinearJacobian2[dh, \ j, \ i], \ 
          AngularJacobian2[dh, \ j, \ i]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Static force definitions", "Subsection"],

Cell[CellGroupData[{

Cell["Propagation of static forces in robot manipulators", "Subsubsection"],

Cell[BoxData[
    \(\(fs[dh_, i_, i_, \ fend_: {fx, fy, 0}] := \[IndentingNewLine]If[
          i\  == \ Length[dh], fend, 
          R[dh, i, i + 1] . fs[dh, \ i + 1, \ i + 1, \ fend]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Propagation of static moments in robot manipulators", "Subsubsection"],

Cell[BoxData[
    \(\(ns[dh_, i_, i_, \ fend_: {fx, fy, 0}] := \[IndentingNewLine]If[
          i\  == \ Length[dh], \ {0, 0, 0}, 
          R[dh, i, i + 1] . 
              ns[dh, i + 1, i + 1, \ fend] + \((P[dh, i, i + 1]~
                Cross~\((R[dh, i, i + 1] . 
                    fs[dh, i + 1, i + 1, \ fend])\))\)];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Torques required to support static forces and moments at \
end-effector\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(\[Tau]s[dh_, i_, fend_: {fx, fy, 0}] := \[IndentingNewLine]If[
          dh[\([i, 5]\)]\  == \ 
            0, \ \(ns[dh, i, i, \ fend]\)[\([3]\)], \(fs[dh, i, i, 
              fend]\)[\([3]\)]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Dynamics definitions", "Subsection"],

Cell[CellGroupData[{

Cell[TextData[{
  "Angular acceleration  of link {i} in frame {j} coordinates (",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^i\)\(\(\[Omega]\& . \
\)\_i\)\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^j\)\(\(\[Omega]\& . \
\)\_i\)\)]],
  ")"
}], "Subsubsection"],

Cell[BoxData[
    \(\(\(\[Alpha][dh_, \ 0, \ 0] := \ {0, 0, 0};\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(\(\[Alpha][dh_, \ i_, \ i_] := 
        R[dh, i, i - 1] . \[Alpha][dh, i - 1, 
              i - 1] + \((R[dh, i, i - 1] . \[Omega][dh, i - 1, i - 1])\)~
            Cross~If[dh[\([i, 5]\)]\  == \ 0, \ 
              Dt[dh[\([i, 4]\)], t] {0, 0, 1}, {0, 0, 0}]\  + 
          If[dh[\([i, 5]\)]\  == \ 0, \ 
            Dt[dh[\([i, 4]\)], \ {t, 2}] {0, 0, 1}, {0, 0, 0}];\)\)], "Input"],

Cell[BoxData[
    \(\(\[Alpha][dh_, \ j_, \ i_] := 
        R[dh, \ j, \ i] . \[Alpha][dh, \ i, i];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Linear acceleration  of link {i} in frame {j} coordinates ",
  "(",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^i\)\(\(v\& . \)\_i\)\
\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^j\)\(\(v\& . \)\_i\)\
\)]],
  ")"
}], "Subsubsection"],

Cell[BoxData[
    \(\(\(a[dh_, 0, 0] := {0, g, 0};\)\(\ \)\( (*\ 
      0\ gravity\ *) \)\)\)], "Input"],

Cell[BoxData[
    \(\(a[dh_, \ i_, \ i_] := 
        R[dh, i, \ 
              i - 1] . \((\[Alpha][dh, i - 1, i - 1]~Cross~
                  P[dh, i - 1, i] + \[Omega][dh, i - 1, i - 1]~
                  Cross~\((\[Omega][dh, i - 1, i - 1]~Cross~
                      P[dh, i - 1, i])\) + a[dh, \ i - 1, \ i - 1])\) + 
          2  \[Omega][dh, i, i]~Cross~
              If[dh[\([i, 5]\)]\  == \ 1, 
                Dt[dh[\([i, 3]\)], t] {0, 0, 1}, \ {0, 0, 0}]\  + \ 
          If[dh[\([i, 5]\)] == 1, 
            Dt[dh[\([i, 3]\)], {t, 2}] {0, 0, 1}, {0, 0, 0}];\)\)], "Input"],

Cell[BoxData[
    \(\(a[dh_, \ j_, \ i_] := R[dh, \ j, \ i] . a[dh, \ i, i];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Linear acceleration of center of mass of link {i} in frame {j} coordinates \
(",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^i\)\(\(v\& . \)\_\(C\
\_i\)\)\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^j\)\(\(v\& . \)\_\(C\
\_i\)\)\)]],
  ")"
}], "Subsubsection"],

Cell[BoxData[
    \(\(ac[{dh_, Pc_}, \ i_, 
          i_] := \[Alpha][dh, i, i]~Cross~
            Pc[\([i + 1]\)] + \[Omega][dh, i, i]~
            Cross~\((\[Omega][dh, i, i]~Cross~Pc[\([i + 1]\)])\) + 
          a[dh, i, i];\)\)], "Input"],

Cell[BoxData[
    \(\(ac[{dh_, \ Pc_}, \ j_, \ i_] := 
        R[dh, j, i] . ac[{dh, Pc}, i, i];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Force forward propagation (",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^i\)\(F\_i\)\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^i\)\(N\_i\)\)]],
  ")"
}], "Subsubsection"],

Cell[BoxData[
    \(\(F[{dh_, in_, m_, Pc_}, i_, i_] := 
        m[\([i + 1]\)]\ ac[{dh, Pc}, \ i, i];\)\)], "Input"],

Cell[BoxData[
    \(\(Nd[{dh_, in_, m_, Pc_}, i_, i_] := 
        in[\([i + 1]\)] . \[Alpha][dh, \ i, i]\  + \ \[Omega][dh, \ i, i]~
            Cross~\((in[\([i + 1]\)] . \[Omega][dh, i, i])\);\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Force backward propagation (",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^i\)\(f\_i\)\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^i\)\(n\_i\)\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\[InvisiblePrefixScriptBase]\(\[Tau]\_i\)\)]],
  ")"
}], "Subsubsection"],

Cell[BoxData[
    \(\(f[{dh_, in_, m_, Pc_}, i_, i_] := \[IndentingNewLine]If[
          i\  == \ Length[dh], \ {0, 0, 0}, 
          R[dh, i, i + 1] . f[{dh, \ in, \ m, \ Pc}, \ i + 1, \ i + 1]\  + \ 
            F[{dh, \ in, \ m, \ Pc}, \ i, 
              i]\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(n[{dh_, in_, m_, Pc_}, i_, i_] := \[IndentingNewLine]If[
          i\  == \ Length[dh], \ {0, 0, 0}, 
          Nd[{dh, \ in, \ m, \ Pc}, i, i] + 
            R[dh, i, i + 1] . 
              n[{dh, \ in, \ m, \ Pc}, i + 1, i + 1] + \((Pc[\([i + 1]\)]~
                Cross~F[{dh, \ in, \ m, \ Pc}, i, i])\) + \((P[dh, i, i + 1]~
                Cross~\((R[dh, i, i + 1] . 
                    f[{dh, \ in, \ m, \ Pc}, i + 1, 
                      i + 1])\))\)];\)\)], "Input"],

Cell[BoxData[
    \(\(\[Tau][{dh_, in_, m_, Pc_}, i_] := \[IndentingNewLine]If[
          dh[\([i, 5]\)]\  == \ 
            0, \ \(n[{dh, \ in, \ m, \ Pc}, i, 
              i]\)[\([3]\)], \[IndentingNewLine]\(f[{dh, \ in, \ m, \ Pc}, i, 
              i]\)[\([3]\)]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  " ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\  = \ 
        M(\[CapitalTheta]) + V(\[CapitalTheta], \[CapitalTheta]\& . ) + 
          G(\[CapitalTheta])\)]]
}], "Subsubsection"],

Cell[BoxData[
    \(\(\[Tau]All[{dh_, in_, m_, 
            Pc_}] := \[IndentingNewLine]Table[\[Tau][{dh, \ in, \ m, \ Pc}, 
              i]\  // \ TrigReduce, {i, 1, Length[dh] - 1}];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Separate M(\[CapitalTheta]) matrix, and V(\[CapitalTheta], ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalTheta]\& . \)]],
  ") and G(\[CapitalTheta]) vectors"
}], "Subsubsection"],

Cell[BoxData[
    \(\(\[Tau]MatrixFromModel[\[Tau]vec_, \ 
          vars_] := \[IndentingNewLine]Module[{MM, \ VV, \ 
            GV}, \[IndentingNewLine]MM\  = \ 
            Outer[Coefficient[#1, \ Dt[#2, \ {t, 2}]] &, \ \[Tau]vec, \ 
              vars]; \ GV\  = \ 
            Flatten[Outer[Coefficient[#1, \ #2] &, \ \[Tau]vec, \ {g}]*g]; 
          VV\  = \ Flatten[\[Tau]vec\  - \ 
                  MM . \((\ \(Dt[#, \ {t, 2}] &\)\  /@ \ vars)\)\  - \ 
                  GV\  // \ Simplify]; \[IndentingNewLine]{MM, \ VV, \ 
              GV}\  // \ Chop\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(\(\[Tau]Matrix[{dh_, in_, m_, 
            Pc_}] := \[IndentingNewLine]Module[{\[Tau]vec, 
            vars}, \[IndentingNewLine]vars\  = \ 
            ExtractVariables[
              dh]; \[IndentingNewLine]\[Tau]vec\  = \ \[Tau]All[{dh, \ in, \ 
                m, \ Pc}]; \
\[IndentingNewLine]\[Tau]MatrixFromModel[\[Tau]vec, \ 
            vars]\[IndentingNewLine]];\)\(\ \)\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  " ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalTheta]\&\[DoubleDot] = \
\(\(M(\[CapitalTheta])\)\^\(-1\)\)[\[Tau]\  - \ 
            V(\[CapitalTheta], \[CapitalTheta]\& . ) - 
            G(\[CapitalTheta])]\)]]
}], "Subsubsection"],

Cell[BoxData[
    \(\(ManipulatorDifferentialEquations[{dh_, in_, m_, Pc_}, \ \[Tau]t_] := 
        Module[{\[Tau]vec, \ mm, \ vars, \ ddvars}, 
          vars\  = \ ExtractVariables[dh]; 
          ddvars\  = \ \(Dt[#, {t, 2}] &\)\  /@ \ 
              vars; \[Tau]vec\  = \ \(\[Tau][{dh, \ in, \ m, \ 
                    Pc}, \ #] &\)\  /@ \ Table[i, {i, 1, Length[dh] - 1}]; 
          mm\  = Outer[Coefficient[#1, \ #2] &, \ \[Tau]vec, ddvars]\  // \ 
              Simplify; 
          remainder\  = \ \((\[Tau]vec\  - \ mm . ddvars)\)\  // \ Simplify; 
          Inverse[mm] . \((\[Tau]t - 
                remainder)\)\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(ManipulatorDiffEqsFromModel[\[Tau]vec_, 
          vars_] := \[IndentingNewLine]Module[{mm, vv, gv, 
            fvars}, \[IndentingNewLine]{mm, vv, 
              gv} = \[Tau]MatrixFromModel[\[Tau]vec, 
              vars]; \[IndentingNewLine]fvars\  = \ 
            Table[ToExpression["\<\[Tau]\>" <> ToString[i]], {i, 1, 
                Length[\[Tau]vec]}]; \[IndentingNewLine]Inverse[
                mm] . \((fvars\  - \ vv\  - \ gv)\)\  // \ 
            Chop\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  " Nice display for ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\  = 
        M(\[CapitalTheta]) + V(\[CapitalTheta], \[CapitalTheta]\& . ) + 
          G(\[CapitalTheta])\)]]
}], "Subsubsection"],

Cell[BoxData[
    \(\(DisplayManipulatorDynamicModel[model_, 
          i_] := \[IndentingNewLine]Module[{fvar}, \[IndentingNewLine]fvar\  \
= ToExpression["\<\[Tau]\>" <> 
                ToString[i]]; \[IndentingNewLine]fvar \[Equal] \ 
              model[\([i]\)]\  // \ 
            TraditionalForm\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Nice display of M(\[CapitalTheta]), V(\[CapitalTheta], ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalTheta]\& . \)]],
  ") and G(\[CapitalTheta])"
}], "Subsubsection"],

Cell[BoxData[
    \(\(DisplayManipulatorMassMatrix[{mm_, vv_, 
            gv_}] := \[IndentingNewLine]\((M[\[CapitalTheta]] \[Equal] \ 
              DisplayMatrixOrVector[mm])\)\  // \ 
          TraditionalForm;\)\)], "Input"],

Cell[BoxData[
    \(\(DisplayManipulatorVelocityVector[{mm_, vv_, 
            gv_}] := \[IndentingNewLine]\((V[\[CapitalTheta], \[CapitalTheta]\
\& . ] \[Equal] \ DisplayMatrixOrVector[vv])\)\  // \ 
          TraditionalForm;\)\)], "Input"],

Cell[BoxData[
    \(\(DisplayManipulatorGravityVector[{mm_, vv_, 
            gv_}] := \[IndentingNewLine]\((G[\[CapitalTheta]] \[Equal] \ 
              DisplayMatrixOrVector[gv])\)\  // \ 
          TraditionalForm;\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  " Nice display of differential equations ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalTheta]\&\[DoubleDot] = \
\(\(M(\[CapitalTheta])\)\^\(-1\)\)[\[Tau]\  - \ 
            V(\[CapitalTheta], \(\[CapitalTheta]\& . \)) - 
            G(\[CapitalTheta])]\)]]
}], "Subsubsection"],

Cell[BoxData[
    \(\(DisplayManipulatorDifferentialEquation[model_, dh_, 
          i_] := \[IndentingNewLine]Module[{vars}, \[IndentingNewLine]vars\  \
= \ ExtractVariables[
              dh]; \[IndentingNewLine]Dt[vars[\([i]\)], {t, 2}]\  \[Equal] \ 
              model[\([i]\)]\  // \ 
            TraditionalForm\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  " Nice display of inertia tensors (",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^C\_i\)\(I\_i\)\)]],
  ")"
}], "Subsubsection"],

Cell[BoxData[
    \(\(DisplayManipulatorInertiaTensor[in_, 
          i_] := \(("\<\>"^Subscript["\<C\>", i]\ Subscript["\<I\>", 
                  i]\  // \ TraditionalForm)\) \[Equal] \((in[\([i + 
                    1]\)]\  // \ TraditionalForm)\);\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Animation functions", "Subsection"],

Cell[BoxData[
    \(\(ConvertTrajToGraphics[
          l_] := \[IndentingNewLine]Join[{{Thickness[ .015], \ 
              RGBColor[0, 0, 1], \ 
              Line[l]}}, \({PointSize[ .02], \ RGBColor[1, 0, 0], \ 
                Point[#]} &\)\  /@ \ l];\)\)], "Input"],

Cell[BoxData[
    \(\(ExtractMaximalPlotRange[ranges_, \ 
          l_:  2] := \[IndentingNewLine]Module[{r2}, \[IndentingNewLine]If[
            l\  == \ 2, 
            r2\  = \ Transpose[
                Partition[Flatten[ranges], 4]]; \[IndentingNewLine]{{Min[
                  r2[\([1]\)]], \ Max[r2[\([2]\)]]}, \ {Min[r2[\([3]\)]], \ 
                Max[r2[\([4]\)]]}}, 
            r2\  = \ Transpose[
                Partition[Flatten[ranges], 6]]; {{Min[r2[\([1]\)]], \ 
                Max[r2[\([2]\)]]}, {Min[r2[\([3]\)]], \ 
                Max[r2[\([4]\)]]}, \ {Min[r2[\([5]\)]], \ 
                Max[r2[\([6]\)]]}}]\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(AnimateGraphicsList[traj_, \ 
          l_:  2] := \[IndentingNewLine]Module[{f, \ g, \ 
            plrng}, \[IndentingNewLine]If[l\  == \ 2, \ f\  = \ Graphics, \ 
            f = Graphics3D\ ]; 
          g\  = \ \(Show[f[#], \ DisplayFunction -> Identity, \ 
                  PlotRange -> All] &\)\  /@ \ 
              traj; \[IndentingNewLine]plrng\  = \ 
            ExtractMaximalPlotRange[\(FullOptions[#, \ PlotRange] &\)\  /@ \ 
                g, l]; \[IndentingNewLine]If[
            l\  \[Equal] \ 
              2, \[IndentingNewLine]\(Show[f[#], \ PlotRange -> plrng, 
                  Frame \[Rule] True, \ AspectRatio -> Automatic] &\)\  /@ \ 
              traj, \[IndentingNewLine]\(Show[f[#], \ PlotRange -> plrng, \ 
                  BoxRatios -> Automatic] &\)\  /@ \ 
              traj]\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(AnimateTrajectory[dh_, \ traj_, \ l_:  2, \ n_:  20, \ 
          tfin_:  False] := 
        Module[{nF, \ joints, \ p0, \ ptake, \ vars, \ rul, \ traj2, \ 
            tf}, \[IndentingNewLine]If[l\  == \ 2, \ p0\  = \ {0, 0}; \ 
            ptake\  = \ 2, \ p0\  = \ {0, 0, 0}; \ 
            ptake\  = \ 3]; \[IndentingNewLine]vars\  = \ 
            ExtractVariables[dh]; \[IndentingNewLine]nF\  = \ 
            Length[dh]; \[IndentingNewLine]joints\  = \ 
            Join[{p0}, 
              Table[Take[P[dh, \ 0, \ i]\  // \ Chop, \ {1, ptake}], {i, 1, \ 
                  nF}]]; \[IndentingNewLine]rul\  = \(\((#[\([1]\)]\  -> \ \
\(#[\([2]\)]\)[t])\) &\)\  /@ \ \(({vars, \ traj}\  // \ Transpose)\); 
          traj2\  = \ joints\  /. \ rul; \[IndentingNewLine]If[
            tfin\  == \ False, tf\  = \ traj[\([1, 1, 1, 2]\)], \ 
            tf\  = \ tfin, \ tf = tfin]; 
          AnimateGraphicsList[
            Table[ConvertTrajToGraphics[traj2], \ {t, \ 0, \ tf, \ tf/n}], 
            l]\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(ConvertTrajToGraphics2[l_, \ c1_, 
          c2_] := \[IndentingNewLine]Join[{{Thickness[ .015], \ c1, \ 
              Line[l]}}, \({PointSize[ .02], \ c2, \ Point[#]} &\)\  /@ \ 
            l];\)\)], "Input"],

Cell[BoxData[
    \(\(AnimateTwoTrajectories[dh_, \ traj1_, \ traj2_, l_:  2, \ n_:  20, \ 
          tfin_:  False] := 
        Module[{nF, \ joints, \ p0, \ ptake, \ vars, \ rul1, rul2, \ traj1b, 
            traj2b, \ tf}, \[IndentingNewLine]If[l\  == \ 2, \ 
            p0\  = \ {0, 0}; \ ptake\  = \ 2, \ p0\  = \ {0, 0, 0}; \ 
            ptake\  = \ 3]; \[IndentingNewLine]vars\  = \ 
            ExtractVariables[dh]; \[IndentingNewLine]nF\  = \ 
            Length[dh]; \[IndentingNewLine]joints\  = \ 
            Join[{p0}, 
              Table[Take[P[dh, \ 0, \ i]\  // \ Chop, \ {1, ptake}], {i, 1, \ 
                  nF}]]; \[IndentingNewLine]rul1\  = \(\((#[\([1]\)]\  -> \ \
\(#[\([2]\)]\)[t])\) &\)\  /@ \ \(({vars, \ traj1}\  // \ Transpose)\); 
          traj1b = \ 
            joints\  /. \ 
              rul1; \[IndentingNewLine]rul2\  = \(\((#[\([1]\)]\  -> \ \
\(#[\([2]\)]\)[t])\) &\)\  /@ \ \(({vars, \ traj2}\  // \ Transpose)\); 
          traj2b = \ joints\  /. \ rul2; \[IndentingNewLine]If[
            tfin\  == \ False, tf\  = \ traj1[\([1, 1, 1, 2]\)], \ 
            tf\  = \ tfin, \ tf = tfin]; 
          AnimateGraphicsList[
            Table[ConvertTrajToGraphics2[traj1b, Yellow, Green]~Join~
                ConvertTrajToGraphics2[traj2b, Blue, Red], \ {t, \ 0, \ tf, \ 
                tf/n}], l]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Velocity kinematic simulation definitions", "Section"],

Cell[CellGroupData[{

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " conversion routine to use numerical differential equations solver"
}], "Subsection"],

Cell[BoxData[
    \(\(MakeDiffEq[p_, \ 
          dh_] := \[IndentingNewLine]Module[{r, \ vars, d1, d2, 
            eq}, \[IndentingNewLine]vars\  = \ \ ExtractVariables[
              dh]; \[IndentingNewLine]r = \ \(\((# -> #[t])\) &\)\  /@ \ 
              vars; \[IndentingNewLine]d1\  = \ 
            D[vars\  /. \ r\ , t]; \[IndentingNewLine]d2\  = \ 
            p\  /. \ 
              r; \[IndentingNewLine]eqs\  = \(\((#[\([1]\)]\  == \ \
#[\([2]\)])\) &\)\  /@ \ \(({d1, d2}\  // \ 
                  Transpose)\)\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
General simulation of a specified Cartesian velocity \
trajectory\
\>", "Subsection"],

Cell[TextData[{
  "This function simulates a trajectory specified by the velocity \
relationship ",
  Cell[BoxData[
      \(TraditionalForm\`\[Theta]\& .  = \(J\^g\) x\& . \)]],
  ", where ",
  Cell[BoxData[
      \(TraditionalForm\`J\^g\)]],
  " (jg) specifies some user defined inverse relationship (matrix), and the \
other function arguments are: dh = list of DH parameters, traj = desired \
end-effector velocity trajectory (as a function of t), initial = a list of \
initial joint angles for \[Theta], and time = the simulation length (1, \
time)."
}], "Text"],

Cell[BoxData[
    \(\(SimulateVelocityTrajectory[jg_, \ dh_, \ traj_, \ initial_, \ 
          time_:  1] := \[IndentingNewLine]Module[{\ v\[Theta], \ vars, \ 
            eq1, \ eq2, \ eq, \ 
            sol}, \[IndentingNewLine]v\[Theta]\  = \ \ jg . 
              traj; \[IndentingNewLine]eq1\  = \ 
            MakeDiffEq[v\[Theta], dh]; \[IndentingNewLine]vars\  = \ 
            ExtractVariables[
              dh]; \[IndentingNewLine]eq2\  = \ \(\((\(#[\([1]\)]\)[
                      0] == #[\([2]\)])\) &\)\  /@ \ 
              Transpose[{vars, \ initial}]; \[IndentingNewLine]eq\  = \ 
            Join[eq1, \ eq2]; \[IndentingNewLine]sol\  = \ 
            NDSolve[eq, \ vars, \ {t, \ 0, \ time}]\  // \ 
              Flatten; \[IndentingNewLine]vars\  /. \ 
            sol\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Generalized Inverse Jacobian velocity trajectories", "Subsection"],

Cell[TextData[{
  "This function simulates a trajectory specified by the velocity \
relationship ",
  Cell[BoxData[
      \(TraditionalForm\`\[Theta]\& .  = \(J\^*\) x\& . \)]],
  ", where ",
  Cell[BoxData[
      \(TraditionalForm\`\(J\^*\)\)]],
  " is the pseudo-inverse of the Jacobian (inverse in the case of square \
matrices). The function arguments are: dh = list of DH parameters, pick = a \
list of length six of 0s and 1s, which specifies which rows of the Jacobian \
to use, traj = desired end-effetor velocity trajectory (as a function of t), \
initial = a list of initial joint angles for \[Theta], and time = the \
simulation length (1, time)."
}], "Text"],

Cell[BoxData[
    \(\(SimulateInverseJacobianVelocityTrajectory[dh_, \ pick_, \ traj_, \ 
          initial_, \ 
          time_:  1] := \[IndentingNewLine]Module[{\ nF, \ j, 
            jg}, \[IndentingNewLine]nF\  = \ 
            Length[dh]; \[IndentingNewLine]j = \ 
            Jacobian1[dh, \ 0, \ nF]; \[IndentingNewLine]j = 
            Flatten[\(Take[j, \ #] &\)\  /@ \ Position[pick, 1], 
              1]; \[IndentingNewLine]If[
            MatchQ[Dimensions[j], \ {tmp_, \ tmp_}] == True, 
            Print["\<Square Jacobian: Using Inverse[ ]\>"]; 
            jg\  = \ Inverse[j]\  // \ Chop, 
            Print["\<Nonsquare Jacobian: Using Pseudoinverse\>"]; 
            jg\  = \ Transpose[j] . Inverse[j . Transpose[j]]\  // \ Chop]; 
          SimulateVelocityTrajectory[jg, \ dh, \ traj, \ initial, \ 
            time]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Short-hand for above function", "Subsection"],

Cell[BoxData[
    \(\(SIJVT[a__] := 
        SimulateInverseJacobianVelocityTrajectory[a];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Jacobian transpose velocity trajectories", "Subsection"],

Cell[TextData[{
  "This function simulates a trajectory specified by the velocity \
relationship ",
  Cell[BoxData[
      \(TraditionalForm\`\[Theta]\& .  = \(J\^T\) x\& . \)]],
  ", where ",
  Cell[BoxData[
      \(TraditionalForm\`J\^T\)]],
  " is the transpose of the Jacobian. The function arguments are: dh = list \
of DH parameters, pick = a list of length six of 0s and 1s, which specifies \
which rows of the Jacobian to use, traj = desired end-effetor velocity \
trajectory (as a function of t), initial = a list of initial joint angles for \
\[Theta], and time = the simulation length (0, time)."
}], "Text"],

Cell[BoxData[
    \(\(SimulateTransposeJacobianVelocityTrajectory[dh_, \ pick_, \ traj_, \ 
          initial_, \ 
          time_:  1] := \[IndentingNewLine]Module[{\ nF, \ j, 
            jt}, \[IndentingNewLine]nF\  = \ 
            Length[dh]; \[IndentingNewLine]j = \ 
            Jacobian1[dh, \ 0, \ nF]; \[IndentingNewLine]j = 
            Flatten[\(Take[j, \ #] &\)\  /@ \ Position[pick, 1], 
              1]; \[IndentingNewLine]jt\  = \ 
            Transpose[j]; \[IndentingNewLine]SimulateVelocityTrajectory[jt, \ 
            dh, \ traj, \ initial, \ time]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Short-hand for above function", "Subsection"],

Cell[BoxData[
    \(\(STJVT[a__] := 
        SimulateTransposeJacobianVelocityTrajectory[a];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Inverse kinematics through Jacobian transpose", "Subsection"],

Cell[BoxData[
    \(\(IKTJOneStep[j_, \ p_, \ dh_, \ theta_, xf_, \ 
          dt_:  0.02\ ] := \[IndentingNewLine]Module[{\ rul, nF, \ jn, \ pn, 
            xc}, rul\  = \(\((#[\([1]\)]\  -> \ #[\([2]\)])\) &\)\  /@ \ \
\(({ExtractVariables[dh], \ theta}\  // \ Transpose)\); 
          jn = \ j /. \ rul\  // \ N; \[IndentingNewLine]xc\  = \ 
            p /. \ rul\  // \ N; \[IndentingNewLine]theta\  + \ 
            dt*Transpose[
                  jn] . \((xf - xc)\)\[IndentingNewLine]];\)\)], "Input"],

Cell["\<\
These functions provide an iterative solution to the inverse \
kinematics (position only, for now). \
\>", "Text"],

Cell[BoxData[
    \(\(InverseKinematicsTransposeJacobian[dh_, \ pick_, \ initial_, \ xf_, \ 
          dt_] := \[IndentingNewLine]Module[{nF, j, p, \ 
            res}, \[IndentingNewLine]nF\  = \ 
            Length[dh]; \[IndentingNewLine]j = \ 
            Jacobian1[dh, \ 0, \ nF]; \[IndentingNewLine]j = 
            Flatten[\(Take[j, \ #] &\)\  /@ \ Position[pick, 1], 
              1]; \[IndentingNewLine]p\  = \ 
            P[dh, \ 0, \ nF]; \[IndentingNewLine]p\  = 
            Flatten[\(Take[p, \ #] &\)\  /@ \ Position[pick, 1], 
              1]; \[IndentingNewLine]res\  = \ 
            FixedPointList[IKTJOneStep[j, p, dh, \ #, \ xf, dt] &, \ initial, 
              SameTest \[Rule] \((Sqrt[\((#1 - #2)\) . \((#1 - #2)\)] < 
                      0.0001 &)\)]; \[IndentingNewLine]\(ListInterpolation[#, \
\ {{0, \ dt*\((Length[res] - 1)\)}}] &\)\  /@ \ 
            Transpose[res]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Short-hand for above function", "Subsection"],

Cell[BoxData[
    \(\(IKTJ[a__] := InverseKinematicsTransposeJacobian[a];\)\)], "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Dynamics simulation definitions", "Section"],

Cell[CellGroupData[{

Cell["Euler integration", "Subsection"],

Cell[BoxData[
    \(\(eulerIntegrateOne[{tt_, \[Theta]_, \ d\[Theta]_}, traj_, \ dt_, dh_] := 
        Module[{var, \ dvar, \ r1, \ r2, \ dd\[Theta], \[Theta]plus, 
            d\[Theta]plus}, \[IndentingNewLine]var\  = \ 
            ExtractVariables[
              dh]; \[IndentingNewLine]dvar\  = \ \(Dt[#, t] &\)\  /@ \ 
              var; \[IndentingNewLine]r1\  = \ \(\((Rule\  @@ \ #)\) &\)\  \
/@ \ Transpose[{var, \[Theta]\ }]; 
          r2\  = \ \(\((Rule\  @@ \ #)\) &\)\  /@ \ 
              Transpose[{var, \ d\[Theta]}]; 
          dd\[Theta]\  = \ 
            traj\  /. Join[r1, r2]; \[IndentingNewLine]dd\[Theta]\  = \ 
            dd\[Theta]\  /. \ {t -> 
                  tt}; \[IndentingNewLine]d\[Theta]plus\  = \ 
            d\[Theta]\  + \ 
              dd\[Theta]*
                dt; \[IndentingNewLine]\[Theta]plus\  = \ \[Theta]\  + \ 
              d\[Theta]*dt\  + \ 
              0.5*dd\[Theta]*dt*dt; \[IndentingNewLine]{tt + 
              dt, \[Theta]plus, \ 
            d\[Theta]plus}\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(eulerIntegrate[{\[Theta]_, \ d\[Theta]_}, traj_, dt_, steps_, dh_] := 
        Module[{path, \ p2, p3}, \[IndentingNewLine]path\  = \ 
            NestList[
              eulerIntegrateOne[#, \ traj, \ dt, \ dh] &, \ {0. , \[Theta], \ 
                d\[Theta]}, \ 
              steps]; \[IndentingNewLine]p2\  = \((\(Take[#, {1, 
                      2}] &\)\  /@ \ 
                path)\); \[IndentingNewLine]p3\  = \(Table[{#[\([1]\)], \ \
#[\([2, i]\)]}, {i, 1, Length[#[\([2]\)]]}] &\)\  /@ \ p2; 
          Interpolation\  /@ \ 
            Transpose[p3]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " integration"
}], "Subsection"],

Cell[BoxData[
    \(\(MakeSubstitutionRules[
          dh_] := \[IndentingNewLine]Module[{vars, dvars, 
            vs}, \[IndentingNewLine]vars\  = \ 
            ExtractVariables[dh]; \[IndentingNewLine]dvars\  = \ 
            ExtractFirstDerivatives[dh]; 
          vs\  = \ MakeVelocityVariables[
              dh]; \[IndentingNewLine]{\(\((# -> #[t])\) &\)\  /@ \ 
                vars, \ \(\((#[\([1]\)] -> \(#[\([2]\)]\)[t])\) &\)\  /@ \ 
                Transpose[{dvars, \ vs}]}\  // \ 
            Flatten\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeDifferentialEquations[traj_, \ {\[Theta]_, \ d\[Theta]_}, 
          dh_] := \[IndentingNewLine]Module[{vars, \ rul, peq1, vs\ , eq1, 
            eq2, eq3, eq4}, 
          vars\  = \ \ ExtractVariables[
              dh]; \[IndentingNewLine]vt\  = \ \(\((#[t])\) &\)\  /@ \ 
              vars; \[IndentingNewLine]peq1\  = \ 
            traj\  /. \ \((\(\((# -> #[t])\) &\)\  /@ \ 
                  vars)\); \[IndentingNewLine]eq1\  = \ \(\((#[\([1]\)] == #[\
\([2]\)])\) &\)\  /@ \ Transpose[{\(D[#, {t, 2}] &\)\  /@ \ vt, \ peq1}]; 
          eq2\  = \ \(\((#[\([1]\)]\  == \ #[\([2]\)])\) &\)\  /@ \ 
              Transpose[{vt\  /. \ 
                    t -> 0, \[Theta]}]; \[IndentingNewLine]eq3\  = \ \
\(\((#[\([1]\)]\  == \ #[\([2]\)])\) &\)\  /@ \ 
              Transpose[{D[vt, t]\  /. \ t -> 0, \ d\[Theta]}]; 
          Flatten[{eq1, eq2, eq3}]\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(SimulateDynamics[{\[Theta]_, \ d\[Theta]_}, traj_, tmax_, dh_, \ 
          opts___] := \[IndentingNewLine]Module[{vars, 
            eqs}, \[IndentingNewLine]vars\  = \ 
            ExtractVariables[dh]; \[IndentingNewLine]eqs\  = \ 
            MakeDifferentialEquations[traj, \ {\[Theta], \ d\[Theta]}, \ dh]; 
          Flatten[vars\  /. \ 
              NDSolve[eqs, \ vars, {t, 0, tmax}, \ 
                opts]]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Control simulation definitions", "Section"],

Cell[CellGroupData[{

Cell["\<\
Create desired trajectory with derivatives as interpolation \
function\
\>", "Subsection"],

Cell[BoxData[
    \(\(CreateDesiredTrajectory[pts_, {tmin_, tmax_, dt_}, \ 
          fnc_:  MultipleCubicPolyTrajectories] \
:= \[IndentingNewLine]Module[{poly}, \[IndentingNewLine]poly\  = \ 
            fnc[pts]\  // \ 
              Chop; \[IndentingNewLine]{MultiplePolyToInterpolationFunction[
              poly, {tmin, tmax, 
                dt}], \[IndentingNewLine]MultiplePolyToInterpolationFunction[
              Evaluate[D[poly, t]], {tmin, tmax, 
                dt}], \[IndentingNewLine]MultiplePolyToInterpolationFunction[
              Evaluate[D[poly, {t, 2}]], {tmin, tmax, 
                dt}]}\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Manipulator-specific controller functions", "Subsection"],

Cell["\<\
This function expects as input, the dynamic model (as output from \
\[Tau]All[...]), the DH parameters, {tvia, \[Theta]via}, which represent the \
points of the desired trajectory, the frequency of control updates, the \
length of simulation tmax, the controlLaw to apply, and any parameters that \
serve as input to the controlLaw function.
The output of this function is a list {desired, actual, torques} which are \
the desired trajectory (cubic function), the actual trajectory, and the \
corresponding torques.\
\>", "Text"],

Cell[BoxData[
    \(\(ControlManipulatorFromModel[model_, \ dh_, pts_, \ freq_, \ 
          tmax_, \ {controlLaw_, \ params___}, \ 
          opts___] := \[IndentingNewLine]Module[{des, ddes, dddes, fvars, 
            vars, \ eqs, \ desired, \ \[Theta]init, \ d\[Theta]init, \ 
            traj}, \[IndentingNewLine]vars\  = \ 
            ExtractVariables[dh]; \[IndentingNewLine]fvars\  = \ 
            Table[ToExpression["\<\[Tau]\>" <> ToString[i]], {i, 1, 
                Length[model]}]; 
          eqs\  = \ \(\((#[\([1]\)] == #[\([2]\)])\) &\)\  /@ \ 
              Transpose[{model, fvars}]; \[IndentingNewLine]desired\  = 
            CreateDesiredTrajectory[
              pts, {0. , tmax, \ 
                1/freq}]; \[IndentingNewLine]\[Theta]init\  = \ 
            Drop[First[pts], \(-1\)]; \[IndentingNewLine]d\[Theta]init\  = \ 
            Table[0, {Length[vars]}]; \[IndentingNewLine]traj\  = \ 
            ControlAll[eqs\ , \ vars, \ fvars, {\[Theta]init, d\[Theta]init}, 
              tmax, N[1/freq], {controlLaw, \ desired, params}, \ opts]; 
          Partition[Flatten[{desired[\([1]\)], \ traj}], \ 
            Length[vars]]\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(ControlManipulatorFromModel2[model_, \ dh_, \ freq_, \ 
          tmax_, \ {controlLaw_, \ desired_, params___}, \ 
          opts___] := \[IndentingNewLine]Module[{fvars, vars, \ 
            eqs, \ \[Theta]init, \ d\[Theta]init, \ 
            traj}, \[IndentingNewLine]vars\  = \ 
            ExtractVariables[dh]; \[IndentingNewLine]fvars\  = \ 
            Table[ToExpression["\<\[Tau]\>" <> ToString[i]], {i, 1, 
                Length[model]}]; \[IndentingNewLine]eqs\  = \ \(\((#[\([1]\)] \
== #[\([2]\)])\) &\)\  /@ \ 
              Transpose[{model, 
                  fvars}]; \[IndentingNewLine]\[Theta]init\  = \ \(#[
                    0. ] &\)\  /@ \ \((desired[\([1]\)])\)\  // \ 
              Chop; \[IndentingNewLine]d\[Theta]init\  = \ 
            Table[0, {Length[vars]}]; \[IndentingNewLine]traj\  = \ 
            ControlAll[eqs\ , \ vars, \ fvars, {\[Theta]init, d\[Theta]init}, 
              tmax, N[1/freq], {controlLaw, \ desired, params}, \ 
              opts]; \[IndentingNewLine]Partition[
            Flatten[{desired[\([1]\)], \ traj}], \ 
            Length[vars]]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Sample  control laws", "Subsection"],

Cell[CellGroupData[{

Cell["Control law descriptions", "Subsubsection"],

Cell[BoxData[
    \(\(ControlLaws::usage\  = \ "\<Controller options:\n\n\>" \
<> \[IndentingNewLine]\*"\"\<\\tOpenLoopControl[\!\(\[CapitalTheta]\_d\),\!\(\
\[CapitalTheta]\& . \_d\),\!\(\(\(\\\ \
\)\(\[CapitalTheta]\)\)\&\[DoubleDot]\_d\),,,{\[Tau]1,...,\[Tau]n}, vars]:\\n\
\>\"" <> \[IndentingNewLine]"\<\t\t\[Tau] = {\[Tau]1,...,\[Tau]n}\n\>" \
<> \[IndentingNewLine]\*"\"\<\\t\\t  = M(\!\(\[CapitalTheta]\_d\))\!\(\
\[CapitalTheta]\&\[DoubleDot]\_d\) + V(\[CapitalTheta],\!\(\[CapitalTheta]\& \
. \)) + G(\[CapitalTheta])\\n\\n\>\"" \
<> \[IndentingNewLine]\*"\"\<\\tPID[,,,E, \!\(E\& . \), \!\(K\_p\), \
\!\(K\_V\), \!\(K\_i\)]:\\n\>\"" <> \[IndentingNewLine]\*"\"\<\\t\\t\[Tau] = \
\!\(K\_p\).E + \!\(K\_V\).\!\(E\& . \) + \!\(K\_i\).\[Integral]E\
\[DifferentialD]t\\n\\n\>\"" \
<> \[IndentingNewLine]\*"\"\<\\tPIDWithAcceleration[,,\!\(\[CapitalTheta]\&\
\[DoubleDot]\_d\),E, \!\(E\& . \), \!\(K\_p\), \!\(K\_V\), \
\!\(K\_i\)]:\\n\>\"" <> \[IndentingNewLine]\*"\"\<\\t\\t\[Tau] = \!\(K\_p\).E \
+ \!\(K\_V\).\!\(E\& . \) + \!\(K\_i\).\[Integral]E\[DifferentialD]t + \!\(\
\[CapitalTheta]\&\[DoubleDot]\_d\)\\n\\n\>\"" \
<> \[IndentingNewLine]\*"\"\<\\tGravityCompensation[\!\(\[CapitalTheta]\_d\),\
\!\(\[CapitalTheta]\& . \_d\),\!\(\[CapitalTheta]\&\[DoubleDot]\_d\),E, \!\(E\
\& . \), \!\(K\_p\), \!\(K\_V\), \!\(K\_i\),\>\"" \
<> \[IndentingNewLine]"\<{,,G(\[CapitalTheta])}, vars]:\n\>" \
<> \[IndentingNewLine]\*"\"\<\\t\\t\[Tau] = \!\(K\_p\).E + \!\(K\_V\).\!\(E\& \
. \) + \!\(K\_i\).\[Integral]E\[DifferentialD]t + \!\(\[CapitalTheta]\&\
\[DoubleDot]\_d\) + G(\[CapitalTheta])\\n\\n\>\"" <> \[IndentingNewLine]\*"\"\
\<\\tPartitionedControl[\!\(\[CapitalTheta]\_d\),\!\(\[CapitalTheta]\& . \
\_d\),\!\(\[CapitalTheta]\&\[DoubleDot]\_d\),E, \!\(E\& . \), \!\(K\_p\), \
\!\(K\_V\), \!\(K\_i\), \>\"" <> \
\*"\"\<{M(\[CapitalTheta]),V(\[CapitalTheta],\!\(\[CapitalTheta]\& . \)),G(\
\[CapitalTheta])}, vars]:\\n\>\"" <> \[IndentingNewLine]\*"\"\<\\t\\t\[Tau] = \
M(\[CapitalTheta])\!\(\[Tau]\&^\) + V(\[CapitalTheta],\!\(\[CapitalTheta]\& . \
\)) + G(\[CapitalTheta]) where \>\"" \
<> \[IndentingNewLine]\*"\"\<\!\(\[Tau]\&^\) = \!\(K\_p\).E + \
\!\(K\_V\).\!\(E\& . \) + \!\(K\_i\).\[Integral]E\[DifferentialD]t + \!\(\
\[CapitalTheta]\&\[DoubleDot]\_d\)\>\"";\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Open loop \"control\"", "Subsubsection"],

Cell[BoxData[
    \(\(OpenLoopControl[\[Theta]d_, d\[Theta]d_, \ dd\[Theta]d_, _, _, \ 
          model_, \ 
          vars_] := \[IndentingNewLine]Module[{r1, r2, r3, \ dv, \ 
            ddv}, \[IndentingNewLine]dv\  = \ \(Dt[#, t] &\)\  /@ \ 
              vars; \[IndentingNewLine]ddv\  = \ \(Dt[#, \ {t, 2}] &\)\  /@ \ 
              vars; \[IndentingNewLine]r1\  = \ \(\((#[\([1]\)] -> \
#[\([2]\)])\) &\)\  /@ \ 
              Transpose[{vars, \ \[Theta]d}]; \[IndentingNewLine]r2\  = \ \
\(\((#[\([1]\)] -> #[\([2]\)])\) &\)\  /@ \ Transpose[{dv, \ d\[Theta]d}]; 
          r3\  = \ \(\((#[\([1]\)] -> #[\([2]\)])\) &\)\  /@ \ 
              Transpose[{ddv, \ dd\[Theta]d}]; 
          model\  /. \ \((r1~Join~r2~Join~
                r3)\)\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simple PID control ", "Subsubsection"],

Cell["\<\
The first three arguments are the desired \[Theta], d\[Theta] and \
dd\[Theta] , and are not used for this controller.\
\>", "Text"],

Cell[BoxData[
    \(\(PID[_, \ _, \ _, \ err_, \ derr_, \ kp_, \ kv_, \ 
          ki_] := \[IndentingNewLine]kp . err\  + \ kv . derr\  + \ 
          ki . IntegralError;\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[" PID control with desired acceleration feedback", "Subsubsection"],

Cell["\<\
The first two arguments are the desired \[Theta] and d\[Theta], and \
are not used for this controller.\
\>", "Text"],

Cell[BoxData[
    \(\(PIDWithAcceleration[_, \ _, \ dd\[Theta]d_, \ err_, \ derr_, \ kp_, \ 
          kv_, \ ki_] := 
        kp . err\  + \ kv . derr\  + \ ki . IntegralError + 
          dd\[Theta]d;\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Partitioned control", "Subsubsection"],

Cell[BoxData[
    \(\(PartitionedControl[\[Theta]d_, \ d\[Theta]d_, \ dd\[Theta]d_, \ 
          err_, \ derr_, \ kp_, \ kv_, \ ki_, model_, 
          vars_] := \[IndentingNewLine]Module[{dvars, \[Theta], d\[Theta], 
            r1, r2, mm, vv, 
            gv, \[Tau]p}, \[IndentingNewLine]dvars\  = \ \(Dt[#, 
                  t] &\)\  /@ \ 
              vars; \[IndentingNewLine]\[Theta] = \[Theta]d - 
              err; \[IndentingNewLine]d\[Theta] = 
            d\[Theta]d - 
              derr; \[IndentingNewLine]r1\  = \ \(\((#[\([1]\)] -> \
#[\([2]\)])\) &\)\  /@ \ 
              Transpose[{vars, \ \[Theta]}]; \[IndentingNewLine]r2\  = \ \
\(\((#[\([1]\)] -> #[\([2]\)])\) &\)\  /@ \ 
              Transpose[{dvars, \ d\[Theta]}]; \[IndentingNewLine]{mm, vv, 
              gv} = model\  /. \ \((r1~Join~
                  r2)\); \[IndentingNewLine]\[Tau]p\  = \ 
            kp . err\  + \ kv . derr\  + \ ki . IntegralError + 
              dd\[Theta]d\ ; \[IndentingNewLine]mm . \[Tau]p\  + \ vv\  + \ 
            gv\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["PID control with gravity compensation", "Subsubsection"],

Cell[BoxData[
    \(\(GravityCompensation[\[Theta]d_, \ d\[Theta]d_, \ dd\[Theta]d_, \ 
          err_, \ derr_, \ kp_, \ kv_, \ ki_, model_, 
          vars_] := \[IndentingNewLine]Module[{n, mm, vv, 
            gv}, \[IndentingNewLine]n\  = 
            Length[vars]; \[IndentingNewLine]mm\  = 
            IdentityMatrix[n]; \[IndentingNewLine]vv\  = \ 
            Table[0. , {n}]; \[IndentingNewLine]gv\  = \ 
            Last[model]; \[IndentingNewLine]PartitionedControl[\[Theta]d, \ 
            d\[Theta]d, \ dd\[Theta]d, \ err, \ derr, \ kp, \ kv, \ 
            ki, {mm, vv, gv}, vars]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["C-code generation functions", "Section"],

Cell[CellGroupData[{

Cell["\<\
Rewrite torque equations (dynamic model) in terms of second \
derivatives\
\>", "Subsection"],

Cell[CellGroupData[{

Cell["Rewrite torque equations in terms of second derivatives", \
"Subsubsection"],

Cell[BoxData[
    \(\(DynamicModelToSecondOrderDifferentialEquations[model_, 
          vars_] := \[IndentingNewLine]Module[{fvars, 
            eqs}, \[IndentingNewLine]fvars\  = \ 
            Table[ToExpression["\<tau\>" <> ToString[i]], {i, 1, 
                Length[model]}]; 
          eqs\  = \ \(\((#[\([1]\)] == #[\([2]\)])\) &\)\  /@ \ 
              Transpose[{model, 
                  fvars}]; \[IndentingNewLine]IsolateSecondDerivatives[eqs, 
            vars]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Generate list of arguments for a specific function type", "Subsection"],

Cell[BoxData[
    \(\(part[exp_, \ l_] := \ Part\  @@ \ Join[{exp}, l];\)\)], "Input"],

Cell[BoxData[
    \(\(GenerateFunctionArguments[cmodel_, \ vars_, \ 
          fnc_] := \[IndentingNewLine]Module[{tmp}, \[IndentingNewLine]tmp = \
\(\((\(part[cmodel, \ Drop[#, \(-1\)]] &\)\  /@ \ 
                    Position[cmodel, fnc])\) // \ Sort\) // \ 
              Union\ ; \[IndentingNewLine]\(#[\([1]\)] &\)\  /@ \ 
            tmp\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Create conversion rules", "Subsection"],

Cell[CellGroupData[{

Cell["Generic conversion rule function", "Subsubsection"],

Cell[BoxData[
    \(\(CreateConversionRule[one_, \ {fnc_, params___}, \ str_, \ 
          n_] := \[IndentingNewLine]Module[{tmp}, \[IndentingNewLine]tmp\  = \
\ If[Length[one]\  == \ 0, \ {one}, \ List\  @@ \ one]; 
          fnc[one, \ params] -> 
            ToExpression[
              ToString[str] <> \ 
                ToString[n]]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Conversion rules for cosine terms", "Subsubsection"],

Cell[BoxData[
    \(\(CosineConversionRules[cmodel_, 
          vars_] := \[IndentingNewLine]Module[{tmp}, \[IndentingNewLine]tmp\  \
= \ GenerateFunctionArguments[cmodel, vars, Cos]; \[IndentingNewLine]Table[
            CreateConversionRule[tmp[\([i]\)], {Cos}, \ "\<c\>", \ i], \ {i, 
              1, Length[tmp]}]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Conversion rules for sine terms", "Subsubsection"],

Cell[BoxData[
    \(\(SineConversionRules[cmodel_, 
          vars_] := \[IndentingNewLine]Module[{tmp}, \[IndentingNewLine]tmp\  \
= \ GenerateFunctionArguments[cmodel, vars, Sin]; 
          Table[CreateConversionRule[tmp[\([i]\)], {Sin}, \ "\<s\>", \ 
              i], \ {i, 1, Length[tmp]}]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Conversion rules for derivative terms", "Subsubsection"],

Cell[BoxData[
    \(\(DerivativeConversionRules[cmodel_, \ 
          vars_] := \[IndentingNewLine]Module[{tmp}, \[IndentingNewLine]tmp\  \
= \ GenerateFunctionArguments[cmodel, vars, Dt]; 
          Table[CreateConversionRule[tmp[\([i]\)], {Dt, t}, \ "\<dth\>", \ 
              i], \ {i, 1, Length[tmp]}]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Conversion rules for all terms", "Subsubsection"],

Cell[BoxData[
    \(\(AllConversionRules[cmodel_, 
          vars_] := \[IndentingNewLine]Join[\[IndentingNewLine]\
CosineConversionRules[cmodel, vars], \[IndentingNewLine]SineConversionRules[
            cmodel, vars], \[IndentingNewLine]DerivativeConversionRules[
            cmodel, vars]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Create C variable assignments for trig functions", "Subsection"],

Cell[CellGroupData[{

Cell["Generic function (works for both cosine and sine)", "Subsubsection"],

Cell[BoxData[
    \(\(MakeTrigVariableAssignments[one_, \ fnc_, \ str_, \ 
          n_] := \[IndentingNewLine]Module[{str2}, \[IndentingNewLine]str2\  \
= \ StringReplace[ToString[CForm[one]], \ "\<\[Theta]\>" -> "\<th\>"]; 
          str <> ToString[n] <> "\< = \>" <> ToString[fnc] <> "\<(\>" <> 
            str2 <> "\<)\>"\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cosine variable assignments", "Subsubsection"],

Cell[BoxData[
    \(\(MakeCosVariableAssignments[cmodel_, 
          vars_] := \[IndentingNewLine]Module[{tmp}, \[IndentingNewLine]tmp\  \
= \ GenerateFunctionArguments[cmodel, \ vars, \ 
              Cos]; \[IndentingNewLine]Table[
            MakeTrigVariableAssignments[tmp[\([i]\)], Cos, \ "\<c\>", \ 
              i], \ {i, 1, Length[tmp]}]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Sine variable assignments", "Subsubsection"],

Cell[BoxData[
    \(\(MakeSinVariableAssignments[cmodel_, 
          vars_] := \[IndentingNewLine]Module[{tmp}, \[IndentingNewLine]tmp\  \
= \ GenerateFunctionArguments[cmodel, \ vars, \ 
              Sin]; \[IndentingNewLine]Table[
            MakeTrigVariableAssignments[tmp[\([i]\)], Sin, \ "\<s\>", \ 
              i], \ {i, 1, Length[tmp]}]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Create C variable assignments and declarations", "Subsection"],

Cell[CellGroupData[{

Cell["C variable assignments", "Subsubsection"],

Cell[BoxData[
    \(\(MakeVariableAssignments[cmodel_, vars_, \ 
          tauInclude_:  True] := \[IndentingNewLine]Module[{n, e1, e2, e3, 
            e4, e5}, \[IndentingNewLine]n\  = \ 
            Length[vars]; \[IndentingNewLine]e1\  = \ 
            Table["\<th\>" <> ToString[i] <> "\< = th[\>" <> 
                ToString[i - 1] <> "\<]\>", \ {i, 1, \ 
                n}]; \[IndentingNewLine]e2\  = \ 
            Table["\<dth\>" <> ToString[i] <> "\< = dth[\>" <> 
                ToString[i - 1] <> "\<]\>", \ {i, 1, \ 
                n}]; \[IndentingNewLine]e3\  = \ 
            Table["\<tau\>" <> ToString[i] <> "\< = tau[\>" <> 
                ToString[i - 1] <> "\<]\>", \ {i, 1, \ 
                n}]; \[IndentingNewLine]e4\  = \ 
            MakeCosVariableAssignments[cmodel, 
              vars]; \[IndentingNewLine]e5\  = \ 
            MakeSinVariableAssignments[cmodel, vars]; \[IndentingNewLine]If[
            tauInclude\  \[Equal] \ True, \[IndentingNewLine]Join[e1, e2, e3, 
              e4, e5], \[IndentingNewLine]Join[e1, e2, e4, 
              e5]\[IndentingNewLine]]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["C variable declarations", "Subsubsection"],

Cell[BoxData[
    \(\(MakeVariableDeclarations[
          varEqs_] := \[IndentingNewLine]Module[{tmp1, 
            tmp2}, \[IndentingNewLine]tmp1\  = \(\((# - 
                    1)\) &\)\  /@ \ \((First\  /@ \ 
                  Partition[
                    Flatten[\(StringPosition[#, \ "\< =\>"] &\)\  /@ \ 
                        varEqs], 
                    2])\); \[IndentingNewLine]tmp2\  = \ {varEqs, 
                tmp1}\  // \ 
              Transpose; \[IndentingNewLine]Table["\<double \>" <> 
              StringTake[tmp2[\([i, 1]\)], \ tmp2[\([i, 2]\)]], {i, 1, 
              Length[tmp2]}]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Convert dynamic model to robot C code (for Runge-Kutta integration)\
\
\>", "Subsection"],

Cell[CellGroupData[{

Cell["\<\
Convert second-order diff. eqs. to set of first-order diff. \
eqs.\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(SecondOrderToFirstOrderDifferentialEquations[cmodel_, 
          vars_] := \[IndentingNewLine]Module[{eqs1, 
            eqs2}, \[IndentingNewLine]eqs1 = \ \(Dt[#, t] &\)\  /@ \ 
              vars; \[IndentingNewLine]eqs2\  = \ 
            Join[eqs1, cmodel]\  /. \ 
                AllConversionRules[cmodel\  // \ Chop, vars]\  // \ Chop; 
          Table["\<f[\>" <> ToString[i - 1] <> "\<] = \>" <> 
              ToString[CForm[eqs2[\([i]\)]\  // \ N]], {i, 1, 
              Length[eqs2]}]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Convert dynamic model to robot C code", "Subsubsection"],

Cell[BoxData[
    \(\(DynamicModelToDiffEqCCode[model_, vars_, nvals_] := 
        Module[{cmodel, e0, e1, e2, all}, 
          cmodel = 
            DynamicModelToSecondOrderDifferentialEquations[model\  // \ Chop, 
                vars]\  // \ Chop; \[IndentingNewLine]e1 = 
            MakeVariableAssignments[cmodel, vars, \ 
              True]; \[IndentingNewLine]e0 = 
            MakeVariableDeclarations[e1]; \[IndentingNewLine]e2 = 
            SecondOrderToFirstOrderDifferentialEquations[
              cmodel\  /. \ nvals\  // \ Chop, 
              vars]; \[IndentingNewLine]all = 
            Join[e0, e1, 
              e2]; \[IndentingNewLine]\(\(("\<  \>" <> #)\) &\) /@ \((\(\((# <> \
"\<;\>")\) &\) /@ all)\)\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Convert dynamic model to model C code (for model-based \
control)\
\>", "Subsection"],

Cell[CellGroupData[{

Cell["Convert matrix/vector to C code", "Subsubsection"],

Cell[BoxData[
    \(\(MatrixOrVectorToCCode[mat_, vars_, \ str_, 
          convRules_] := \[IndentingNewLine]Module[{v, \ eqs, body, prelog, 
            epilog}, \[IndentingNewLine]v\  = \ 
            Flatten[mat]; \[IndentingNewLine]eqs\  = \ 
            v\  /. \ convRules; \[IndentingNewLine]prelog = {"\<if(\>" <> 
                str <> "\< != NULL) {^^^\>"}; \[IndentingNewLine]body = 
            Table["\< \>" <> ToString[str] <> "\<[\>" <> 
                ToString[i - 1] <> "\<] = \>" <> 
                ToString[CForm[eqs[\([i]\)]]], {i, 1, 
                Length[eqs]}]; \[IndentingNewLine]epilog\  = \ {"\<}^^^\>"}; \
\[IndentingNewLine]prelog~Join~body~Join~
            epilog\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Convert dynamic model to model C code", "Subsubsection"],

Cell[BoxData[
    \(\(DynamicModelToMatrixCCode[model_, vars_, 
          nvals_] := \[IndentingNewLine]Module[{cmodel, \ mm, vv, gv, e0, \ 
            e1, e2, e3, e4, \ all, \ 
            convRules}, \[IndentingNewLine]cmodel\  = \ \
\(DynamicModelToSecondOrderDifferentialEquations[model\  // \ Chop, vars]\  // 
                Simplify\) // \ Chop; \[IndentingNewLine]convRules\  = \ 
            AllConversionRules[Flatten[cmodel\  // \ Chop], 
              vars]; \[IndentingNewLine]e1\  = \ 
            MakeVariableAssignments[cmodel, vars, \ 
              False]; \[IndentingNewLine]e0\  = \ 
            MakeVariableDeclarations[e1]; \[IndentingNewLine]{mm, vv, 
              gv} = \ \((\[Tau]MatrixFromModel[model, vars]\  // 
                    Simplify)\) /. \ nvals\  // \ 
              Chop; \[IndentingNewLine]e2\  = 
            MatrixOrVectorToCCode[mm, \ vars, \ "\<m\>", \ 
              convRules]; \[IndentingNewLine]e3\  = 
            MatrixOrVectorToCCode[vv, \ vars, \ "\<v\>", \ 
              convRules]; \[IndentingNewLine]e4\  = 
            MatrixOrVectorToCCode[gv, \ vars, \ "\<g\>", \ 
              convRules]; \[IndentingNewLine]all\  = \ 
            Join[e0, e1, e2, e3, 
              e4]; \[IndentingNewLine]\(StringReplace[#, "\<^^^;\>" -> \
"\<\>"] &\)\  /@ \ \((\(\(("\<  \>" <> #)\) &\)\  /@ \ \((\(\((# <> \
"\<;\>")\) &\)\  /@ \ all)\))\)\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Generate file header information", "Subsection"],

Cell[CellGroupData[{

Cell["Convert date to string", "Subsubsection"],

Cell[BoxData[
    \(\(ConvertDateToString[] := \[IndentingNewLine]Module[{yr, \ mo, \ day, 
            hr, min, sec, \ date}, \[IndentingNewLine]date = \ 
            Date[]; \[IndentingNewLine]{yr, \ mo, \ day, hr, min, sec} = \ 
            ToString\  /@ \ date; \[IndentingNewLine]mo <> "\</\>" <> 
            day <> "\</\>" <> yr\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Convert dynamic parameters to strings", "Subsubsection"],

Cell[BoxData[
    \(\(ConvertDynamicParametersToString[dyn_, \ 
          prefix_] := \[IndentingNewLine]Module[{s1, s2, s3, s4, s5, s6, s7, 
            s8, sn}, \[IndentingNewLine]sn = {"\<\>"}; \[IndentingNewLine]s1 \
= {prefix <> "\< DH parameters:\>"}; \[IndentingNewLine]s2 = \
\(StringReplace[#, "\<\[Theta]\>" -> "\<th\>"] &\)\  /@ \ {TableForm[
                    dyn[\([1]\)], \ TableSpacing \[Rule] {0, 2}]\  // \ 
                  ToString}; \[IndentingNewLine]s3 = {prefix <> "\< inertia \
tensors:\>"}; \[IndentingNewLine]s4 = \({\(ToString[
                        TableForm\ [#, TableSpacing \[Rule] {0, 2}]] &\) /@ \ 
                    Drop[dyn[\([2]\)], 1], 
                  Table[sn[\([1]\)], {Length[dyn[\([2]\)]] - 1}]}\  // \ 
                Transpose\)\  // \ 
              Flatten; \[IndentingNewLine]s5 = {prefix <> \ "\< masses:\>"}; \
\[IndentingNewLine]s6 = 
            ToString\  /@ \ 
              Drop[dyn[\([3]\)], 
                1]; \[IndentingNewLine]s7 = {prefix <> "\< Pc vectors:\>"}; \
\[IndentingNewLine]s8 = {TableForm[Drop[Last[dyn], 1], \ 
                  TableSpacing \[Rule] {1, 2}]\  // \ 
                ToString}; \[IndentingNewLine]Join\  @@ \ {s1, sn, s2, sn, 
              s3, sn, s4, sn, s5, sn, s6, sn, s7, sn, s8, 
              sn}\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compose top file header", "Subsubsection"],

Cell[BoxData[
    \(\(ComposeTopHeader[
          file_] := \[IndentingNewLine]Module[{s1, s2, s3, spcs, s4tmp, 
            s4}, \[IndentingNewLine]spcs = "\<                                \
                                           \>"; \[IndentingNewLine]s1 = \
{"\</*************************************************************************\
****/\>"}; \[IndentingNewLine]s2 = {"\</*                                     \
                                      */\>"}; \[IndentingNewLine]s3 = \
{"\</*************************************************************************\
****/\>"}; \[IndentingNewLine]s3 = {"\</* Author: Michael C. Nechyba          \
                                      */\>"}; \[IndentingNewLine]s4tmp = "\< \
File: \>" <> file; \[IndentingNewLine]s4 = {"\</*\>" <> s4tmp <> 
                StringTake[spcs, 
                  StringLength[spcs] - 
                    StringLength[
                      s4tmp]] <> "\<*/\>"}; \[IndentingNewLine]Join\  @@ \ \
{s1, s2, s3, s4, s2, s1}\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compose parameter file header", "Subsubsection"],

Cell[BoxData[
    \(\(ComposeHeader[dynIdealized_, \ 
          dynReal_] := \[IndentingNewLine]Module[{sn, so, sc, s1, s2, s3, s4, 
            s5, s6, \ 
            s7}, \[IndentingNewLine]sn = {"\<\>"}; \[IndentingNewLine]so = {"\
\</*\>"}; \ \[IndentingNewLine]sc = \ {"\<*/\>"}; \[IndentingNewLine]s1 = \
{"\</*************************************************************************\
****/\>"}; \[IndentingNewLine]s2 = {"\<  File generated by Mathematica on \>" \
<> ConvertDateToString[] <> "\<.\>"}; \[IndentingNewLine]s3 = 
            ConvertDynamicParametersToString[
              dynIdealized\  // \ 
                N, "\<Idealized\>"]; \[IndentingNewLine]s4 = 
            ConvertDynamicParametersToString[
              dynReal\  // \ 
                N, "\<Real\>"]; \[IndentingNewLine]s5 = "\<#include <stdio.h>\
\n#include <math.h>\n#include <stdlib.h>\n#include <string.h>\>"; \
\[IndentingNewLine]s6 = "\<#include \"manip.h\"\>"; \[IndentingNewLine]s7 = "\
\<#include \"robots.h\"\>"; \[IndentingNewLine]Join\  @@ \ {s1, so, s2, sc, 
              s1, sn, s1, so, s3, sc, s1, sn, s1, so, s4, sn, sc, s1, 
              sn, {s5}, sn, {s6}, {s7}}\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["C code file output", "Subsection"],

Cell[CellGroupData[{

Cell["Write C code function to file pointer", "Subsubsection"],

Cell[BoxData[
    \(\(WriteCCodeFunction[fp_, \ code_, \ func_, \ 
          str_] := \[IndentingNewLine]Module[{}, 
          WriteString[
            fp, \ "\<\n\n\
/*****************************************************************************\
/\n\>"]; WriteString[fp, \ "\<\nint \>" <> ToString[func] <> str]; 
          Do[WriteString[fp, \ code[\([i]\)] <> "\<\n\>"], {i, 1, 
              Length[code]}]; \[IndentingNewLine]WriteString[
            fp, \ "\<\n  return(1);\n}\>"];\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Write C robot code to file pointer", "Subsubsection"],

Cell[BoxData[
    \(\(WriteCCodeRobot[fp_, \ code_, \ 
          func_] := \[IndentingNewLine]Module[{str}, \[IndentingNewLine]str\  \
= \ "\<(double *f, double *th, double *dth,\ndouble *tau)\n{\n\>"; \
\[IndentingNewLine]WriteCCodeFunction[fp, code, func, 
            str];\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Write C model code to file pointer", "Subsubsection"],

Cell[BoxData[
    \(\(WriteCCodeModel[fp_, \ code_, \ 
          func_] := \[IndentingNewLine]Module[{str}, \[IndentingNewLine]str\  \
= "\<(double *m, double *v, double *g,\ndouble *th, double *dth)\n{\n\>"; \
\[IndentingNewLine]WriteCCodeFunction[fp, code, func, 
            str];\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Write C code segments to file (default append)", "Subsubsection"],

Cell[TextData[{
  "This function appends the two code segments \"cRobot\" and \"cModel\" to a \
\"file.\" The code segment \"cRobot\" corresponds to the dynamic model \
suitable for Runge-Kutta integration. The code segment \"cModel\" computes \
the dynamic model in parts. For ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau] = \(M(\[CapitalTheta])\)\ \[CapitalTheta]\&\
\[DoubleDot] + V(\[CapitalTheta], \(\[CapitalTheta]\& . \)) + 
          G(\[CapitalTheta])\)]],
  ", it computes ",
  Cell[BoxData[
      \(TraditionalForm\`M(\[CapitalTheta])\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`V(\[CapitalTheta], \(\[CapitalTheta]\& . \))\)]],
  ", and ",
  Cell[BoxData[
      \(TraditionalForm\`G(\[CapitalTheta])\)]],
  "separately.The function names \"fRobot\" and \"fModel\" are assigned to \
each code segment."
}], "Text"],

Cell[BoxData[
    \(\(AppendCCodeToFile[file_, \ cRobot_, \ fRobot_, \ cModel_, \ fModel_, 
          open_:  OpenAppend] := \[IndentingNewLine]Module[{fp}, \n\t
          fp\  = \ open[file]; \n\tWriteCCodeRobot[fp, cRobot, fRobot]; \n\t
          WriteCCodeModel[fp, cModel, fModel]; \n\t
          Close[fp];\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Complete C file", "Subsubsection"],

Cell[BoxData[
    \(\(CompleCCodeToFile[outputFile_, \ dyn_, \ modelParameters_, \ 
          robotParameters_, \ modelDiffEqs_, modelMatrices_, \ 
          robotDiffEqs_, \ robotMatrices_, f1_, f2_, f3_, 
          f4_] := \[IndentingNewLine]Module[{fp, \ headerTop, \ 
            header}, \[IndentingNewLine]fp\  = \ 
            OpenWrite[outputFile]; \[IndentingNewLine]headerTop\  = \ 
            ComposeTopHeader[outputFile]; \[IndentingNewLine]Do[
            WriteString[fp, \ headerTop[\([i]\)] <> "\<\n\>"], {i, 1, 
              Length[headerTop]}]; 
          header\  = \ 
            ComposeHeader[dyn\  /. \ modelParameters\ , \ 
              dyn\  /. \ robotParameters]; \[IndentingNewLine]Do[
            WriteString[fp, \ header[\([i]\)] <> "\<\n\>"], {i, 1, 
              Length[header]}]; \[IndentingNewLine]Close[
            fp]; \[IndentingNewLine]AppendCCodeToFile[outputFile, \ 
            modelDiffEqs, \ f1, \ modelMatrices, \ f2]; 
          AppendCCodeToFile[outputFile, \ robotDiffEqs, \ f3, \ 
            robotMatrices, \ f4]; \[IndentingNewLine]fp\  = \ 
            OpenAppend[outputFile]; \[IndentingNewLine]WriteString[
            fp, "\<\n\n\
/*****************************************************************************\
/\n\>"]; Close[fp]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]]
}, Open  ]]
}, Closed]]
},
FrontEndVersion->"5.0 for Macintosh",
ScreenRectangle->{{0, 1280}, {0, 828}},
WindowSize->{684, 413},
WindowMargins->{{Automatic, 1}, {Automatic, 0}},
PrintingCopies->1,
PrintingPageRange->{1, 9999},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"Magnification"->1},
CellLabelAutoDelete->True,
StyleDefinitions -> "Classroom.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 49, 0, 62, "Title"],
Cell[1806, 53, 136, 3, 28, "Text"],

Cell[CellGroupData[{
Cell[1967, 60, 65, 0, 59, "Section"],

Cell[CellGroupData[{
Cell[2057, 64, 41, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[2123, 68, 60, 0, 43, "Subsubsection"],
Cell[2186, 70, 250, 5, 111, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2473, 80, 51, 0, 43, "Subsubsection"],
Cell[2527, 82, 220, 4, 95, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2784, 91, 58, 0, 43, "Subsubsection"],
Cell[2845, 93, 113, 2, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2995, 100, 58, 0, 43, "Subsubsection"],
Cell[3056, 102, 114, 2, 47, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[3219, 110, 67, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[3311, 114, 67, 0, 43, "Subsubsection"],
Cell[3381, 116, 370, 9, 79, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3788, 130, 120, 3, 43, "Subsubsection"],
Cell[3911, 135, 148, 2, 63, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4096, 142, 103, 3, 43, "Subsubsection"],
Cell[4202, 147, 74, 1, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4313, 153, 68, 0, 42, "Subsubsection"],
Cell[4384, 155, 147, 3, 59, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4568, 163, 134, 5, 42, "Subsubsection"],
Cell[4705, 170, 594, 10, 91, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5336, 185, 108, 3, 42, "Subsubsection"],
Cell[5447, 190, 82, 1, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5566, 196, 71, 0, 42, "Subsubsection"],
Cell[5640, 198, 159, 3, 59, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5836, 206, 86, 1, 42, "Subsubsection"],
Cell[5925, 209, 81, 1, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6043, 215, 86, 1, 42, "Subsubsection"],
Cell[6132, 218, 105, 2, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6274, 225, 71, 0, 42, "Subsubsection"],
Cell[6348, 227, 330, 7, 107, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6727, 240, 99, 3, 46, "Subsection"],

Cell[CellGroupData[{
Cell[6851, 247, 47, 0, 42, "Subsubsection"],
Cell[6901, 249, 164, 4, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7102, 258, 52, 0, 42, "Subsubsection"],
Cell[7157, 260, 336, 6, 100, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7530, 271, 50, 0, 42, "Subsubsection"],
Cell[7583, 273, 142, 2, 43, "Input"],
Cell[7728, 277, 126, 2, 27, "Input"],
Cell[7857, 281, 132, 2, 27, "Input"],
Cell[7992, 285, 224, 3, 75, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8253, 293, 115, 3, 42, "Subsubsection"],
Cell[8371, 298, 117, 3, 30, "Text"],
Cell[8491, 303, 2304, 40, 347, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10832, 348, 110, 3, 42, "Subsubsection"],
Cell[10945, 353, 226, 3, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11208, 361, 85, 1, 42, "Subsubsection"],
Cell[11296, 364, 491, 8, 107, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11824, 377, 121, 3, 42, "Subsubsection"],
Cell[11948, 382, 226, 3, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12211, 390, 76, 0, 42, "Subsubsection"],
Cell[12290, 392, 1081, 18, 171, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13408, 415, 52, 0, 42, "Subsubsection"],
Cell[13463, 417, 118, 2, 27, "Input"],
Cell[13584, 421, 828, 15, 123, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14449, 441, 77, 0, 42, "Subsubsection"],
Cell[14529, 443, 406, 7, 107, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[14984, 456, 56, 0, 46, "Subsection"],
Cell[15043, 458, 123, 3, 30, "Text"],

Cell[CellGroupData[{
Cell[15191, 465, 64, 0, 42, "Subsubsection"],
Cell[15258, 467, 253, 4, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15548, 476, 110, 3, 42, "Subsubsection"],
Cell[15661, 481, 170, 3, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15868, 489, 60, 0, 42, "Subsubsection"],
Cell[15931, 491, 253, 4, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16221, 500, 121, 3, 42, "Subsubsection"],
Cell[16345, 505, 176, 3, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16558, 513, 59, 0, 42, "Subsubsection"],
Cell[16620, 515, 248, 4, 43, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[16917, 525, 94, 3, 46, "Subsection"],

Cell[CellGroupData[{
Cell[17036, 532, 42, 0, 42, "Subsubsection"],
Cell[17081, 534, 181, 3, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17299, 542, 55, 0, 42, "Subsubsection"],
Cell[17357, 544, 152, 2, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17546, 551, 58, 0, 42, "Subsubsection"],
Cell[17607, 553, 507, 8, 107, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18151, 566, 76, 0, 42, "Subsubsection"],
Cell[18230, 568, 569, 9, 107, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18836, 582, 77, 0, 42, "Subsubsection"],
Cell[18916, 584, 467, 9, 107, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[19432, 599, 69, 0, 46, "Subsection"],

Cell[CellGroupData[{
Cell[19526, 603, 68, 0, 42, "Subsubsection"],
Cell[19597, 605, 571, 9, 123, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20205, 619, 75, 0, 42, "Subsubsection"],
Cell[20283, 621, 190, 3, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20510, 629, 87, 1, 42, "Subsubsection"],
Cell[20600, 632, 242, 4, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20879, 641, 63, 0, 42, "Subsubsection"],
Cell[20945, 643, 483, 8, 107, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[21465, 656, 82, 1, 42, "Subsubsection"],
Cell[21550, 659, 240, 4, 43, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[21839, 669, 56, 0, 46, "Subsection"],

Cell[CellGroupData[{
Cell[21920, 673, 63, 0, 42, "Subsubsection"],
Cell[21986, 675, 866, 14, 139, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[22889, 694, 65, 0, 42, "Subsubsection"],
Cell[22957, 696, 692, 11, 123, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[23686, 712, 69, 0, 42, "Subsubsection"],
Cell[23758, 714, 696, 11, 123, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24491, 730, 56, 0, 42, "Subsubsection"],
Cell[24550, 732, 1029, 17, 171, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[25628, 755, 64, 0, 46, "Subsection"],

Cell[CellGroupData[{
Cell[25717, 759, 67, 0, 42, "Subsubsection"],
Cell[25787, 761, 728, 11, 155, "Input"],
Cell[26518, 774, 664, 12, 139, "Input"],
Cell[27185, 788, 560, 8, 107, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[27782, 801, 60, 0, 42, "Subsubsection"],
Cell[27845, 803, 655, 12, 139, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[28537, 820, 41, 0, 42, "Subsubsection"],
Cell[28581, 822, 933, 15, 171, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29551, 842, 34, 0, 42, "Subsubsection"],
Cell[29588, 844, 807, 13, 155, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[30444, 863, 52, 0, 46, "Subsection"],

Cell[CellGroupData[{
Cell[30521, 867, 75, 0, 42, "Subsubsection"],
Cell[30599, 869, 677, 12, 155, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[31313, 886, 53, 0, 42, "Subsubsection"],
Cell[31369, 888, 110, 3, 27, "Input"],
Cell[31482, 893, 1521, 25, 267, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[33040, 923, 51, 0, 43, "Subsubsection"],
Cell[33094, 925, 110, 3, 27, "Input"],
Cell[33207, 930, 1685, 28, 319, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[34941, 964, 47, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[35013, 968, 35, 0, 43, "Subsubsection"],
Cell[35051, 970, 318, 6, 95, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[35406, 981, 56, 0, 43, "Subsubsection"],
Cell[35465, 983, 442, 7, 127, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[35944, 995, 104, 3, 43, "Subsubsection"],
Cell[36051, 1000, 310, 5, 79, "Input"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[36422, 1012, 51, 0, 39, "Section"],

Cell[CellGroupData[{
Cell[36498, 1016, 58, 0, 49, "Subsection"],
Cell[36559, 1018, 475, 16, 46, "Text"],

Cell[CellGroupData[{
Cell[37059, 1038, 76, 0, 43, "Subsubsection"],
Cell[37138, 1040, 315, 5, 79, "Input"],
Cell[37456, 1047, 103, 2, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37596, 1054, 97, 3, 43, "Subsubsection"],
Cell[37696, 1059, 146, 2, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37879, 1066, 97, 3, 43, "Subsubsection"],
Cell[37979, 1071, 81, 1, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[38097, 1077, 348, 9, 43, "Subsubsection"],
Cell[38448, 1088, 340, 6, 127, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[38825, 1099, 334, 8, 43, "Subsubsection"],
Cell[39162, 1109, 79, 1, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[39278, 1115, 215, 6, 43, "Subsubsection"],
Cell[39496, 1123, 79, 1, 47, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[39624, 1130, 61, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[39710, 1134, 315, 11, 43, "Subsubsection"],
Cell[40028, 1147, 76, 1, 47, "Input"],
Cell[40107, 1150, 228, 4, 47, "Input"],
Cell[40338, 1156, 114, 2, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[40489, 1163, 296, 9, 43, "Subsubsection"],
Cell[40788, 1174, 69, 1, 47, "Input"],
Cell[40860, 1177, 294, 6, 63, "Input"],
Cell[41157, 1185, 91, 1, 47, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[41297, 1192, 57, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[41379, 1196, 98, 3, 43, "Subsubsection"],
Cell[41480, 1201, 354, 6, 111, "Input"],
Cell[41837, 1209, 247, 4, 111, "Input"],
Cell[42087, 1215, 281, 5, 95, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42405, 1225, 85, 1, 43, "Subsubsection"],
Cell[42493, 1228, 431, 8, 143, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42961, 1241, 86, 1, 42, "Subsubsection"],
Cell[43050, 1244, 441, 8, 123, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[43528, 1257, 71, 0, 42, "Subsubsection"],
Cell[43602, 1259, 178, 3, 43, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[43829, 1268, 77, 0, 46, "Subsection"],

Cell[CellGroupData[{
Cell[43931, 1272, 112, 3, 42, "Subsubsection"],
Cell[44046, 1277, 385, 7, 123, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[44468, 1289, 113, 3, 42, "Subsubsection"],
Cell[44584, 1294, 675, 13, 139, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[45296, 1312, 101, 3, 42, "Subsubsection"],
Cell[45400, 1317, 157, 3, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[45606, 1326, 46, 0, 46, "Subsection"],

Cell[CellGroupData[{
Cell[45677, 1330, 75, 0, 42, "Subsubsection"],
Cell[45755, 1332, 201, 3, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[45993, 1340, 76, 0, 42, "Subsubsection"],
Cell[46072, 1342, 340, 6, 59, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[46449, 1353, 104, 3, 42, "Subsubsection"],
Cell[46556, 1358, 233, 4, 43, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[46838, 1368, 42, 0, 46, "Subsection"],

Cell[CellGroupData[{
Cell[46905, 1372, 332, 10, 42, "Subsubsection"],
Cell[47240, 1384, 86, 1, 47, "Input"],
Cell[47329, 1387, 405, 7, 79, "Input"],
Cell[47737, 1396, 114, 2, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[47888, 1403, 323, 11, 43, "Subsubsection"],
Cell[48214, 1416, 104, 2, 47, "Input"],
Cell[48321, 1420, 585, 11, 127, "Input"],
Cell[48909, 1433, 91, 1, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[49037, 1439, 350, 11, 43, "Subsubsection"],
Cell[49390, 1452, 242, 5, 63, "Input"],
Cell[49635, 1459, 111, 2, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[49783, 1466, 263, 8, 43, "Subsubsection"],
Cell[50049, 1476, 117, 2, 47, "Input"],
Cell[50169, 1480, 209, 3, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[50415, 1488, 360, 11, 43, "Subsubsection"],
Cell[50778, 1501, 297, 5, 79, "Input"],
Cell[51078, 1508, 501, 9, 95, "Input"],
Cell[51582, 1519, 284, 5, 79, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[51903, 1529, 204, 6, 43, "Subsubsection"],
Cell[52110, 1537, 200, 3, 63, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[52347, 1545, 205, 5, 43, "Subsubsection"],
Cell[52555, 1552, 600, 10, 143, "Input"],
Cell[53158, 1564, 416, 8, 127, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[53611, 1577, 259, 7, 43, "Subsubsection"],
Cell[53873, 1586, 662, 11, 127, "Input"],
Cell[54538, 1599, 525, 9, 127, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[55100, 1613, 219, 6, 43, "Subsubsection"],
Cell[55322, 1621, 339, 6, 111, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[55698, 1632, 193, 5, 43, "Subsubsection"],
Cell[55894, 1639, 229, 4, 63, "Input"],
Cell[56126, 1645, 242, 4, 63, "Input"],
Cell[56371, 1651, 232, 4, 63, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[56640, 1660, 302, 7, 43, "Subsubsection"],
Cell[56945, 1669, 354, 6, 111, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[57336, 1680, 175, 5, 43, "Subsubsection"],
Cell[57514, 1687, 267, 4, 63, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[57830, 1697, 41, 0, 49, "Subsection"],
Cell[57874, 1699, 269, 5, 79, "Input"],
Cell[58146, 1706, 669, 12, 143, "Input"],
Cell[58818, 1720, 873, 15, 155, "Input"],
Cell[59694, 1737, 1065, 18, 171, "Input"],
Cell[60762, 1757, 233, 4, 43, "Input"],
Cell[60998, 1763, 1378, 23, 219, "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[62425, 1792, 60, 0, 39, "Section"],

Cell[CellGroupData[{
Cell[62510, 1796, 156, 4, 49, "Subsection"],
Cell[62669, 1802, 561, 10, 159, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[63267, 1817, 95, 3, 49, "Subsection"],
Cell[63365, 1822, 566, 13, 64, "Text"],
Cell[63934, 1837, 826, 14, 191, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[64797, 1856, 72, 0, 49, "Subsection"],
Cell[64872, 1858, 670, 14, 64, "Text"],
Cell[65545, 1874, 876, 15, 171, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[66458, 1894, 51, 0, 46, "Subsection"],
Cell[66512, 1896, 105, 2, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[66654, 1903, 62, 0, 46, "Subsection"],
Cell[66719, 1905, 618, 13, 63, "Text"],
Cell[67340, 1920, 606, 10, 139, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[67983, 1935, 51, 0, 46, "Subsection"],
Cell[68037, 1937, 107, 2, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[68181, 1944, 67, 0, 46, "Subsection"],
Cell[68251, 1946, 510, 8, 107, "Input"],
Cell[68764, 1956, 124, 3, 30, "Text"],
Cell[68891, 1961, 935, 15, 187, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[69863, 1981, 51, 0, 46, "Subsection"],
Cell[69917, 1983, 88, 1, 27, "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[70054, 1990, 50, 0, 39, "Section"],

Cell[CellGroupData[{
Cell[70129, 1994, 39, 0, 49, "Subsection"],
Cell[70171, 1996, 1060, 20, 207, "Input"],
Cell[71234, 2018, 622, 11, 159, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[71893, 2034, 102, 4, 49, "Subsection"],
Cell[71998, 2040, 552, 10, 127, "Input"],
Cell[72553, 2052, 925, 15, 159, "Input"],
Cell[73481, 2069, 477, 8, 107, "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[74007, 2083, 49, 0, 39, "Section"],

Cell[CellGroupData[{
Cell[74081, 2087, 100, 3, 49, "Subsection"],
Cell[74184, 2092, 652, 11, 191, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[74873, 2108, 63, 0, 49, "Subsection"],
Cell[74939, 2110, 539, 9, 112, "Text"],
Cell[75481, 2121, 1194, 19, 207, "Input"],
Cell[76678, 2142, 1159, 19, 207, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[77874, 2166, 42, 0, 46, "Subsection"],

Cell[CellGroupData[{
Cell[77941, 2170, 49, 0, 42, "Subsubsection"],
Cell[77993, 2172, 2291, 34, 264, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[80321, 2211, 46, 0, 42, "Subsubsection"],
Cell[80370, 2213, 784, 13, 139, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[81191, 2231, 44, 0, 42, "Subsubsection"],
Cell[81238, 2233, 142, 3, 30, "Text"],
Cell[81383, 2238, 186, 3, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[81606, 2246, 72, 0, 42, "Subsubsection"],
Cell[81681, 2248, 127, 3, 30, "Text"],
Cell[81811, 2253, 217, 4, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[82065, 2262, 44, 0, 42, "Subsubsection"],
Cell[82112, 2264, 1073, 19, 187, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[83222, 2288, 62, 0, 42, "Subsubsection"],
Cell[83287, 2290, 633, 10, 139, "Input"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[83981, 2307, 46, 0, 39, "Section"],

Cell[CellGroupData[{
Cell[84052, 2311, 103, 3, 49, "Subsection"],

Cell[CellGroupData[{
Cell[84180, 2318, 82, 1, 43, "Subsubsection"],
Cell[84265, 2321, 511, 9, 143, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[84825, 2336, 77, 0, 49, "Subsection"],
Cell[84905, 2338, 86, 1, 47, "Input"],
Cell[84994, 2341, 380, 6, 127, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[85411, 2352, 45, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[85481, 2356, 57, 0, 43, "Subsubsection"],
Cell[85541, 2358, 374, 7, 111, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[85952, 2370, 58, 0, 43, "Subsubsection"],
Cell[86013, 2372, 354, 5, 127, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[86404, 2382, 56, 0, 43, "Subsubsection"],
Cell[86463, 2384, 331, 5, 127, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[86831, 2394, 62, 0, 43, "Subsubsection"],
Cell[86896, 2396, 342, 5, 127, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[87275, 2406, 55, 0, 43, "Subsubsection"],
Cell[87333, 2408, 327, 5, 127, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[87709, 2419, 70, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[87804, 2423, 74, 0, 43, "Subsubsection"],
Cell[87881, 2425, 363, 5, 111, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[88281, 2435, 52, 0, 43, "Subsubsection"],
Cell[88336, 2437, 383, 6, 127, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[88756, 2448, 50, 0, 43, "Subsubsection"],
Cell[88809, 2450, 383, 6, 127, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[89241, 2462, 68, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[89334, 2466, 47, 0, 43, "Subsubsection"],
Cell[89384, 2468, 1140, 19, 287, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[90561, 2492, 48, 0, 43, "Subsubsection"],
Cell[90612, 2494, 650, 12, 191, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[91311, 2512, 99, 3, 49, "Subsection"],

Cell[CellGroupData[{
Cell[91435, 2519, 99, 3, 43, "Subsubsection"],
Cell[91537, 2524, 555, 9, 175, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[92129, 2538, 62, 0, 43, "Subsubsection"],
Cell[92194, 2540, 764, 14, 207, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[93007, 2560, 95, 3, 49, "Subsection"],

Cell[CellGroupData[{
Cell[93127, 2567, 56, 0, 43, "Subsubsection"],
Cell[93186, 2569, 734, 12, 191, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[93957, 2586, 62, 0, 43, "Subsubsection"],
Cell[94022, 2588, 1427, 24, 303, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[95498, 2618, 54, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[95577, 2622, 47, 0, 43, "Subsubsection"],
Cell[95627, 2624, 371, 5, 127, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[96035, 2634, 62, 0, 43, "Subsubsection"],
Cell[96100, 2636, 1337, 22, 303, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[97474, 2663, 48, 0, 43, "Subsubsection"],
Cell[97525, 2665, 1049, 16, 399, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[98611, 2686, 54, 0, 43, "Subsubsection"],
Cell[98668, 2688, 1202, 19, 351, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[99919, 2713, 40, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[99984, 2717, 62, 0, 43, "Subsubsection"],
Cell[100049, 2719, 523, 9, 175, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[100609, 2733, 59, 0, 43, "Subsubsection"],
Cell[100671, 2735, 312, 5, 111, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[101020, 2745, 59, 0, 43, "Subsubsection"],
Cell[101082, 2747, 319, 5, 127, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[101438, 2757, 71, 0, 43, "Subsubsection"],
Cell[101512, 2759, 840, 20, 82, "Text"],
Cell[102355, 2781, 347, 5, 159, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[102739, 2791, 40, 0, 43, "Subsubsection"],
Cell[102782, 2793, 1321, 22, 367, "Input"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

