(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    139035,       5591]*)
(*NotebookOutlinePosition[    140312,       5630]*)
(*  CellTagsIndexPosition[    140132,       5622]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  StyleBox["C-code",
    FontSlant->"Italic"],
  " manipulator control experiment visualization"
}], "Title"],

Cell[CellGroupData[{

Cell["Local definitions", "Section"],

Cell[CellGroupData[{

Cell["General definitions", "Subsection"],

Cell[CellGroupData[{

Cell["Turn off annoying warnings messages", "Subsubsection"],

Cell[BoxData[{
    \(\(Off[General::spell1];\)\), "\n", 
    \(\(Off[General::spell];\)\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Load in necessary packages", "Subsubsection"],

Cell[BoxData[{
    \(\(<< LinearAlgebra`MatrixManipulation`;\)\), "\n", 
    \(\(<< Graphics`Colors`;\)\), "\[IndentingNewLine]", 
    \(\(<< Graphics`Graphics3D`;\)\)}], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Visualization definitions", "Subsection"],

Cell[CellGroupData[{

Cell["Plot colors/styles", "Subsubsection"],

Cell[BoxData[{
    \(\(colors\  = \ {Red, \ Blue, \ Magenta, \ 
          Green};\)\), "\[IndentingNewLine]", 
    \(\(actualStyle\  = \ \({#} &\)\  /@ \ 
          colors;\)\), "\[IndentingNewLine]", 
    \(\(desiredStyle\  = \ \({dsh, #} &\)\  /@ \ colors;\)\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Short cuts", "Subsubsection"],

Cell[BoxData[{
    \(\(nS = DisplayFunction \[Rule] Identity;\)\), "\[IndentingNewLine]", 
    \(\(yS = 
        DisplayFunction \[Rule] $DisplayFunction;\)\), "\[IndentingNewLine]", \

    \(\(dsh\  = \ Dashing[{0.015, 0.015}];\)\), "\[IndentingNewLine]", 
    \(\(imgSize\  = \ ImageSize \[Rule] 288;\)\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Combine interpolation functions", "Subsubsection"],

Cell[BoxData[
    \(\(CombineInterpolationFunctions[traj1_, traj2_, {tmin_, tmax_}, 
          fnc_] := \[IndentingNewLine]Module[{f1, 
            f2}, \[IndentingNewLine]f1\  = \ \(#[t] &\)\  /@ \ 
              traj1; \[IndentingNewLine]f2\  = \ \(#[t] &\)\  /@ \ 
              traj2; \[IndentingNewLine]\(FunctionInterpolation[#, {t, tmin, 
                  tmax}] &\)\  /@ \ 
            fnc[f1, f2]\[IndentingNewLine]];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Plot a multiple dimensional trajectory (from a list of \
interpolation)\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(PlotMultipleTrajectory[traj_, {tmin_, tmax_}, plStyleList_, 
          opts___] := \[IndentingNewLine]Plot[
          Evaluate[\(#[t] &\)\  /@ \ traj], {t, \ tmin, tmax}, 
          PlotStyle \[Rule] plStyleList, Frame \[Rule] True, 
          PlotRange \[Rule] All, opts];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Empty DH list", "Subsubsection"],

Cell[BoxData[
    \(\(DH\  = \ Table[\(,\)\({3}\)];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Robot #1 (2-link, point-mass assumption)", "Subsubsection"],

Cell[BoxData[
    \(\(DH[\([1]\)] = \ {{0, 0, 0, \[Theta]1, 0}, {0, L1, \ 0, \ \[Theta]2, 
                0}, \ {0, \ L2, \ 0, \ 0, 0}}\  /. \ {L1 \[Rule] 1, 
              L2 \[Rule] 1/2}\  // \ N;\)\)], "Input",
  CellFrame->False]
}, Closed]],

Cell[CellGroupData[{

Cell["Robot #2 (2-link, cylindrical link assumption)", "Subsubsection"],

Cell[BoxData[
    \(\(\(DH[\([2]\)] = \ {{0, 0, 0, \[Theta]1, 0}, {0, L1, \ 0, \ \[Theta]2, 
                0}, \ {0, \ L2, \ 0, \ 0, 0}}\  /. \ {L1 \[Rule] 1, 
              L2 \[Rule] 1/2}\  // \ N;\)\(\ \)\)\)], "Input",
  CellFrame->False]
}, Closed]],

Cell[CellGroupData[{

Cell["Robot #3 (4-link, point-mass assumption)", "Subsubsection"],

Cell[BoxData[
    \(\(DH[\([3]\)] = \ {{0, 0, 0, \[Theta]1, 0}, {\[Pi]/2, 0, \ 
                0, \ \[Theta]2, 0}, \ {0, \ L1, \ 0, \[Theta]3, 0}, {0, L2, 
                0, \[Theta]4, 0}, {0, L3, 0, 0, 0}}\  /. {L1 \[Rule] 1, \ 
              L2 \[Rule] 3/4, L3 \[Rule] 1/2}\  // \ N;\)\)], "Input",
  CellFrame->False]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Elementary rotation and transform definitions", "Subsection"],

Cell[CellGroupData[{

Cell["Definition of elementary rotation matrices", "Subsubsection"],

Cell[BoxData[{
    \(\(rX[
          a_] := {{1, 0, 0}, {0, \ Cos[a], \ \(-Sin[a]\)}, {0, Sin[a], \ 
            Cos[a]}};\)\), "\n", 
    \(\(rY[
          a_] := {{Cos[a], 0, Sin[a]}, {0, 1, 0}, \ {\(-Sin[a]\), 0, 
            Cos[a]}};\)\), "\n", 
    \(\(rZ[
          a_] := {{Cos[a], \ \(-Sin[a]\), 0}, {Sin[a], Cos[a], 0}, {0, 0, 
            1}};\)\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Convert a 3x3 rotation matrix and a 3x1 position vector to a 4x4 \
homogeneous operator\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(RAndPToT[r_, \ p_] := \[IndentingNewLine]Join[
          Transpose[Join[Transpose[r], \ {p}]], {{0, 0, 0, 1}}];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Convert a 3x3 rotation matrix to 4x4 homogeneous rotational \
operator\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(RToT[r_] := RAndPToT[r, \ {0, 0, 0}];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Elementary homogeneous rotational operators", "Subsubsection"],

Cell[BoxData[{
    \(\(RX[a_] := RToT[rX[a]];\)\), "\n", 
    \(\(RY[a_] := RToT[rY[a]];\)\), "\n", 
    \(\(RZ[a_] := RToT[rZ[a]];\)\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Angle-axis rotation [",
  Cell[BoxData[
      \(TraditionalForm\`\(R\_k\)(\[Theta])\)]],
  "]"
}], "Subsubsection"],

Cell[BoxData[
    \(\(RK[\[Theta]_, \ 
          k_] := \[IndentingNewLine]Module[{kx, \ ky, kz, \ v, c, 
            s}, \[IndentingNewLine]{kx, \ ky, \ kz}\  = \ 
            k/Sqrt[k . k]; {{kx^2\ v\  + \ c, kx\ ky\ v\  - \ kz\ s, \ 
                kx\ kz\ v\  + \ ky\ s}, {kx\ ky\ v\  + \ kz\ s, \ 
                ky^2\ v + \ c, \ 
                ky\ kz\ v\  - \ kx\ s}, {kx\ kz\ v\  - \ ky\ s, \ 
                ky\ kz\ v\  + \ kx\ s, \ 
                kz^2\ v\  + \ c}}\  /. \ {v -> \((1 - Cos[\[Theta]])\), \ 
              c -> Cos[\[Theta]], \ s -> Sin[\[Theta]]}];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Convert a 3x1 position vector to a 4x4 homogeneous translational \
operator\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(PToT[p_] := RAndPToT[IdentityMatrix[3], \ p];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Elementary homogeneous translational operators", "Subsubsection"],

Cell[BoxData[{
    \(\(DX[x_] := PToT[{x, 0, 0}];\)\), "\n", 
    \(\(DY[y_] := PToT[{0, y, 0}];\)\), "\n", 
    \(\(DZ[z_] := PToT[{0, 0, z}];\)\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Extract 3x3 rotation matrix from 4x4 homogeneous transform", \
"Subsubsection"],

Cell[BoxData[
    \(\(RFromT[t_] := TakeMatrix[t, {1, 1}, {3, 3}];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Extract 3x1 position vector from 4x4 homogeneous transform", \
"Subsubsection"],

Cell[BoxData[
    \(\(PFromT[t_] := 
        TakeMatrix[t, {1, 4}, {3, 4}]\  // \ Flatten;\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Compute inverse of a 4x4 homogeneous transform", "Subsubsection"],

Cell[BoxData[
    \(\(InverseT[
          t_] := \[IndentingNewLine]Module[{r, \ 
            p}, \[IndentingNewLine]r\  = \ 
            RFromT[t]; \[IndentingNewLine]p\  = \ 
            PFromT[t]; \[IndentingNewLine]RAndPToT[
            Transpose[
              r], \ \(-Transpose[r] . p\)]\[IndentingNewLine]];\)\)], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Static forward kinematic definitions", "Subsection"],

Cell[TextData[{
  "In this notebook, DH parameters should be speficied as table, with one \
entry per link in the form ",
  Cell[BoxData[
      \(TraditionalForm\`{\[Alpha]\_\(i - 1\)\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`a\_\(i - 1\)\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`d\_i\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\[Theta]\_i\)]],
  ", type}, where \"type\" is 0 for revolute joints and 1 for prismatic \
joints."
}], "Text"],

Cell[CellGroupData[{

Cell["Degree to radian conversion (for DH parameter list)", "Subsubsection"],

Cell[BoxData[
    \(\(ConvertFromDegreesOne[{\[Alpha]_, \ a_, \ d_, \ \[Theta]_, \ 
            type___}] := {If[NumericQ[\[Alpha]], \ \[Alpha]\ Degree, \ 
              alpha], \ a, \ d, \ 
            If[NumericQ[\[Theta]], \ \[Theta]\ Degree, \ \[Theta]], \ 
            type}\  // \ FullSimplify;\)\)], "Input"],

Cell[BoxData[
    \(\(ConvertFromDegrees[l_] := 
        ConvertFromDegreesOne\  /@ \ l;\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Basic one link homogeneous transform (based on DH parameters)\
\>", \
"Subsubsection"],

Cell[BoxData[
    \(\(TLink[{\[Alpha]_, \ a_, \ \ d_, \[Theta]_, type___}] := 
        RX[\[Alpha]] . DX[a] . RZ[\[Theta]] . DZ[d];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Compound transform for multiple links (for DH parameter list)\
\>", \
"Subsubsection"],

Cell[BoxData[
    \(\(TAll[l_] := \ Dot\  @@ \((TLink\  /@ \ l)\);\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Homogeneous transform for robot  from frame {j} to frame {i}  ",
  "(",
  Cell[BoxData[
      \(TraditionalForm\`\[InvisiblePrefixScriptBase]\)]],
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^j\)\(\(\
\[InvisiblePrefixScriptBase]\_i\)\(\[InvisiblePrefixScriptBase]T\)\)\)]],
  ")"
}], "Subsubsection"],

Cell[BoxData[
    \(\(T[dh_, \ i_, \ 
          j_] := \[IndentingNewLine]Which[\[IndentingNewLine]i\  == \ j, 
          IdentityMatrix[4], \[IndentingNewLine]i\  < \ j, 
          TAll[Take[dh, \ {i + 1, \ j}]], \[IndentingNewLine]i\  > \ j, 
          InverseT[
            TAll[Take[dh, {j + 1, i}]]]\[IndentingNewLine]];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Rotation matrix for robot from frame {j} to frame {i} (",
  Cell[BoxData[
      \(TraditionalForm\`\[InvisiblePrefixScriptBase]\)]],
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^j\)\(\(\
\[InvisiblePrefixScriptBase]\_i\)\(\[InvisiblePrefixScriptBase]R\)\)\)]],
  ")"
}], "Subsubsection"],

Cell[BoxData[
    \(\(R[dh_, i_, j_] := RFromT[T[dh, \ i, \ j]];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Position vector for robot from frame {j} to frame {i} (",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^i\)\(P\_\(j, 
            ORG\)\)\)]],
  ")"
}], "Subsubsection"],

Cell[BoxData[
    \(\(P[dh_, i_, j_] := PFromT[T[dh, \ i, \ j]];\)\)], "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Animation functions", "Subsection"],

Cell[BoxData[
    \(\(ExtractVariables[
          dh_] := \[IndentingNewLine]Module[{vars}, \[IndentingNewLine]vars\  \
= \ \(If[#[\([5]\)]\  == \ 0, \ #[\([4]\)], \ #[\([3]\)]] &\)\  /@ \ 
              dh; \(Take[vars, \ #] &\)\  /@ \ 
              Position[NumberQ\  /@ \ vars, \ False]\  // \ 
            Flatten\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(ConvertTrajToGraphics[
          l_] := \[IndentingNewLine]Join[{{Thickness[ .015], \ 
              RGBColor[0, 0, 1], \ 
              Line[l]}}, \({PointSize[ .02], \ RGBColor[1, 0, 0], \ 
                Point[#]} &\)\  /@ \ l];\)\)], "Input"],

Cell[BoxData[
    \(\(ExtractMaximalPlotRange[ranges_, \ 
          l_:  2] := \[IndentingNewLine]Module[{r2}, \[IndentingNewLine]If[
            l\  == \ 2, 
            r2\  = \ Transpose[
                Partition[Flatten[ranges], 4]]; \[IndentingNewLine]{{Min[
                  r2[\([1]\)]], \ Max[r2[\([2]\)]]}, \ {Min[r2[\([3]\)]], \ 
                Max[r2[\([4]\)]]}}, 
            r2\  = \ Transpose[
                Partition[Flatten[ranges], 6]]; {{Min[r2[\([1]\)]], \ 
                Max[r2[\([2]\)]]}, {Min[r2[\([3]\)]], \ 
                Max[r2[\([4]\)]]}, \ {Min[r2[\([5]\)]], \ 
                Max[r2[\([6]\)]]}}]\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(AnimateGraphicsList[traj_, \ 
          l_:  2] := \[IndentingNewLine]Module[{f, \ g, \ 
            plrng}, \[IndentingNewLine]If[l\  == \ 2, \ f\  = \ Graphics, \ 
            f = Graphics3D\ ]; 
          g\  = \ \(Show[f[#], \ DisplayFunction -> Identity, \ 
                  PlotRange -> All] &\)\  /@ \ 
              traj; \[IndentingNewLine]plrng\  = \ 
            ExtractMaximalPlotRange[\(FullOptions[#, \ PlotRange] &\)\  /@ \ 
                g, l]; \[IndentingNewLine]If[
            l\  \[Equal] \ 
              2, \[IndentingNewLine]\(Show[f[#], \ PlotRange -> plrng, 
                  Frame \[Rule] True, \ AspectRatio -> Automatic] &\)\  /@ \ 
              traj, \[IndentingNewLine]\(Show[f[#], \ PlotRange -> plrng, \ 
                  BoxRatios -> Automatic] &\)\  /@ \ 
              traj]\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(AnimateTrajectory[dh_, \ traj_, \ l_:  2, \ n_:  20, \ 
          tfin_:  False] := 
        Module[{nF, \ joints, \ p0, \ ptake, \ vars, \ rul, \ traj2, \ 
            tf}, \[IndentingNewLine]If[l\  == \ 2, \ p0\  = \ {0, 0}; \ 
            ptake\  = \ 2, \ p0\  = \ {0, 0, 0}; \ 
            ptake\  = \ 3]; \[IndentingNewLine]vars\  = \ 
            ExtractVariables[dh]; \[IndentingNewLine]nF\  = \ 
            Length[dh]; \[IndentingNewLine]joints\  = \ 
            Join[{p0}, 
              Table[Take[P[dh, \ 0, \ i]\  // \ Chop, \ {1, ptake}], {i, 1, \ 
                  nF}]]; \[IndentingNewLine]rul\  = \(\((#[\([1]\)]\  -> \ \
\(#[\([2]\)]\)[t])\) &\)\  /@ \ \(({vars, \ traj}\  // \ Transpose)\); 
          traj2\  = \ joints\  /. \ rul; \[IndentingNewLine]If[
            tfin\  == \ False, tf\  = \ traj[\([1, 1, 1, 2]\)], \ 
            tf\  = \ tfin, \ tf = tfin]; 
          AnimateGraphicsList[
            Table[ConvertTrajToGraphics[traj2], \ {t, \ 0, \ tf, \ tf/n}], 
            l]\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(ConvertTrajToGraphics2[l_, \ c1_, 
          c2_] := \[IndentingNewLine]Join[{{Thickness[ .015], \ c1, \ 
              Line[l]}}, \({PointSize[ .02], \ c2, \ Point[#]} &\)\  /@ \ 
            l];\)\)], "Input"],

Cell[BoxData[
    \(\(AnimateTwoTrajectories[dh_, \ traj1_, \ traj2_, l_:  2, \ n_:  20, \ 
          tfin_:  False] := 
        Module[{nF, \ joints, \ p0, \ ptake, \ vars, \ rul1, rul2, \ traj1b, 
            traj2b, \ tf}, \[IndentingNewLine]If[l\  == \ 2, \ 
            p0\  = \ {0, 0}; \ ptake\  = \ 2, \ p0\  = \ {0, 0, 0}; \ 
            ptake\  = \ 3]; \[IndentingNewLine]vars\  = \ 
            ExtractVariables[dh]; \[IndentingNewLine]nF\  = \ 
            Length[dh]; \[IndentingNewLine]joints\  = \ 
            Join[{p0}, 
              Table[Take[P[dh, \ 0, \ i]\  // \ Chop, \ {1, ptake}], {i, 1, \ 
                  nF}]]; \[IndentingNewLine]rul1\  = \(\((#[\([1]\)]\  -> \ \
\(#[\([2]\)]\)[t])\) &\)\  /@ \ \(({vars, \ traj1}\  // \ Transpose)\); 
          traj1b = \ 
            joints\  /. \ 
              rul1; \[IndentingNewLine]rul2\  = \(\((#[\([1]\)]\  -> \ \
\(#[\([2]\)]\)[t])\) &\)\  /@ \ \(({vars, \ traj2}\  // \ Transpose)\); 
          traj2b = \ joints\  /. \ rul2; \[IndentingNewLine]If[
            tfin\  == \ False, tf\  = \ traj1[\([1, 1, 1, 2]\)], \ 
            tf\  = \ tfin, \ tf = tfin]; 
          AnimateGraphicsList[
            Table[ConvertTrajToGraphics2[traj1b, Yellow, Green]~Join~
                ConvertTrajToGraphics2[traj2b, Blue, Red], \ {t, \ 0, \ tf, \ 
                tf/n}], l]\[IndentingNewLine]];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Read \"msim\" results file definition", "Subsection"],

Cell[BoxData[
    \(\(ReadResultsFile[file_, 
          n_:  4] := \[IndentingNewLine]Module[{res}, \[IndentingNewLine]res\ \
 = \ Partition[ReadList[file, Number], 3*n + 1]\  // \ 
              Transpose; \[IndentingNewLine]{res[\([1]\)]\  // \ Last}~Join~
            Partition[\(Interpolation[
                    Transpose[{res[\([1]\)], \ #}]] &\)\  /@ \ 
                Drop[res, \ 1], n]\[IndentingNewLine]];\)\)], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Control experiment visualization", "Section",
  CellTags->{"Experiment-specific user definitions", "Experiment"}],

Cell[CellGroupData[{

Cell["Read control results from file", "Subsection"],

Cell[CellGroupData[{

Cell["Set data file", "Subsubsection"],

Cell[BoxData[
    \(\(dataFile\  = \ \
"\</Users/nechyba/Desktop/hw4/manip/results.dat\>";\)\)], "Input",
  CellFrame->True,
  Background->GrayLevel[0.849989]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Read control parameters from file", "Subsubsection"],

Cell[BoxData[
    \(\({tmax, desiredOut, actual, torques} = 
        ReadResultsFile[dataFile];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualization of results", "Subsection"],

Cell[CellGroupData[{

Cell["Join trajectories", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(gdesired = 
        PlotMultipleTrajectory[desiredOut, {0, tmax}, desiredStyle, 
          nS];\)\), "\[IndentingNewLine]", 
    \(\(gactual = 
        PlotMultipleTrajectory[actual, {0, tmax}, actualStyle, 
          nS];\)\), "\[IndentingNewLine]", 
    \(\(Show[gdesired, gactual, yS, \ imgSize];\)\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 288 177.994 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0952381 0.323772 0.177204 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.21429 -0.0125 -3 -9 ]
[.21429 -0.0125 3 0 ]
[.40476 -0.0125 -3 -9 ]
[.40476 -0.0125 3 0 ]
[.59524 -0.0125 -3 -9 ]
[.59524 -0.0125 3 0 ]
[.78571 -0.0125 -3 -9 ]
[.78571 -0.0125 3 0 ]
[.97619 -0.0125 -6 -9 ]
[.97619 -0.0125 6 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .05797 -24 -4.5 ]
[-0.0125 .05797 0 4.5 ]
[-0.0125 .14657 -12 -4.5 ]
[-0.0125 .14657 0 4.5 ]
[-0.0125 .23517 -24 -4.5 ]
[-0.0125 .23517 0 4.5 ]
[-0.0125 .32377 -6 -4.5 ]
[-0.0125 .32377 0 4.5 ]
[-0.0125 .41237 -18 -4.5 ]
[-0.0125 .41237 0 4.5 ]
[-0.0125 .50098 -6 -4.5 ]
[-0.0125 .50098 0 4.5 ]
[-0.0125 .58958 -18 -4.5 ]
[-0.0125 .58958 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.21429 0 m
.21429 .00625 L
s
[(2)] .21429 -0.0125 0 1 Mshowa
.40476 0 m
.40476 .00625 L
s
[(4)] .40476 -0.0125 0 1 Mshowa
.59524 0 m
.59524 .00625 L
s
[(6)] .59524 -0.0125 0 1 Mshowa
.78571 0 m
.78571 .00625 L
s
[(8)] .78571 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(10)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.5 0 m
.5 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .05797 m
.00625 .05797 L
s
[(-1.5)] -0.0125 .05797 1 0 Mshowa
0 .14657 m
.00625 .14657 L
s
[(-1)] -0.0125 .14657 1 0 Mshowa
0 .23517 m
.00625 .23517 L
s
[(-0.5)] -0.0125 .23517 1 0 Mshowa
0 .32377 m
.00625 .32377 L
s
[(0)] -0.0125 .32377 1 0 Mshowa
0 .41237 m
.00625 .41237 L
s
[(0.5)] -0.0125 .41237 1 0 Mshowa
0 .50098 m
.00625 .50098 L
s
[(1)] -0.0125 .50098 1 0 Mshowa
0 .58958 m
.00625 .58958 L
s
[(1.5)] -0.0125 .58958 1 0 Mshowa
.125 Mabswid
0 .07569 m
.00375 .07569 L
s
0 .09341 m
.00375 .09341 L
s
0 .11113 m
.00375 .11113 L
s
0 .12885 m
.00375 .12885 L
s
0 .16429 m
.00375 .16429 L
s
0 .18201 m
.00375 .18201 L
s
0 .19973 m
.00375 .19973 L
s
0 .21745 m
.00375 .21745 L
s
0 .25289 m
.00375 .25289 L
s
0 .27061 m
.00375 .27061 L
s
0 .28833 m
.00375 .28833 L
s
0 .30605 m
.00375 .30605 L
s
0 .34149 m
.00375 .34149 L
s
0 .35921 m
.00375 .35921 L
s
0 .37693 m
.00375 .37693 L
s
0 .39465 m
.00375 .39465 L
s
0 .43009 m
.00375 .43009 L
s
0 .44781 m
.00375 .44781 L
s
0 .46554 m
.00375 .46554 L
s
0 .48326 m
.00375 .48326 L
s
0 .5187 m
.00375 .5187 L
s
0 .53642 m
.00375 .53642 L
s
0 .55414 m
.00375 .55414 L
s
0 .57186 m
.00375 .57186 L
s
0 .04024 m
.00375 .04024 L
s
0 .02252 m
.00375 .02252 L
s
0 .0048 m
.00375 .0048 L
s
0 .6073 m
.00375 .6073 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.21429 .61178 m
.21429 .61803 L
s
.40476 .61178 m
.40476 .61803 L
s
.59524 .61178 m
.59524 .61803 L
s
.78571 .61178 m
.78571 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.07143 .61428 m
.07143 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.16667 .61428 m
.16667 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.5 .61428 m
.5 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .05797 m
1 .05797 L
s
.99375 .14657 m
1 .14657 L
s
.99375 .23517 m
1 .23517 L
s
.99375 .32377 m
1 .32377 L
s
.99375 .41237 m
1 .41237 L
s
.99375 .50098 m
1 .50098 L
s
.99375 .58958 m
1 .58958 L
s
.125 Mabswid
.99625 .07569 m
1 .07569 L
s
.99625 .09341 m
1 .09341 L
s
.99625 .11113 m
1 .11113 L
s
.99625 .12885 m
1 .12885 L
s
.99625 .16429 m
1 .16429 L
s
.99625 .18201 m
1 .18201 L
s
.99625 .19973 m
1 .19973 L
s
.99625 .21745 m
1 .21745 L
s
.99625 .25289 m
1 .25289 L
s
.99625 .27061 m
1 .27061 L
s
.99625 .28833 m
1 .28833 L
s
.99625 .30605 m
1 .30605 L
s
.99625 .34149 m
1 .34149 L
s
.99625 .35921 m
1 .35921 L
s
.99625 .37693 m
1 .37693 L
s
.99625 .39465 m
1 .39465 L
s
.99625 .43009 m
1 .43009 L
s
.99625 .44781 m
1 .44781 L
s
.99625 .46554 m
1 .46554 L
s
.99625 .48326 m
1 .48326 L
s
.99625 .5187 m
1 .5187 L
s
.99625 .53642 m
1 .53642 L
s
.99625 .55414 m
1 .55414 L
s
.99625 .57186 m
1 .57186 L
s
.99625 .04024 m
1 .04024 L
s
.99625 .02252 m
1 .02252 L
s
.99625 .0048 m
1 .0048 L
s
.99625 .6073 m
1 .6073 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .32377 m
1 .32377 L
s
1 0 0 r
.5 Mabswid
[ .015 .015 ] 0 setdash
.02381 .32377 m
.02499 .32377 L
.02605 .32378 L
.02729 .32379 L
.02846 .32381 L
.03053 .32386 L
.03279 .32392 L
.03527 .32402 L
.0379 .32414 L
.04262 .32442 L
.05205 .32523 L
.06244 .32647 L
.07293 .32808 L
.0842 .33021 L
.10458 .33504 L
.1458 .34832 L
.18551 .36496 L
.22371 .38384 L
.26435 .40633 L
.30348 .42963 L
.34506 .45539 L
.38513 .48043 L
.42368 .50407 L
.46468 .52796 L
.50417 .54906 L
.54214 .56692 L
.58257 .58261 L
.6012 .58849 L
.62148 .5938 L
.63251 .59618 L
.64272 .59804 L
.65309 .5996 L
.66284 .60073 L
.67182 .60148 L
.67687 .60178 L
.67911 .60188 L
.6815 .60197 L
.6828 .60201 L
.68398 .60205 L
.68519 .60207 L
.68629 .60209 L
.68759 .60211 L
.68827 .60211 L
.68899 .60212 L
.6902 .60212 L
.69089 .60212 L
.69154 .60212 L
.69219 .60212 L
.6929 .60212 L
.6942 .60212 L
Mistroke
.69663 .60212 L
.7021 .60212 L
.7428 .60212 L
.78198 .60212 L
.82362 .60212 L
.86374 .60212 L
.90234 .60212 L
.94339 .60212 L
.97619 .60212 L
Mfstroke
0 0 1 r
.02381 .60212 m
.06244 .60212 L
.10458 .60212 L
.14415 .60212 L
.18221 .60212 L
.22272 .60212 L
.26171 .60212 L
.30316 .60212 L
.34309 .60212 L
.3815 .60212 L
.42237 .60212 L
.46172 .60212 L
.49955 .60212 L
.53984 .60212 L
.57861 .60212 L
.61984 .60212 L
.65954 .60212 L
.69774 .60212 L
.73838 .60212 L
.77751 .60212 L
.81909 .60212 L
.85916 .60212 L
.89771 .60212 L
.93871 .60212 L
.97619 .60212 L
s
1 0 1 r
.02381 .04542 m
.06244 .04542 L
.10458 .04542 L
.14415 .04542 L
.18221 .04542 L
.22272 .04542 L
.26171 .04542 L
.30316 .04542 L
.34309 .04542 L
.3815 .04542 L
.42237 .04542 L
.46172 .04542 L
.49955 .04542 L
.53984 .04542 L
.57861 .04542 L
.61984 .04542 L
.65954 .04542 L
.69774 .04542 L
.73838 .04542 L
.77751 .04542 L
.81909 .04542 L
.85916 .04542 L
.89771 .04542 L
.93871 .04542 L
.97619 .04542 L
s
0 1 0 r
.02381 .04542 m
.06244 .04542 L
.10458 .04542 L
.14415 .04542 L
.18221 .04542 L
.22272 .04542 L
.26171 .04542 L
.30316 .04542 L
.34309 .04542 L
.3815 .04542 L
.42237 .04542 L
.46172 .04542 L
.49955 .04542 L
.53984 .04542 L
.57861 .04542 L
.61984 .04542 L
.65954 .04542 L
.69774 .04542 L
.73838 .04542 L
.77751 .04542 L
.81909 .04542 L
.85916 .04542 L
.89771 .04542 L
.93871 .04542 L
.97619 .04542 L
s
1 0 0 r
[ ] 0 setdash
.02381 .32377 m
.02499 .32377 L
.02605 .32377 L
.02846 .32375 L
.02952 .32375 L
.03068 .32375 L
.03178 .32375 L
.03279 .32375 L
.03396 .32375 L
.03506 .32376 L
.03626 .32378 L
.03756 .3238 L
.03891 .32383 L
.0402 .32386 L
.04262 .32395 L
.045 .32406 L
.04753 .3242 L
.0521 .32452 L
.06244 .32556 L
.07382 .32716 L
.08454 .32909 L
.10458 .33377 L
.14313 .34634 L
.18413 .36358 L
.22362 .38313 L
.26159 .40421 L
.30202 .42839 L
.34093 .45264 L
.38229 .4787 L
.42213 .50335 L
.46046 .52598 L
.50124 .54814 L
.54051 .56693 L
.57826 .58198 L
.59923 .58884 L
.61846 .59408 L
.63731 .59815 L
.6469 .59979 L
.65715 .60122 L
.66247 .60182 L
.66749 .60229 L
.67238 .60267 L
.67689 .60294 L
.68117 .60313 L
.68333 .60321 L
.68571 .60326 L
.68695 .60329 L
.68809 .6033 L
.6894 .60331 L
.69064 .60332 L
Mistroke
.69179 .60332 L
.69285 .60331 L
.69402 .60331 L
.69526 .60329 L
.69787 .60325 L
.70063 .6032 L
.70557 .60309 L
.71134 .60295 L
.71671 .60284 L
.72175 .60275 L
.72634 .60269 L
.73144 .60264 L
.73424 .60262 L
.73689 .6026 L
.73927 .6026 L
.7415 .60259 L
.74277 .60259 L
.74394 .60258 L
.74523 .60258 L
.74587 .60258 L
.74657 .60258 L
.74727 .60258 L
.74792 .60258 L
.74915 .60258 L
.75047 .60258 L
.75121 .60258 L
.7519 .60258 L
.75304 .60258 L
.7543 .60258 L
.75562 .60258 L
.75684 .60258 L
.76567 .60259 L
.77528 .60261 L
.78069 .60261 L
.78308 .60261 L
.78571 .60261 L
.79051 .60262 L
.79313 .60262 L
.79431 .60262 L
.7956 .60262 L
.79678 .60262 L
.79787 .60262 L
.79912 .60262 L
.80026 .60262 L
.80132 .60262 L
.80232 .60262 L
.80338 .60262 L
.80453 .60262 L
.80568 .60262 L
.80696 .60262 L
Mistroke
.8076 .60262 L
.80829 .60262 L
.80953 .60262 L
.81077 .60262 L
.81195 .60262 L
.813 .60262 L
.81415 .60262 L
.82514 .60262 L
.83053 .60262 L
.83289 .60262 L
.83414 .60262 L
.83485 .60262 L
.83549 .60262 L
.83669 .60262 L
.83782 .60262 L
.83887 .60262 L
.84 .60262 L
.84477 .60262 L
.84608 .60262 L
.84731 .60262 L
.84858 .60262 L
.84926 .60262 L
.84999 .60262 L
.85122 .60262 L
.85251 .60262 L
.85482 .60262 L
.89397 .60262 L
.9321 .60262 L
.97268 .60262 L
.97619 .60262 L
Mfstroke
0 0 1 r
.02381 .60212 m
.06244 .57288 L
.07216 .56283 L
.08255 .55457 L
.08852 .55119 L
.09396 .54894 L
.09658 .54813 L
.09942 .54742 L
.1021 .54692 L
.1034 .54673 L
.10458 .54658 L
.10567 .54647 L
.10685 .54637 L
.10811 .54629 L
.1093 .54624 L
.11041 .54621 L
.11159 .5462 L
.11271 .54621 L
.11372 .54623 L
.11502 .54627 L
.11621 .54633 L
.1189 .54652 L
.12128 .54675 L
.12379 .54703 L
.14415 .55023 L
.15363 .55172 L
.16379 .55301 L
.16871 .5535 L
.17338 .55389 L
.17789 .55418 L
.18212 .55439 L
.18671 .55456 L
.18915 .55462 L
.1904 .55464 L
.19178 .55467 L
.19304 .55469 L
.19438 .5547 L
.19513 .55471 L
.1958 .55471 L
.19713 .55472 L
.19834 .55472 L
.19965 .55472 L
.20089 .55472 L
.20203 .55472 L
.20309 .55471 L
.20423 .5547 L
.20629 .55468 L
.21098 .55462 L
.22045 .55443 L
.2402 .554 L
Mistroke
.25137 .5538 L
.25671 .55373 L
.26173 .55367 L
.26659 .55363 L
.26914 .55361 L
.27184 .55359 L
.27411 .55358 L
.27661 .55357 L
.27886 .55357 L
.28099 .55356 L
.28227 .55356 L
.28367 .55356 L
.28494 .55355 L
.28611 .55355 L
.28738 .55355 L
.2881 .55355 L
.28877 .55355 L
.28948 .55355 L
.29013 .55355 L
.29085 .55355 L
.2916 .55355 L
.29278 .55355 L
.29405 .55355 L
.29524 .55355 L
.29633 .55356 L
.30149 .55356 L
.30636 .55357 L
.3115 .55359 L
.32071 .55361 L
.33075 .55363 L
.33628 .55364 L
.34152 .55365 L
.34643 .55366 L
.35178 .55367 L
.35664 .55367 L
.35896 .55367 L
.36114 .55367 L
.36334 .55367 L
.36536 .55367 L
.3676 .55367 L
.36878 .55367 L
.37002 .55367 L
.37115 .55367 L
.37235 .55367 L
.37349 .55367 L
.37453 .55367 L
.37575 .55367 L
.37707 .55367 L
.37831 .55367 L
.37946 .55367 L
Mistroke
.38071 .55367 L
.3819 .55367 L
.38413 .55367 L
.38914 .55367 L
.3982 .55367 L
.40819 .55367 L
.41367 .55367 L
.41883 .55366 L
.42383 .55366 L
.4252 .55366 L
.42595 .55366 L
.42665 .55366 L
.42788 .55366 L
.42853 .55366 L
.42923 .55366 L
.43045 .55366 L
.43177 .55366 L
.43301 .55366 L
.43416 .55366 L
.4352 .55366 L
.43634 .55366 L
.43754 .55366 L
.43866 .55366 L
.43974 .55366 L
.44089 .55366 L
.44198 .55366 L
.44297 .55366 L
.44409 .55366 L
.44514 .55366 L
.44751 .55366 L
.45249 .55367 L
.45705 .55367 L
.47708 .55367 L
.49553 .55368 L
.53496 .55371 L
.57286 .55373 L
.61322 .55377 L
.65207 .5538 L
.67183 .55382 L
.68226 .55382 L
.69336 .55383 L
.69895 .55383 L
.7042 .55383 L
.70932 .55384 L
.71401 .55384 L
.72321 .55384 L
.72439 .55384 L
.72568 .55384 L
.72689 .55384 L
.72801 .55384 L
Mistroke
.72932 .55384 L
.72997 .55384 L
.7307 .55384 L
.73196 .55384 L
.73315 .55384 L
.73785 .55384 L
.74045 .55384 L
.74173 .55384 L
.74292 .55384 L
.74519 .55384 L
.74767 .55384 L
.74869 .55384 L
.74976 .55384 L
.75094 .55384 L
.75202 .55384 L
.75331 .55384 L
.75404 .55384 L
.75472 .55384 L
.75727 .55384 L
.76203 .55384 L
.76718 .55384 L
.76867 .55384 L
.77007 .55384 L
.77133 .55384 L
.77205 .55384 L
.7727 .55384 L
.77388 .55384 L
.77512 .55384 L
.77618 .55384 L
.77733 .55384 L
.77858 .55384 L
.77992 .55384 L
.78118 .55384 L
.78233 .55384 L
.78349 .55384 L
.78455 .55384 L
.78573 .55384 L
.78697 .55384 L
.78798 .55384 L
.78906 .55384 L
.79025 .55384 L
.79136 .55384 L
.79202 .55384 L
.79273 .55384 L
.794 .55384 L
.79548 .55384 L
.79621 .55384 L
.79688 .55384 L
.7982 .55384 L
.7989 .55384 L
Mistroke
.79963 .55384 L
.80082 .55384 L
.80212 .55384 L
.80341 .55384 L
.80458 .55384 L
.80584 .55384 L
.80651 .55384 L
.80721 .55384 L
.80848 .55384 L
.80965 .55384 L
.81075 .55384 L
.81196 .55384 L
.81689 .55384 L
.81937 .55384 L
.82004 .55384 L
.82077 .55384 L
.82207 .55384 L
.82319 .55384 L
.82423 .55384 L
.82532 .55384 L
.82652 .55384 L
.82903 .55384 L
.83134 .55384 L
.83696 .55384 L
.84227 .55384 L
.8522 .55384 L
.89107 .55384 L
.90122 .55384 L
.90657 .55384 L
.90951 .55384 L
.91095 .55384 L
.91227 .55384 L
.91343 .55384 L
.91466 .55384 L
.91589 .55384 L
.91658 .55384 L
.91723 .55384 L
.92271 .55384 L
.93238 .55384 L
.97138 .55384 L
.97619 .55384 L
Mfstroke
1 0 1 r
.02381 .04542 m
.04262 .03003 L
.04754 .02574 L
.0522 .0224 L
.05756 .01944 L
.06244 .01751 L
.06504 .01674 L
.06744 .01616 L
.07007 .01567 L
.07286 .01528 L
.074 .01515 L
.07524 .01504 L
.07747 .01488 L
.07883 .01481 L
.08013 .01477 L
.08129 .01474 L
.08255 .01472 L
.08327 .01472 L
.08405 .01472 L
.08478 .01472 L
.08545 .01473 L
.08673 .01475 L
.08738 .01476 L
.08808 .01478 L
.09113 .01488 L
.09398 .01501 L
.10458 .01567 L
.12322 .01691 L
.12832 .01719 L
.13375 .01743 L
.13838 .0176 L
.14349 .01774 L
.14624 .0178 L
.14876 .01784 L
.15153 .01787 L
.15292 .01789 L
.15444 .0179 L
.15579 .01791 L
.15702 .01791 L
.1577 .01791 L
.15843 .01792 L
.15913 .01792 L
.15977 .01792 L
.16092 .01792 L
.16218 .01792 L
.1635 .01791 L
.16473 .01791 L
.16597 .0179 L
.16733 .0179 L
.1698 .01788 L
Mistroke
.17443 .01784 L
.18485 .01773 L
.19585 .0176 L
.20621 .01749 L
.2155 .01742 L
.22074 .01739 L
.22555 .01737 L
.22782 .01737 L
.23025 .01736 L
.23236 .01735 L
.23464 .01735 L
.23604 .01735 L
.23732 .01735 L
.23853 .01735 L
.23986 .01734 L
.24105 .01734 L
.24217 .01734 L
.24334 .01734 L
.24461 .01734 L
.24588 .01734 L
.24706 .01734 L
.24835 .01734 L
.24908 .01735 L
.24973 .01735 L
.25232 .01735 L
.25511 .01735 L
.26006 .01736 L
.26473 .01737 L
.28425 .0174 L
.2946 .01742 L
.3003 .01743 L
.3057 .01743 L
.31081 .01744 L
.31636 .01744 L
.32142 .01745 L
.32383 .01745 L
.32501 .01745 L
.3261 .01745 L
.32712 .01745 L
.32821 .01745 L
.33049 .01745 L
.333 .01745 L
.33533 .01745 L
.3376 .01745 L
.33887 .01745 L
.34002 .01745 L
.3412 .01745 L
.34247 .01745 L
.34319 .01745 L
.34385 .01745 L
Mistroke
.34516 .01745 L
.34634 .01745 L
.34764 .01745 L
.34886 .01745 L
.34999 .01745 L
.35512 .01745 L
.35957 .01745 L
.36085 .01745 L
.36206 .01745 L
.36314 .01745 L
.36432 .01745 L
.3851 .01745 L
.40467 .01744 L
.42613 .01744 L
.44657 .01744 L
.46564 .01743 L
.50456 .01742 L
.54594 .01739 L
.58581 .01737 L
.62416 .01735 L
.64356 .01734 L
.66496 .01733 L
.67472 .01732 L
.68523 .01732 L
.69007 .01732 L
.69515 .01732 L
.6995 .01732 L
.70196 .01732 L
.70425 .01732 L
.723 .01732 L
.74331 .01731 L
.744 .01731 L
.74466 .01731 L
.74541 .01731 L
.7461 .01731 L
.74735 .01731 L
.74806 .01731 L
.74871 .01731 L
.75003 .01731 L
.75077 .01731 L
.75144 .01731 L
.75218 .01731 L
.75298 .01731 L
.75374 .01731 L
.75443 .01731 L
.75566 .01731 L
.75681 .01731 L
.75807 .01731 L
.75877 .01731 L
.75942 .01731 L
Mistroke
.76071 .01731 L
.76144 .01731 L
.76212 .01731 L
.76336 .01731 L
.76467 .01731 L
.76706 .01731 L
.76963 .01731 L
.77087 .01731 L
.77205 .01731 L
.77311 .01731 L
.77426 .01731 L
.77498 .01731 L
.77565 .01731 L
.77692 .01731 L
.77762 .01731 L
.77827 .01731 L
.77974 .01731 L
.78483 .01731 L
.80524 .01731 L
.81521 .01731 L
.82431 .01731 L
.82881 .01731 L
.83119 .01731 L
.83371 .01731 L
.83477 .01731 L
.83591 .01731 L
.83716 .01731 L
.83834 .01731 L
.83943 .01731 L
.84057 .01731 L
.84262 .01731 L
.86227 .01731 L
.90036 .01731 L
.94091 .01731 L
.97619 .01731 L
Mfstroke
0 1 0 r
.02381 .04542 m
.02499 .04558 L
.02605 .04577 L
.02729 .04592 L
.02846 .046 L
.02954 .04605 L
.03053 .04607 L
.03163 .04607 L
.03279 .04606 L
.03395 .04603 L
.0352 .04599 L
.03746 .04591 L
.03876 .04587 L
.04014 .04583 L
.04144 .0458 L
.04262 .04578 L
.04392 .04577 L
.04516 .04577 L
.04585 .04577 L
.04649 .04578 L
.04719 .04579 L
.04793 .0458 L
.04917 .04583 L
.05033 .04587 L
.05297 .04599 L
.05754 .04628 L
.06244 .04668 L
.07281 .04763 L
.08408 .04856 L
.08924 .0489 L
.09473 .04919 L
.09941 .04939 L
.10458 .04955 L
.10719 .04961 L
.10994 .04966 L
.11228 .04969 L
.11485 .04971 L
.1163 .04972 L
.11762 .04973 L
.11835 .04973 L
.11914 .04974 L
.11989 .04974 L
.12058 .04974 L
.12181 .04974 L
.12251 .04974 L
.12317 .04974 L
.12449 .04974 L
.12523 .04973 L
.12591 .04973 L
.12846 .04972 L
Mistroke
.13083 .0497 L
.13619 .04965 L
.14594 .04953 L
.16478 .0493 L
.16994 .04924 L
.17538 .04919 L
.18008 .04916 L
.18512 .04913 L
.18789 .04912 L
.19042 .04911 L
.19299 .0491 L
.19536 .0491 L
.19812 .04909 L
.19958 .04909 L
.20112 .04909 L
.20246 .04909 L
.20372 .04908 L
.20443 .04908 L
.20509 .04908 L
.20581 .04908 L
.20657 .04908 L
.20784 .04908 L
.20899 .04908 L
.20963 .04908 L
.21032 .04909 L
.21157 .04909 L
.21275 .04909 L
.21399 .04909 L
.21622 .04909 L
.22124 .0491 L
.22676 .04911 L
.24788 .04915 L
.25788 .04917 L
.26715 .04918 L
.27178 .04918 L
.2767 .04918 L
.28165 .04919 L
.28446 .04919 L
.28711 .04919 L
.2883 .04919 L
.28942 .04919 L
.29058 .04919 L
.29185 .04919 L
.29409 .04919 L
.29519 .04919 L
.29621 .04919 L
.2974 .04919 L
.29869 .04919 L
.29979 .04919 L
.30095 .04919 L
Mistroke
.3022 .04919 L
.30337 .04919 L
.30465 .04919 L
.30537 .04919 L
.30603 .04919 L
.3108 .04919 L
.31588 .04919 L
.32068 .04919 L
.32503 .04919 L
.33582 .04919 L
.34575 .04918 L
.35512 .04918 L
.35984 .04918 L
.36502 .04918 L
.36568 .04918 L
.36641 .04918 L
.3671 .04918 L
.36774 .04918 L
.36894 .04918 L
.37022 .04918 L
.37087 .04918 L
.37158 .04918 L
.37287 .04918 L
.37361 .04918 L
.37439 .04918 L
.37579 .04918 L
.37699 .04918 L
.37828 .04918 L
.37949 .04918 L
.38059 .04918 L
.38189 .04918 L
.38329 .04918 L
.38584 .04918 L
.39612 .04918 L
.40573 .04918 L
.41057 .04918 L
.41196 .04918 L
.41328 .04918 L
.41446 .04918 L
.41576 .04918 L
.41651 .04918 L
.41721 .04918 L
.41853 .04918 L
.42007 .04918 L
.42148 .04918 L
.42272 .04918 L
.42403 .04918 L
.42534 .04918 L
.42608 .04918 L
.42678 .04918 L
Mistroke
.42805 .04918 L
.42869 .04918 L
.42939 .04918 L
.43064 .04918 L
.43181 .04918 L
.43251 .04918 L
.43318 .04918 L
.43444 .04918 L
.43514 .04918 L
.43579 .04918 L
.43651 .04918 L
.43727 .04918 L
.43855 .04918 L
.43928 .04918 L
.43995 .04918 L
.44117 .04918 L
.44247 .04918 L
.44363 .04918 L
.44472 .04918 L
.44591 .04918 L
.44719 .04918 L
.46576 .04918 L
.48699 .04918 L
.50711 .04917 L
.54543 .04917 L
.5862 .04916 L
.62545 .04915 L
.66319 .04915 L
.68253 .04915 L
.68782 .04914 L
.68854 .04914 L
.68922 .04914 L
.69 .04914 L
.69071 .04914 L
.692 .04914 L
.69267 .04914 L
.6934 .04914 L
.69461 .04914 L
.69588 .04914 L
.69706 .04914 L
.69817 .04914 L
.69948 .04914 L
.70068 .04914 L
.70134 .04914 L
.70206 .04914 L
.70338 .04914 L
.70403 .04914 L
.70471 .04914 L
.70594 .04914 L
.70726 .04914 L
Mistroke
.70801 .04914 L
.7087 .04914 L
.70997 .04914 L
.71113 .04914 L
.71176 .04914 L
.71246 .04914 L
.71372 .04914 L
.71494 .04914 L
.71608 .04914 L
.71727 .04914 L
.71858 .04914 L
.71971 .04914 L
.72075 .04914 L
.72308 .04914 L
.72431 .04914 L
.72561 .04914 L
.72685 .04914 L
.72797 .04914 L
.72929 .04914 L
.73003 .04914 L
.73072 .04914 L
.73198 .04914 L
.73331 .04914 L
.73843 .04914 L
.73984 .04914 L
.74134 .04914 L
.74207 .04914 L
.74277 .04914 L
.74406 .04914 L
.7452 .04914 L
.74642 .04914 L
.74758 .04914 L
.74864 .04914 L
.7537 .04914 L
.76394 .04914 L
.78235 .04914 L
.78708 .04914 L
.78967 .04914 L
.79095 .04914 L
.79213 .04914 L
.79324 .04914 L
.79442 .04914 L
.7956 .04914 L
.7969 .04914 L
.80124 .04914 L
.82193 .04914 L
.86222 .04914 L
.901 .04914 L
.94224 .04914 L
.97619 .04914 L
Mistroke
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00803oool2o`0002d0oooo1Ol0000_0?ooo`?o0000
;@3oool3o`0002l0oooo0ol0000[0?ooo`?o0000103oool2o`0000@0oooo000O0?ooo`04o`00003o
ool0ooooo`0002d0oooo00Co00000?ooo`3ooooo0000<03oool00ol00000oooo0?ooo`0[0?ooo`05
o`00003oool0oooo0?ooool00000;@3oool01Ol00000oooo0?ooo`3ooooo000002/0oooo00?o0000
0?ooo`3oool00P3oool01?l00000oooo0?ooool000030?ooo`007`3oool01?l00000oooo0?ooool0
000^0?ooo`03o`00003oool0oooo02d0oooo1Ol0000/0?ooo`05o`00003oool0oooo0?ooool00000
;@3oool01Ol00000oooo0?ooo`3ooooo000002/0oooo00?o00000?ooo`3oool00P3oool01?l00000
oooo0?ooool000030?ooo`007`3oool01?l00000oooo0?ooool0000_0?ooo`03o`00003oool0oooo
02`0oooo00Co00000?ooo`3ooooo0000;@3oool4o`0002l0oooo0ol0000/0?ooo`03o`00003oool0
oooo0080oooo00Co00000?ooo`3ooooo00000`3oool001l0oooo00Co00000?ooo`3ooooo0000;03o
ool01Ol00000oooo0?ooo`3ooooo000002h0oooo00?o00000?ooool00000;@3oool00ol00000oooo
0?ooo`0_0?ooo`05o`00003oool0oooo0?ooool00000:`3oool00ol00000oooo0?ooo`020?ooo`04
o`00003oool0ooooo`0000<0oooo000O0?ooo`04o`00003oool0ooooo`0002`0oooo00Go00000?oo
o`3oool0ooooo`00000_0?ooo`;o0000;P3oool00ol00000oooo0?ooo`0^0?ooo`05o`00003oool0
oooo0?ooool00000:P3oool2o`0000@0oooo00Co00000?ooo`3ooooo00000`3oool00200oooo0_l0
000^0?ooo`?o0000<@3oool00ol00000oooo0?ooo`0]0?ooo`;o0000;`3oool3o`0002`0oooo00?o
00000?ooo`3oool00`3oool2o`0000@0oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40
oooo003o0?ooob40oooo000K0?ooool000001@0000010?ooo`006`3oool00`00003o00000?ooo`03
0?ooo`03o`00003oool0oooo00T0oooo00?o00000?ooo`3oool02P3oool00ol00000oooo0?ooo`09
0?ooo`03o`00003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00ol00000oooo0?ooo`09
0?ooo`03o`00003oool0oooo00T0oooo00?o00000?ooo`3oool02P3oool00`000000oooo0?ooo`09
0?ooo`03o`00003oool0oooo00X0oooo00?o00000?ooo`3oool02@3oool00ol00000oooo0?ooo`09
0?ooo`030000003oool0oooo00X0oooo00?o00000?ooo`3oool02@3oool00ol00000oooo0?ooo`09
0?ooo`03o`00003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00ol00000oooo0?ooo`09
0?ooo`03o`00003oool0oooo00X0oooo00?o00000?ooo`3oool02@3oool00`000000oooo0?ooo`02
0?ooo`7o00000@0000010?ooo`006`3oool00`000000oooo0?ooo`030?ooo`03o`00003oool0oooo
02h0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo
02l0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`030?ooo`4000000@3oool001/0
oooo00<000000?ooo`3oool0o`3oool10?ooo`4000000@3oool001/0oooo00<000000?ooo`3oool0
3@3ooolEool0o`d0ooooV?oo0?l<0?l0o`Goo`3o8`3o0?l50?ooo`4000000@3oool001/0oooo00<0
00000?ooo`3oool02`3oool2ool0oaD0oooo3Ooo0?oA0?ooo`4000000@3oool001/0oooo00<00000
o`00003oool02P3oool00ooo0?l0oooo0?ooo`3b0?ooo`7o00000@0000010?ooo`006`3oool00`00
0000oooo0?ooo`090?ooo`03ool0o`3oool0oooo0?@0oooo0@0000010?ooo`006`3oool00`000000
oooo0?ooo`070?ooo`;oo`3om`3oool100000040oooo000K0?ooo`030000003oool0oooo00H0oooo
00?oo`3o0?ooo`3oool0m`3oool100000040oooo000K0?ooo`0300000?l00000oooo00D0oooo00?o
o`3o0?ooo`3oool0m`3oool1o`00004000000@3oool001/0oooo00<000000?ooo`3oool0103oool0
0ooo0?l0oooo0?ooo`2<0?ooo`03003o003oool0oooo06X0oooo0@0000010?ooo`001`3oool3o`00
00@0oooo00?o00000?ooo`3oool00P3oool3o`0000D0oooo00<000000?ooo`3oool00`3oool@o`3o
00@00?l00`3oool5003o00<0oooo1@00o`030?ooo`D00?l00`3oool5003o0080oooo1@00o`030?oo
o`D00?l00`3oool5003o00<0oooo1@00o`030?ooo`@00?l00`3oool5003o00<0oooo1@00o`030?oo
o`D00?l00`3oool5003o0080oooo1@00o`030?ooo`D00?l00`3oool5003o00<0oooo1@00o`030?oo
o`D00?l00P3oool5003o00<0oooo1000o`001?l0o`00oooo0?ooo`3oool5003o00<0oooo1@00o`03
0?ooo`04o`3o0000o`000?l0003o00<0oooo1@00o`030?ooo`D00?l00`3oool5003o00<0oooo00?o
0?l0003o0000o`000P00o`030?ooo`@00?l00`3oool5003o00L0oooo0@0000010?ooo`00203oool0
0ol00000oooo0?ooo`070?ooo`05o`00003oool0oooo0?ooool00000103oool00`000000oooo0?oo
o`0C0?ooonSo0?l01@3oool100000040oooo0005o`0000<0oooo00?o00000?ooo`3oool02`3oool0
0ol00000oooo0?ooo`020?ooo`030000003oool0oooo0?l0oooo0@3oool100000040oooo00080?oo
o`03o`00003oool0oooo00/0oooo00?o00000?ooo`3oool00P3oool00`00003o00000?ooo`3o0?oo
o`7o00000@0000010?ooo`00203oool00ol00000oooo0?ooo`070?ooo`Co00001@3oool00`000000
oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`001`3oool2o`0000T0oooo00?o00000?ooo`3oool0
1P3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`00203oool00ol00000oooo0?oo
o`070?ooo`Go0000103oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`006`3oool0
0`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`006`3oool00`00003o00000?ooo`3o0?oo
o`7o00000@0000010?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`00
6`3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`006`3oool00`000000oooo0?oo
o`3o0?ooo`40oooo0@0000010?ooo`006`3oool00`00003o00000?ooo`3o0?ooo`7o00000@000001
0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`006`3oool00`000000
oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`40oooo
0@0000010?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`006`3oool0
0`00003o00000?ooo`3o0?ooo`7o00000@0000010?ooo`006`3oool00`000000oooo0?ooo`3o0?oo
o`40oooo0@0000010?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`00
6`3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`006`3oool00`00003o00000?oo
o`3o0?ooo`7o00000@0000010?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@000001
0?ooo`004`3oool3o`0000D0oooo00<000000?ooo`3oool0o`3oool10?ooo`4000000@3oool001@0
oooo00?o00000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`00
303oool5o`0000<0oooo00?o00000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?ooo`40oooo
0@0000010?ooo`00503oool00ol00000oooo0?ooo`040?ooo`0300000?l00000oooo0?l0oooo0Ol0
000100000040oooo000D0?ooo`03o`00003oool0oooo00@0oooo00<000000?ooo`3oool0o`3oool1
0?ooo`4000000@3oool001<0oooo0_l000060?ooo`030000003oool0oooo0?l0oooo0@3oool10000
0040oooo000D0?ooo`03o`00003oool0oooo00@0oooo00<000000?ooo`3oool0o`3oool10?ooo`40
00000@3oool001/0oooo00<000000?ooo`3oool0o`3oool10?ooo`4000000@3oool001/0oooo00<0
0000o`00003oool0o`3oool1o`00004000000@3oool001/0oooo00<000000?ooo`3oool0o`3oool1
0?ooo`4000000@3oool001/0oooo00<000000?ooo`3oool0o`3oool10?ooo`4000000@3oool001/0
oooo00<000000?ooo`3oool0o`3oool10?ooo`4000000@3oool001/0oooo00<00000o`00003oool0
o`3oool1o`00004000000@3oool001/0oooo00<000000?ooo`3oool0o`3oool10?ooo`4000000@3o
ool001/0oooo00<000000?ooo`3oool0o`3oool10?ooo`4000000@3oool001/0oooo00<000000?oo
o`3oool0o`3oool10?ooo`4000000@3oool001/0oooo00<000000?ooo`3oool0o`3oool10?ooo`40
00000@3oool001/0oooo00<00000o`00003oool0o`3oool1o`00004000000@3oool001/0oooo00<0
00000?ooo`3oool0o`3oool10?ooo`4000000@3oool001/0oooo00<000000?ooo`3oool0o`3oool1
0?ooo`4000000@3oool001/0oooo00<000000?ooo`3oool0o`3oool10?ooo`4000000@3oool001/0
oooo00<00000o`00003oool0o`3oool1o`00004000000@3oool001/0oooo00<000000?ooo`3oool0
o`3oool10?ooo`4000000@3oool000P0oooo0_l000040?ooo`03o`00003oool0oooo0080oooo0ol0
00050?ooo`030000003oool0oooo0?l0oooo0@3oool100000040oooo00070?ooo`04o`00003oool0
ooooo`0000L0oooo00Go00000?ooo`3oool0ooooo`0000040?ooo`030000003oool0oooo0?l0oooo
0@3oool100000040oooo0005o`000080oooo00Co00000?ooo`3ooooo00002`3oool00ol00000oooo
0?ooo`020?ooo`030000003oool0oooo0?l0oooo0@3oool100000040oooo00070?ooo`04o`00003o
ool0ooooo`0000/0oooo00?o00000?ooo`3oool00P3oool00`00003o00000?ooo`3o0?ooo`7o0000
0@0000010?ooo`001`3oool01?l00000oooo0?ooool000070?ooo`Co00001@3oool00`000000oooo
0?ooo`3o0?ooo`40oooo0@0000010?ooo`001`3oool01?l00000oooo0?ooool000070?ooo`03o`00
003oool0oooo00H0oooo00<000000?ooo`3oool0o`3oool10?ooo`4000000@3oool000P0oooo0_l0
00080?ooo`Go0000103oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`006`3oool0
0`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`006`3oool00`00003o00000?ooo`3o0?oo
o`7o00000@0000010?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`00
6`3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`006`3oool00`000000oooo0?oo
o`3o0?ooo`40oooo0@0000010?ooo`006`3oool00`00003o00000?ooo`3o0?ooo`7o00000@000001
0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`006`3oool00`000000
oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`40oooo
0@0000010?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`006`3oool0
0`00003o00000?ooo`3o0?ooo`7o00000@0000010?ooo`006`3oool00`000000oooo0?ooo`3o0?oo
o`40oooo0@0000010?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`00
6`3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?ooo`006`3oool00`00003o00000?oo
o`3o0?ooo`7o00000@0000010?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`40oooo0@000001
0?ooo`00503oool2o`0000D0oooo00<000000?ooo`3oool0o`3oool10?ooo`4000000@3oool001<0
oooo00Co00000?ooo`3ooooo0000103oool00`000000oooo0?ooo`3o0?ooo`40oooo0@0000010?oo
o`004`3oool01?l00000oooo0?ooool000040?ooo`030000003oool0oooo0?l0oooo0@3oool10000
0040oooo000C0?ooo`04o`00003oool0ooooo`0000@0oooo1P00000=ool00?4000000@3oool001<0
oooo00Co00000?ooo`3ooooo0000103oool00`000000oooo0?ooo`0>0?ooo`Ooo`00j`3oool10000
0040oooo000C0?ooo`04o`00003oool0ooooo`0000@0oooo00<000000?ooo`3oool04P3oool00ooo
0000oooo0?ooo`03ool00>P0oooo0@0000010?ooo`00503oool2o`0000D0oooo00<000000?ooo`3o
ool0603oool3ool00>D0oooo0@0000010?ooo`006`3oool00`000000oooo0?ooo`0J0?ooo`Coo`00
hP3oool100000040oooo000K0?ooo`0300000?l00000oooo01d0oooo1?oo003N0?ooo`7o00000@00
00010?ooo`006`3oool00`000000oooo0?ooo`0P0?ooo`Coo`00g03oool100000040oooo000K0?oo
o`030000003oool0oooo02@0oooo0ooo003I0?ooo`4000000@3oool001/0oooo00<000000?ooo`3o
ool09`3oool3ool00=H0oooo0@0000010?ooo`006`3oool00`00003o00000?ooo`0X0?ooo`03ool0
003oooooo`0000;oo`00dP3oool1o`00004000000@3oool001/0oooo00<000000?ooo`3oool0;03o
ool3ool00=40oooo0@0000010?ooo`006`3oool00`000000oooo0?ooo`0]0?ooo`Coo`00c`3oool1
00000040oooo000K0?ooo`030000003oool0oooo02l0oooo00Coo`000?oooooo003oo`00c@3oool1
00000040oooo000K0?ooo`030000003oool0oooo03<0oooo0_oo003;0?ooo`4000000@3oool001/0
oooo00<00000o`00003oool0<`3oool4ool00<P0oooo0Ol0000100000040oooo000K0?ooo`030000
003oool0oooo03H0oooo0ooo00370?ooo`4000000@3oool001/0oooo00<000000?ooo`3oool0>@3o
ool2ool00<D0oooo0@0000010?ooo`006`3oool00`000000oooo0?ooo`0j0?ooo`;oo`00a03oool1
00000040oooo000K0?ooo`0300000?l00000oooo03`0oooo0_oo00310?ooo`7o00000@0000010?oo
o`006`3oool00`000000oooo0?ooo`0m0?ooo`?oo`00`03oool100000040oooo00080?ooo`;o0000
103oool00ol00000oooo0?ooo`020?ooo`?o00001@3oool00`000000oooo0?ooo`100?ooo`03ool0
003oool0oooo0;d0oooo0@0000010?ooo`001`3oool01?l00000oooo0?ooool000070?ooo`05o`00
003oool0oooo0?ooool00000103oool00`000000oooo0?ooo`110?ooo`;oo`00_@3oool100000040
oooo00070?ooo`04o`00003oool0ooooo`0000/0oooo00?o00000?ooo`3oool00P3oool00`000000
oooo0?ooo`130?ooo`;oo`00^`3oool100000040oooo00070?ooo`04o`00003oool0ooooo`0000/0
oooo00?o00000?ooo`3oool00P3oool00`00003o00000?ooo`140?ooo`;oo`00^@3oool1o`000040
00000@3oool000L0oooo00Co00000?ooo`3ooooo00001`3oool4o`0000D0oooo00<000000?ooo`3o
ool0AP3oool2ool00;P0oooo0@0000010?ooo`001`3oool01?l00000oooo0?ooool000070?ooo`03
o`00003oool0oooo00H0oooo00<000000?ooo`3oool0A`3oool3ool00;H0oooo0@0000010?ooo`00
203oool2o`0000P0oooo1Ol000040?ooo`030000003oool0oooo04T0oooo0_oo002e0?ooo`400000
0@3oool001/0oooo00<000000?ooo`3oool0B`3oool2ool00;<0oooo0@0000010?ooo`006`3oool0
0`00003o00000?ooo`1=0?ooo`;oo`00/03oool1o`00004000000@3oool001/0oooo00<000000?oo
o`3oool0CP3oool2ool00;00oooo0@0000010?ooo`006`3oool00`000000oooo0?ooo`1@0?ooo`;o
o`00[P3oool100000040oooo000K0?ooo`030000003oool0oooo0540oooo0ooo002/0?ooo`400000
0@3oool001/0oooo00<00000o`00003oool0E03oool00ooo0000oooo0?ooo`2X0?ooo`7o00000@00
00010?ooo`006`3oool00`000000oooo0?ooo`1E0?ooo`;oo`00Z@3oool100000040oooo000K0?oo
o`030000003oool0oooo05H0oooo0_oo002X0?ooo`4000000@3oool001/0oooo00<000000?ooo`3o
ool0F03oool2ool00:H0oooo0@0000010?ooo`006`3oool00`000000oooo0?ooo`1J0?ooo`;oo`00
Y03oool100000040oooo000K0?ooo`0300000?l00000oooo05/0oooo0_oo002R0?ooo`7o00000@00
00010?ooo`006`3oool00`000000oooo0?ooo`1M0?ooo`;oo`00X@3oool100000040oooo000K0?oo
o`030000003oool0oooo05l0oooo00?oo`000?ooo`3oool0WP3oool100000040oooo000K0?ooo`03
0000003oool0oooo0600oooo0_oo002N0?ooo`4000000@3oool001/0oooo00<00000o`00003oool0
HP3oool2ool009/0oooo0Ol0000100000040oooo000K0?ooo`030000003oool0oooo06<0oooo0_oo
002K0?ooo`4000000@3oool001<0oooo0ol000050?ooo`030000003oool0oooo06D0oooo00?oo`00
0?ooo`3oool0V03oool100000040oooo000D0?ooo`03o`00003oool0oooo00@0oooo00<000000?oo
o`3oool0IP3oool00ooo0000oooo0?ooo`2G0?ooo`4000000@3oool001@0oooo00?o00000?ooo`3o
ool0103oool00`000000oooo0?ooo`1W0?ooo`?oo`00UP3oool100000040oooo000D0?ooo`03o`00
003oool0oooo00@0oooo00<00000o`00003oool0J@3oool2ool009@0oooo0Ol0000100000040oooo
000D0?ooo`03o`00003oool0oooo00@0oooo00<000000?ooo`3oool0JP3oool3ool009<0oooo0@00
00010?ooo`004`3oool2o`0000H0oooo00<000000?ooo`3oool0K03oool2ool00980oooo0@000001
0?ooo`00503oool00ol00000oooo0?ooo`040?ooo`030000003oool0oooo06h0oooo0ooo002?0?oo
o`4000000@3oool001/0oooo00<000000?ooo`3oool0L03oool3ool008d0oooo0@0000010?ooo`00
6`3oool00`00003o00000?ooo`1b0?ooo`;oo`00R`3oool1o`00004000000@3oool001/0oooo00<0
00000?ooo`3oool0M03oool2ool008X0oooo0@0000010?ooo`006`3oool00`000000oooo0?ooo`1f
0?ooo`;oo`00R03oool100000040oooo000K0?ooo`030000003oool0oooo07P0oooo0_oo00260?oo
o`4000000@3oool001/0oooo00<00000o`00003oool0N@3oool2ool008@0oooo0Ol0000100000040
oooo000K0?ooo`030000003oool0oooo07/0oooo0_oo00230?ooo`4000000@3oool001/0oooo00<0
00000?ooo`3oool0O@3oool3ool00800oooo0@0000010?ooo`006`3oool00`000000oooo0?ooo`0E
0?ooo`co003oGP3oool2ool007l0oooo0@0000010?ooo`006`3oool00`000000oooo0?ooo`0C0?oo
o`;o003o303oool8o`00oeL0oooo0ooo001m0?ooo`4000000@3oool001/0oooo00<00000o`00003o
ool04@3oool2o`00oaH0oooo[_l00?l@0000o`Go003o3@000?l2o`00o`@0oooo0Ol0000100000040
oooo000K0?ooo`030000003oool0oooo0100oooo00?o003o0?ooo`3oool0LP3oool2ool007T0oooo
0@0000010?ooo`006`3oool00`000000oooo0?ooo`0?0?ooo`03o`00o`3oool0oooo07D0oooo0_oo
001g0?ooo`4000000@3oool001/0oooo00<000000?ooo`3oool03`3oool00ol00?l0oooo0?ooo`1g
0?ooo`;oo`00M@3oool100000040oooo000K0?ooo`0300000?l00000oooo00h0oooo00?o003o0?oo
o`3oool0NP3oool3ool00740oooo0Ol0000100000040oooo000K0?ooo`030000003oool0oooo00`0
oooo0_l00?n00?ooo`;oo`00L03oool100000040oooo00070?ooo`?o0000103oool00ol00000oooo
0?ooo`020?ooo`?o00001@3oool00`000000oooo0?ooo`0;0?ooo`03o`00o`3oool0oooo0880oooo
0ooo001]0?ooo`4000000@3oool000P0oooo00?o00000?ooo`3oool01`3oool01Ol00000oooo0?oo
o`3ooooo000000@0oooo00<000000?ooo`3oool02@3oool2o`00ohL0oooo1?oo001Z0?ooo`400000
0@3oool000P0oooo00?o00000?ooo`3oool02`3oool00ol00000oooo0?ooo`020?ooo`030000003o
ool0oooo00P0oooo00?o003o0?ooo`3oool0RP3oool3ool006P0oooo0@0000010?ooo`00203oool0
0ol00000oooo0?ooo`0;0?ooo`03o`00003oool0oooo0080oooo00<00000o`00003oool01`3oool0
0ol00?l0oooo0?ooo`2>0?ooo`Goo`00HP3oool1o`00004000000@3oool000P0oooo00?o00000?oo
o`3oool01`3oool4o`0000D0oooo00<000000?ooo`3oool01@3oool2o`00oiD0oooo1Ooo001O0?oo
o`4000000@3oool000L0oooo0_l000090?ooo`03o`00003oool0oooo00H0oooo00<000000?ooo`3o
ool0103oool00ol00?l0oooo0?ooo`2J0?ooo`Goo`00FP3oool100000040oooo00080?ooo`03o`00
003oool0oooo00L0oooo1Ol000040?ooo`030000003oool0oooo00<0oooo1Ol00?l30?ooo`Go003o
0P3oool5o`00o`<0oooo1Ol00?l30?ooo`Go003o0`3oool5o`00o`<0oooo1Ol00?l20?ooo`Go003o
0`3oool5o`00o`<0oooo1Ol00?l30?ooo`Go003o0`3oool4o`00o`<0oooo1Ol00?l30?ooo`Go003o
0`3oool5o`00o`<0oooo1Ol00?l20?ooo`Go003o0`3oool5o`00o`<0oooo1?l00?l010000?l0oooo
0?ooo`3oool50000o`<0oooo1@000?l00`3oooooo`00ool00007ool000@0003o00?o003o0?ooo`3o
ool06ooo00030?l000Soo`007P3o0002ool000D0oooo0@0000010?ooo`006`3oool00`000000oooo
0?ooo`2^0?ooo`Ooo`00B`3oool100000040oooo000K0?ooo`0300000?l00000oooo0?l0oooo0Ol0
000100000040oooo000K0?ooo`030000003oool0oooo00<0oooo00?o00000?ooo`3oool02@3oool0
0ol00000oooo0?ooo`0:0?ooo`03o`00003oool0oooo00T0oooo00?o00000?ooo`3oool02@3oool0
0ol00000oooo0?ooo`0:0?ooo`03o`00003oool0oooo00T0oooo00?o00000?ooo`3oool02@3oool0
0ol00000oooo0?ooo`0:0?ooo`03o`00003oool0oooo00T0oooo00?o00000?ooo`3oool02P3oool0
0ol00000oooo0?ooo`090?ooo`03o`00003oool0oooo00T0oooo00?o00000?ooo`3oool02P3oool0
0ol00000oooo0?ooo`090?ooo`03o`00003oool0oooo00T0oooo00?o00000?ooo`3oool02P3oool0
0ol00000oooo0?ooo`090?ooo`03o`00003oool0oooo00T0oooo00?o00000?ooo`3oool02P3oool0
0ol00000oooo0?ooo`090?ooo`03o`00003oool0oooo00<0oooo0@0000010?ooo`006`3ooooo0000
00D000000@3oool00?l0oooo8@3oool00?l0oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.35193, -2.1393, \
0.0404426, 0.0217358}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Error plot", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(error\  = \ 
        CombineInterpolationFunctions[desiredOut, actual, {0, tmax}, 
          Subtract];\)\), "\n", 
    \(\(gerror = 
        PlotMultipleTrajectory[error, {0, tmax}, actualStyle, 
          imgSize];\)\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 288 177.994 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0952381 0.0570789 1.70566 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.21429 -0.0125 -3 -9 ]
[.21429 -0.0125 3 0 ]
[.40476 -0.0125 -3 -9 ]
[.40476 -0.0125 3 0 ]
[.59524 -0.0125 -3 -9 ]
[.59524 -0.0125 3 0 ]
[.78571 -0.0125 -3 -9 ]
[.78571 -0.0125 3 0 ]
[.97619 -0.0125 -6 -9 ]
[.97619 -0.0125 6 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .05708 -6 -4.5 ]
[-0.0125 .05708 0 4.5 ]
[-0.0125 .14236 -24 -4.5 ]
[-0.0125 .14236 0 4.5 ]
[-0.0125 .22764 -18 -4.5 ]
[-0.0125 .22764 0 4.5 ]
[-0.0125 .31293 -24 -4.5 ]
[-0.0125 .31293 0 4.5 ]
[-0.0125 .39821 -18 -4.5 ]
[-0.0125 .39821 0 4.5 ]
[-0.0125 .48349 -24 -4.5 ]
[-0.0125 .48349 0 4.5 ]
[-0.0125 .56878 -18 -4.5 ]
[-0.0125 .56878 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.21429 0 m
.21429 .00625 L
s
[(2)] .21429 -0.0125 0 1 Mshowa
.40476 0 m
.40476 .00625 L
s
[(4)] .40476 -0.0125 0 1 Mshowa
.59524 0 m
.59524 .00625 L
s
[(6)] .59524 -0.0125 0 1 Mshowa
.78571 0 m
.78571 .00625 L
s
[(8)] .78571 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(10)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.5 0 m
.5 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .05708 m
.00625 .05708 L
s
[(0)] -0.0125 .05708 1 0 Mshowa
0 .14236 m
.00625 .14236 L
s
[(0.05)] -0.0125 .14236 1 0 Mshowa
0 .22764 m
.00625 .22764 L
s
[(0.1)] -0.0125 .22764 1 0 Mshowa
0 .31293 m
.00625 .31293 L
s
[(0.15)] -0.0125 .31293 1 0 Mshowa
0 .39821 m
.00625 .39821 L
s
[(0.2)] -0.0125 .39821 1 0 Mshowa
0 .48349 m
.00625 .48349 L
s
[(0.25)] -0.0125 .48349 1 0 Mshowa
0 .56878 m
.00625 .56878 L
s
[(0.3)] -0.0125 .56878 1 0 Mshowa
.125 Mabswid
0 .07414 m
.00375 .07414 L
s
0 .09119 m
.00375 .09119 L
s
0 .10825 m
.00375 .10825 L
s
0 .12531 m
.00375 .12531 L
s
0 .15942 m
.00375 .15942 L
s
0 .17647 m
.00375 .17647 L
s
0 .19353 m
.00375 .19353 L
s
0 .21059 m
.00375 .21059 L
s
0 .2447 m
.00375 .2447 L
s
0 .26176 m
.00375 .26176 L
s
0 .27881 m
.00375 .27881 L
s
0 .29587 m
.00375 .29587 L
s
0 .32998 m
.00375 .32998 L
s
0 .34704 m
.00375 .34704 L
s
0 .3641 m
.00375 .3641 L
s
0 .38115 m
.00375 .38115 L
s
0 .41527 m
.00375 .41527 L
s
0 .43232 m
.00375 .43232 L
s
0 .44938 m
.00375 .44938 L
s
0 .46644 m
.00375 .46644 L
s
0 .50055 m
.00375 .50055 L
s
0 .51761 m
.00375 .51761 L
s
0 .53466 m
.00375 .53466 L
s
0 .55172 m
.00375 .55172 L
s
0 .04002 m
.00375 .04002 L
s
0 .02297 m
.00375 .02297 L
s
0 .00591 m
.00375 .00591 L
s
0 .58583 m
.00375 .58583 L
s
0 .60289 m
.00375 .60289 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.21429 .61178 m
.21429 .61803 L
s
.40476 .61178 m
.40476 .61803 L
s
.59524 .61178 m
.59524 .61803 L
s
.78571 .61178 m
.78571 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.07143 .61428 m
.07143 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.16667 .61428 m
.16667 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.5 .61428 m
.5 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .05708 m
1 .05708 L
s
.99375 .14236 m
1 .14236 L
s
.99375 .22764 m
1 .22764 L
s
.99375 .31293 m
1 .31293 L
s
.99375 .39821 m
1 .39821 L
s
.99375 .48349 m
1 .48349 L
s
.99375 .56878 m
1 .56878 L
s
.125 Mabswid
.99625 .07414 m
1 .07414 L
s
.99625 .09119 m
1 .09119 L
s
.99625 .10825 m
1 .10825 L
s
.99625 .12531 m
1 .12531 L
s
.99625 .15942 m
1 .15942 L
s
.99625 .17647 m
1 .17647 L
s
.99625 .19353 m
1 .19353 L
s
.99625 .21059 m
1 .21059 L
s
.99625 .2447 m
1 .2447 L
s
.99625 .26176 m
1 .26176 L
s
.99625 .27881 m
1 .27881 L
s
.99625 .29587 m
1 .29587 L
s
.99625 .32998 m
1 .32998 L
s
.99625 .34704 m
1 .34704 L
s
.99625 .3641 m
1 .3641 L
s
.99625 .38115 m
1 .38115 L
s
.99625 .41527 m
1 .41527 L
s
.99625 .43232 m
1 .43232 L
s
.99625 .44938 m
1 .44938 L
s
.99625 .46644 m
1 .46644 L
s
.99625 .50055 m
1 .50055 L
s
.99625 .51761 m
1 .51761 L
s
.99625 .53466 m
1 .53466 L
s
.99625 .55172 m
1 .55172 L
s
.99625 .04002 m
1 .04002 L
s
.99625 .02297 m
1 .02297 L
s
.99625 .00591 m
1 .00591 L
s
.99625 .58583 m
1 .58583 L
s
.99625 .60289 m
1 .60289 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
1 0 0 r
.5 Mabswid
.02381 .05708 m
.04262 .06152 L
.06244 .06491 L
.07286 .06621 L
.08255 .06715 L
.0932 .0679 L
.09916 .0682 L
.10458 .06841 L
.10941 .06854 L
.11185 .06859 L
.11325 .06861 L
.11454 .06863 L
.11563 .06864 L
.11682 .06865 L
.11808 .06866 L
.11928 .06866 L
.1204 .06867 L
.12159 .06867 L
.12271 .06866 L
.12373 .06866 L
.12503 .06865 L
.1264 .06864 L
.12887 .06861 L
.13152 .06857 L
.13443 .06852 L
.13964 .06838 L
.14451 .06822 L
.15496 .06778 L
.16486 .06725 L
.18342 .06605 L
.20164 .06469 L
.20663 .06431 L
.21201 .0639 L
.21709 .06361 L
.22175 .06342 L
.24152 .06263 L
.26253 .06183 L
.30215 .06035 L
.34026 .05871 L
.38082 .05686 L
.41987 .05509 L
.46137 .05327 L
.50135 .0516 L
.53981 .05012 L
.58073 .04868 L
.62014 .04709 L
.63993 .04635 L
.64859 .04608 L
.65802 .04585 L
.6632 .04575 L
Mistroke
.668 .04567 L
.67259 .04561 L
.67515 .04559 L
.67747 .04558 L
.67878 .04557 L
.6795 .04557 L
.68017 .04556 L
.68135 .04556 L
.68265 .04556 L
.68337 .04556 L
.68406 .04556 L
.68535 .04556 L
.68607 .04556 L
.68673 .04556 L
.68824 .04557 L
.68944 .04557 L
.69057 .04558 L
.69181 .04565 L
.69315 .04572 L
.69836 .04601 L
.73956 .04892 L
.76012 .05053 L
.77014 .05128 L
.77923 .05192 L
.78379 .05223 L
.78505 .05231 L
.78637 .05237 L
.7875 .05239 L
.78873 .05242 L
.79769 .05258 L
.80261 .05265 L
.80788 .0527 L
.81043 .05272 L
.81285 .05274 L
.81504 .05275 L
.81739 .05276 L
.81862 .05276 L
.81975 .05277 L
.8209 .05277 L
.82197 .05277 L
.82315 .05277 L
.82443 .05277 L
.82569 .05277 L
.82639 .05277 L
.82703 .05277 L
.82834 .05277 L
.82956 .05277 L
.83089 .05277 L
.8323 .05276 L
.83467 .05275 L
Mistroke
.83728 .05274 L
.84186 .05272 L
.84679 .05268 L
.85569 .05261 L
.87541 .05239 L
.89693 .05213 L
.90691 .05203 L
.91239 .05198 L
.91754 .05194 L
.92236 .05191 L
.9249 .0519 L
.92757 .05189 L
.92983 .05189 L
.93111 .05188 L
.93231 .05188 L
.93341 .05188 L
.93441 .05188 L
.93557 .05188 L
.93666 .05188 L
.9378 .05188 L
.93903 .05188 L
.94019 .05188 L
.94126 .05188 L
.94373 .05189 L
.94633 .0519 L
.94887 .05192 L
.95123 .05193 L
.95659 .05198 L
.96126 .05204 L
.9655 .0521 L
.97501 .05229 L
.97619 .05232 L
Mfstroke
0 0 1 r
.02381 .05708 m
.04262 .24453 L
.06244 .39272 L
.07286 .45229 L
.08255 .49754 L
.0932 .53672 L
.09912 .55412 L
.10458 .56758 L
.10944 .57755 L
.1148 .58649 L
.12018 .59342 L
.12316 .59642 L
.12589 .59867 L
.12828 .60026 L
.13089 .6016 L
.13233 .60217 L
.13366 .60259 L
.13493 .60291 L
.13563 .60304 L
.13628 .60314 L
.13755 .60327 L
.13821 .60331 L
.13891 .60332 L
.1401 .60328 L
.14139 .60316 L
.14255 .60298 L
.14379 .60272 L
.14603 .60206 L
.14859 .60103 L
.15131 .59961 L
.1562 .59632 L
.16146 .59177 L
.16727 .5857 L
.18739 .55822 L
.1976 .54196 L
.20318 .53286 L
.20836 .52441 L
.21064 .52072 L
.2128 .51727 L
.21399 .51537 L
.21529 .51473 L
.21646 .51453 L
.21754 .51435 L
.22242 .5137 L
.22488 .51345 L
.22759 .51323 L
.22883 .51315 L
.23017 .51308 L
.23136 .51302 L
.23262 .51298 L
Mistroke
.23388 .51295 L
.23501 .51293 L
.23615 .51292 L
.23723 .51292 L
.23849 .51293 L
.23912 .51294 L
.23982 .51295 L
.24105 .51298 L
.2422 .51302 L
.2448 .51313 L
.24758 .5133 L
.25261 .51372 L
.25736 .51424 L
.26628 .51549 L
.28623 .51925 L
.2951 .52116 L
.30466 .52326 L
.30711 .5238 L
.30845 .52409 L
.3097 .52432 L
.31076 .52435 L
.3119 .52438 L
.31431 .52443 L
.31866 .52451 L
.32336 .52457 L
.32626 .5246 L
.32753 .52461 L
.32892 .52462 L
.33015 .52463 L
.33147 .52464 L
.3322 .52464 L
.33287 .52464 L
.33417 .52465 L
.33542 .52465 L
.33658 .52465 L
.33785 .52465 L
.33849 .52465 L
.3392 .52465 L
.34039 .52464 L
.34166 .52464 L
.34398 .52463 L
.34637 .52461 L
.34893 .52459 L
.35353 .52455 L
.36399 .5244 L
.37504 .52419 L
.38508 .52396 L
.39439 .52374 L
.39948 .52362 L
.40197 .52356 L
Mistroke
.40306 .52353 L
.40425 .5235 L
.40552 .52349 L
.40673 .52348 L
.40939 .52347 L
.41506 .52346 L
.42503 .52344 L
.44472 .52341 L
.4657 .52338 L
.476 .52337 L
.48713 .52335 L
.49224 .52334 L
.49503 .52333 L
.49764 .52333 L
.49878 .52333 L
.49986 .52332 L
.50111 .52332 L
.50225 .52331 L
.5073 .52328 L
.54565 .52304 L
.58645 .52273 L
.62574 .5224 L
.66351 .52211 L
.68287 .52198 L
.68818 .52195 L
.69376 .52193 L
.69852 .52191 L
.70373 .5219 L
.71383 .52187 L
.72336 .52186 L
.72863 .52185 L
.73434 .52184 L
.73949 .52184 L
.74431 .52183 L
.74888 .52183 L
.75379 .52183 L
.75658 .52183 L
.75923 .52183 L
.76161 .52183 L
.76296 .52183 L
.76421 .52183 L
.7649 .52183 L
.76553 .52183 L
.76623 .52183 L
.76696 .52183 L
.7682 .52183 L
.7689 .52183 L
.76956 .52183 L
.77078 .52183 L
.7719 .52183 L
Mistroke
.7744 .52183 L
.77708 .52183 L
.77998 .52183 L
.78252 .52183 L
.78392 .52183 L
.78461 .52183 L
.78525 .52183 L
.78647 .52183 L
.78711 .52183 L
.7878 .52183 L
.79012 .52183 L
.79469 .52183 L
.79749 .52183 L
.80014 .52183 L
.80249 .52183 L
.80508 .52183 L
.80764 .52182 L
.80894 .52182 L
.81034 .52182 L
.81165 .52182 L
.81284 .52182 L
.81404 .52182 L
.81518 .52182 L
.8165 .52182 L
.81716 .52182 L
.8179 .52182 L
.81919 .52182 L
.82041 .52182 L
.82184 .52182 L
.82315 .52182 L
.82611 .52183 L
.83084 .52183 L
.8359 .52183 L
.84502 .52183 L
.86572 .52183 L
.88562 .52184 L
.8961 .52184 L
.9072 .52184 L
.91198 .52184 L
.9171 .52184 L
.92004 .52184 L
.92273 .52184 L
.92543 .52184 L
.92678 .52184 L
.928 .52184 L
.92918 .52184 L
.9303 .52184 L
.93148 .52184 L
.93276 .52184 L
.93388 .52184 L
Mistroke
.93506 .52184 L
.93607 .52184 L
.93717 .52184 L
.93848 .52184 L
.93967 .52184 L
.94235 .52184 L
.94483 .52184 L
.94717 .52184 L
.95065 .52184 L
.95449 .52184 L
.96225 .52184 L
.96945 .52184 L
.97619 .52183 L
Mfstroke
1 0 1 r
.02381 .05708 m
.04262 .14815 L
.06244 .22232 L
.08255 .2773 L
.09396 .30046 L
.10458 .31736 L
.11508 .33005 L
.12023 .33495 L
.125 .33873 L
.12924 .34154 L
.1339 .34405 L
.1386 .346 L
.14357 .34747 L
.14606 .34799 L
.14737 .34821 L
.14877 .3484 L
.15011 .34855 L
.15133 .34864 L
.15255 .34871 L
.15371 .34875 L
.15484 .34876 L
.15603 .34875 L
.15705 .34871 L
.15816 .34865 L
.15937 .34856 L
.16066 .34844 L
.16298 .34814 L
.16567 .34769 L
.16859 .34707 L
.1739 .34563 L
.18386 .34199 L
.20325 .33252 L
.20856 .32963 L
.21135 .3281 L
.21287 .32726 L
.21361 .32685 L
.21428 .32648 L
.21553 .32645 L
.21686 .32641 L
.21969 .32635 L
.22464 .32625 L
.22935 .32617 L
.23433 .3261 L
.23694 .32607 L
.2398 .32604 L
.24243 .32602 L
.24484 .32601 L
.24728 .326 L
.24959 .32599 L
.25086 .32598 L
Mistroke
.25203 .32598 L
.25332 .32598 L
.25404 .32598 L
.25469 .32598 L
.25588 .32598 L
.25697 .32598 L
.25817 .32598 L
.25944 .32598 L
.26054 .32598 L
.26157 .32599 L
.26391 .32599 L
.26652 .326 L
.26893 .32601 L
.27439 .32605 L
.2843 .32613 L
.30287 .32633 L
.30752 .32639 L
.30879 .3264 L
.30943 .32641 L
.31013 .32641 L
.31137 .32641 L
.31252 .32641 L
.32157 .3264 L
.34219 .32638 L
.35274 .32637 L
.36269 .32637 L
.36703 .32636 L
.37162 .32636 L
.37402 .32636 L
.37665 .32636 L
.37905 .32636 L
.38129 .32636 L
.38254 .32636 L
.38323 .32636 L
.38388 .32636 L
.38502 .32636 L
.38628 .32636 L
.38695 .32636 L
.38765 .32636 L
.3889 .32636 L
.3896 .32636 L
.39024 .32636 L
.39172 .32636 L
.39301 .32636 L
.39422 .32636 L
.3965 .32636 L
.39895 .32636 L
.40161 .32636 L
.40269 .32636 L
.40384 .32636 L
Mistroke
.40511 .32636 L
.4063 .32636 L
.41126 .32637 L
.42016 .32638 L
.44131 .32643 L
.46141 .32648 L
.49962 .32661 L
.51931 .32671 L
.54028 .32682 L
.58189 .32705 L
.62197 .32727 L
.66054 .32745 L
.6805 .32753 L
.68535 .32754 L
.68802 .32755 L
.68933 .32756 L
.69054 .32756 L
.69124 .32756 L
.69197 .32756 L
.69353 .32757 L
.69625 .32757 L
.70157 .32758 L
.71095 .32759 L
.71587 .3276 L
.72105 .3276 L
.72561 .32761 L
.73059 .32761 L
.73479 .32761 L
.73934 .32761 L
.74209 .32761 L
.74463 .32761 L
.74706 .32761 L
.74829 .32761 L
.74962 .32761 L
.75082 .32762 L
.75192 .32762 L
.75302 .32762 L
.75407 .32762 L
.75529 .32762 L
.75641 .32762 L
.75765 .32762 L
.75896 .32762 L
.75965 .32762 L
.76038 .32762 L
.76169 .32762 L
.76463 .32761 L
.76718 .32761 L
.76996 .32761 L
.77992 .32761 L
.78244 .32761 L
Mistroke
.78377 .32761 L
.7845 .32761 L
.78518 .32761 L
.78641 .32761 L
.78707 .32761 L
.78777 .32761 L
.79016 .32761 L
.79462 .32761 L
.79948 .32761 L
.80239 .32761 L
.80508 .32761 L
.80788 .32761 L
.81041 .32761 L
.81161 .32761 L
.8129 .32761 L
.81402 .32761 L
.81525 .32761 L
.81656 .32761 L
.81727 .32761 L
.81794 .32761 L
.81913 .32761 L
.82041 .32761 L
.82163 .32761 L
.82232 .32761 L
.82297 .32761 L
.82428 .32761 L
.8257 .32761 L
.82824 .32761 L
.83058 .32761 L
.83597 .32761 L
.84165 .32761 L
.85125 .32761 L
.86176 .32761 L
.88301 .32761 L
.90231 .32761 L
.91194 .32761 L
.91702 .3276 L
.92239 .3276 L
.92465 .3276 L
.92708 .3276 L
.92848 .3276 L
.92975 .3276 L
.93105 .3276 L
.93225 .3276 L
.93333 .3276 L
.93448 .3276 L
.93576 .3276 L
.93691 .3276 L
.93801 .3276 L
.9392 .3276 L
Mistroke
.94032 .3276 L
.94134 .3276 L
.94352 .3276 L
.9455 .3276 L
.94994 .3276 L
.95407 .3276 L
.95797 .32761 L
.9667 .32761 L
.97163 .32761 L
.97619 .32761 L
Mfstroke
0 1 0 r
.02381 .05708 m
.04262 .04247 L
.06244 .03094 L
.07286 .02631 L
.08255 .0228 L
.0932 .01978 L
.09912 .01844 L
.10458 .0174 L
.10944 .01664 L
.1148 .01596 L
.12018 .01543 L
.12316 .01521 L
.12589 .01504 L
.12845 .01492 L
.13089 .01483 L
.13223 .01479 L
.13347 .01476 L
.13483 .01474 L
.13559 .01473 L
.13628 .01472 L
.13754 .01472 L
.13869 .01472 L
.13996 .01472 L
.1406 .01473 L
.1413 .01473 L
.14248 .01475 L
.14374 .01478 L
.14603 .01483 L
.14859 .01492 L
.15131 .01504 L
.1562 .01531 L
.16727 .01617 L
.18739 .01836 L
.19698 .01956 L
.20708 .02086 L
.21125 .02139 L
.21245 .02154 L
.21358 .02168 L
.2146 .02177 L
.2157 .02179 L
.21825 .02182 L
.22059 .02184 L
.22294 .02185 L
.22514 .02187 L
.22729 .02188 L
.22851 .02188 L
.22961 .02188 L
.23082 .02188 L
.23214 .02189 L
.23338 .02189 L
Mistroke
.23453 .02189 L
.23584 .02188 L
.23706 .02188 L
.23816 .02188 L
.23935 .02187 L
.24203 .02186 L
.24448 .02185 L
.24933 .02181 L
.25463 .02176 L
.26534 .02161 L
.28653 .02124 L
.29655 .02104 L
.30582 .02086 L
.30697 .02084 L
.30822 .02081 L
.3094 .02079 L
.31047 .02078 L
.31162 .02078 L
.31282 .02078 L
.31541 .02077 L
.32082 .02076 L
.32345 .02076 L
.32584 .02075 L
.32794 .02075 L
.33022 .02075 L
.33147 .02075 L
.33262 .02075 L
.33372 .02075 L
.33489 .02075 L
.33607 .02075 L
.33718 .02075 L
.33849 .02075 L
.3397 .02075 L
.34094 .02075 L
.34165 .02075 L
.3423 .02075 L
.3435 .02075 L
.34478 .02075 L
.34715 .02075 L
.34966 .02075 L
.35413 .02076 L
.35895 .02076 L
.36422 .02077 L
.3825 .02081 L
.39341 .02083 L
.39877 .02085 L
.40113 .02085 L
.40238 .02086 L
.40372 .02086 L
.40493 .02086 L
Mistroke
.40604 .02087 L
.40822 .02087 L
.41298 .02087 L
.42302 .02087 L
.46204 .0209 L
.48264 .02091 L
.49268 .02092 L
.50177 .02093 L
.52027 .02096 L
.53998 .02099 L
.58007 .02106 L
.61864 .02113 L
.6382 .02116 L
.65966 .02119 L
.67028 .02121 L
.68033 .02122 L
.68471 .02122 L
.68935 .02123 L
.69178 .02123 L
.69445 .02123 L
.69917 .02123 L
.70856 .02124 L
.71869 .02124 L
.72432 .02124 L
.72948 .02124 L
.73442 .02125 L
.73709 .02125 L
.73961 .02125 L
.74201 .02125 L
.74462 .02125 L
.74698 .02125 L
.74917 .02125 L
.75039 .02125 L
.75153 .02125 L
.75278 .02125 L
.75348 .02125 L
.75412 .02125 L
.75541 .02125 L
.75614 .02125 L
.75681 .02125 L
.75805 .02125 L
.75937 .02125 L
.76061 .02125 L
.76176 .02125 L
.76434 .02125 L
.76901 .02125 L
.77326 .02125 L
.77774 .02125 L
.78008 .02125 L
.78263 .02125 L
Mistroke
.7838 .02124 L
.78503 .02124 L
.78609 .02124 L
.78725 .02124 L
.79258 .02124 L
.79511 .02125 L
.79743 .02125 L
.79961 .02125 L
.80199 .02125 L
.8033 .02125 L
.80451 .02125 L
.80567 .02125 L
.80691 .02125 L
.80806 .02125 L
.8093 .02125 L
.81048 .02125 L
.81156 .02125 L
.81254 .02125 L
.81361 .02125 L
.81474 .02125 L
.81578 .02125 L
.81838 .02125 L
.82079 .02125 L
.82623 .02125 L
.83103 .02124 L
.83613 .02124 L
.85468 .02124 L
.89242 .02124 L
.90184 .02124 L
.91176 .02124 L
.91695 .02124 L
.9197 .02124 L
.92263 .02124 L
.92534 .02124 L
.9278 .02124 L
.92899 .02124 L
.93027 .02124 L
.93149 .02124 L
.93261 .02124 L
.93386 .02124 L
.93505 .02124 L
.93611 .02124 L
.93728 .02124 L
.93854 .02124 L
.93924 .02124 L
.93988 .02124 L
.9423 .02124 L
.94454 .02124 L
.94697 .02124 L
.95136 .02124 L
Mistroke
.95659 .02124 L
.96212 .02124 L
.972 .02124 L
.97619 .02124 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`008?ooool2o`0002gooooo1Ol0000_ooooo`?o0000
;Oooool3o`0002oooooo0ol0000[ooooo`?o00001?ooool2o`0000Cooooo000Oooooo`04o`000?oo
ooooooooo`0002gooooo00Co0000oooooooooooo0000<?ooool00ol0003oooooooooo`0[ooooo`05
o`000?ooooooooooooooool00000;Oooool01Ol0003ooooooooooooooooo000002_ooooo00?o0000
ooooooooool00_ooool01?l0003oooooooooool00003ooooo`007oooool01?l0003oooooooooool0
000^ooooo`03o`000?oooooooooo02gooooo1Ol0000/ooooo`05o`000?ooooooooooooooool00000
;Oooool01Ol0003ooooooooooooooooo000002_ooooo00?o0000ooooooooool00_ooool01?l0003o
ooooooooool00003ooooo`007oooool01?l0003oooooooooool0000_ooooo`03o`000?oooooooooo
02cooooo00Co0000oooooooooooo0000;Oooool4o`0002oooooo0ol0000/ooooo`03o`000?oooooo
oooo00;ooooo00Co0000oooooooooooo00000oooool001oooooo00Co0000oooooooooooo0000;?oo
ool01Ol0003ooooooooooooooooo000002kooooo00?o0000ooooool00000;Oooool00ol0003ooooo
ooooo`0_ooooo`05o`000?ooooooooooooooool00000:oooool00ol0003oooooooooo`02ooooo`04
o`000?ooooooooooo`0000?ooooo000Oooooo`04o`000?ooooooooooo`0002cooooo00Go0000oooo
ooooooooooooo`00000_ooooo`;o0000;_ooool00ol0003oooooooooo`0^ooooo`05o`000?oooooo
ooooooooool00000:_ooool2o`0000Cooooo00Co0000oooooooooooo00000oooool0023ooooo0_l0
000^ooooo`?o0000<Oooool00ol0003oooooooooo`0]ooooo`;o0000;oooool3o`0002cooooo00?o
0000ooooooooool00oooool2o`0000Cooooo003oooooob7ooooo003oooooob7ooooo003oooooob7o
oooo003oooooob7ooooo000Kooooool000001@000001ooooo`006oooool00`00003o0000ooooo`03
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Wooooo00<00000ooooooooool02_ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02_ooool00`00003oooooooooo`09
ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09
ooooo`0300000?oooooooooo00[ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00[ooooo00<00000ooooooooool02Oooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool02Oooool00`00003oooooooooo`02
ooooo`7o00000@000001ooooo`006oooool00`00003oooooooooo`03ooooo`03o`000?oooooooooo
02kooooo00<00000ooooooooool0;oooool00`00003oooooooooo`0^ooooo`0300000?oooooooooo
02oooooo00<00000ooooooooool0;_ooool00`00003oooooooooo`03ooooo`4000000Oooool001_o
oooo00<00000ooooooooool0ooooool1ooooo`4000000Oooool001_ooooo00<00000ooooooooool0
5ooooolEo`3o0=Cooooo0@000001ooooo`006oooool00`00003oooooooooo`0Dooooo`?o0?l05Ooo
ool9o`3o013ooooo/?l0o`04003o00;o0?l01Oooool10000007ooooo000Kooooo`0300000?l0003o
oooo017ooooo0ol0o`0Poooooa7o0?l0^_ooool1o`00004000000Oooool001_ooooo00<00000oooo
ooooool03oooool2o`3o0>oooooo0@000001ooooo`006oooool00`00003oooooooooo`0<ooooo`?o
0?l0lOooool10000007ooooo000Kooooo`0300000?oooooooooo00[ooooo0_l0o`3dooooo`400000
0Oooool001_ooooo00<00000o`000?ooool02Oooool00ol0o`3oooooooooo`3cooooo`7o00000@00
0001ooooo`006oooool00`00003oooooooooo`07ooooo`;o0?l0moooool10000007ooooo000Doooo
o`;o00001Oooool00`00003oooooooooo`06ooooo`03o`3o0?oooooooooo093ooooo8?oo0017oooo
o`4000000Oooool001?ooooo00Co0000oooooooooooo00001?ooool00`00003oooooooooo`05oooo
o`03o`3o0?oooooooooo07;ooooo7ooo000Poooooa3oo`006_ooool70?l000goo`000`3o00000ooo
003oooooooooo`03ooooo`4000000Oooool001?ooooo00Co0000oooooooooooo00001?ooool00`00
003oooooooooo`04ooooo`03o`3o0?oooooooooo05kooooo5Ooo001?ooooo`koo`00303o000Goooo
o`03ool00?oooooooooo00?ooooo0@000001ooooo`004oooool01?l0003oooooooooool00004oooo
o`0300000?l0003ooooo00?ooooo00Co0?l0ool00?oo003oo`00BOoooolEool009[ooooo0Ol00001
0000007ooooo000Cooooo`04o`000?ooooooooooo`0000Cooooo00<00000ooooooooool00oooool0
1?oo0?ooooooooooooooool5ool002Wooooo6ooo002`ooooo`4000000Oooool001?ooooo00Co0000
oooooooooooo00001?ooool00`00003oooooooooo`03ooooo`03ool0oooooooooooo00Kooooo2Ooo
000Booooo`koo`00boooool10000007ooooo000Dooooo`;o00001Oooool00`00003oooooooooo`03
ooooo`03o`00oooo0?oooooo00oooooo4_oo003Iooooo`4000000Oooool001_ooooo00<00000o`00
0?ooool00oooool00ol00?ooo`3oooooo`3iooooo`7o00000@000001ooooo`006oooool00`00003o
ooooooooo`03ooooo`03o`00oooo0?oooooo0?[ooooo0@000001ooooo`006oooool00`00003ooooo
ooooo`04ooooo`03ool0oooooooooooo0?Wooooo0@000001ooooo`006oooool00`00003ooooooooo
o`04ooooo`03ool0oooooooooooo0?Wooooo0@000001ooooo`006oooool00`00003oooooooooo`04
ooooo`03o`00oooo0?oooooo0?Wooooo0@000001ooooo`006oooool00`00003o0000ooooo`04oooo
o`03o`00oooo0?oooooo0?Sooooo0Ol000010000007ooooo000Kooooo`0300000?oooooooooo00Co
oooo00?o003oool0ooooool0nOooool10000007ooooo000Kooooo`0300000?oooooooooo00Cooooo
00?o003oool0ooooool0nOooool10000007ooooo000Kooooo`0300000?oooooooooo00Cooooo00?o
003ooooooooo0?l0nOooool10000007ooooo000Kooooo`0300000?l0003ooooo00Cooooo00?o003o
oooooooo0?l0n?ooool1o`00004000000Oooool001_ooooo00<00000ooooooooool01?ooool00ol0
0?ooooooool0o`3iooooo`4000000Oooool001_ooooo00<00000ooooooooool01Oooool00ol00?oo
o`3oooooo`3hooooo`4000000Oooool001_ooooo00<00000ooooooooool01Oooool00ol00?ooo`3o
ooooo`3hooooo`4000000Oooool001_ooooo00<00000o`000?ooool01Oooool00ol00?ooooooool0
o`3gooooo`7o00000@000001ooooo`006oooool00`00003oooooooooo`05ooooo`03o`00oooooooo
o`3o0?Sooooo0@000001ooooo`000_ooool2o`0000Cooooo00?o0000ooooooooool00oooool2o`00
00?ooooo0ol00005ooooo`0300000?oooooooooo00Gooooo00?o003ooooooooo0?l0n?ooool10000
007ooooo00001Ooooooo0000oooooooooooo000000Sooooo00Ko0000oooooooooooo0000ooooool0
0003ooooo`03o`000?oooooooooo00;ooooo00<00000ooooooooool01Oooool00ol00?ooooooool0
o`3hooooo`4000000Oooool00005ooooool0003oooooooooool000002?ooool01?l0003ooooooooo
ool00005ooooo`03o`000?oooooooooo00;ooooo00<00000ooooooooool01Oooool00ol00?oooooo
ool0o`3hooooo`4000000Oooool00005ooooool0003oooooooooool000002?ooool01?l0003ooooo
ooooool00005ooooo`03o`000?oooooooooo00;ooooo00<00000o`000?ooool01Oooool01?l00?oo
oooooooooooo0?ofooooo`7o00000@000001ooooo`0000Goooooo`000?ooooooooooo`000008oooo
o`05o`000?ooooooooooo`000?ooool01?l00005ooooo`0300000?oooooooooo00Gooooo00Co003o
ooooooooooooo`3omoooool10000007ooooo00001Ooooooo0000oooooooooooo000000Sooooo00Ko
0000oooooooooooo0000ooooool00008ooooo`0300000?oooooooooo00Gooooo00Co003ooooooooo
ooooo`3omoooool10000007ooooo0002ooooo`;o00002_ooool2o`0000;ooooo1Ol00004ooooo`03
00000?oooooooooo00Kooooo00?o003ooooooooo0?l0moooool10000007ooooo000Kooooo`030000
0?l0003ooooo00Kooooo00Co003oooooooooooooo`3omOooool1o`00004000000Oooool001_ooooo
00<00000ooooooooool01_ooool01?l00?oooooooooooooo0?ofooooo`4000000Oooool001_ooooo
00<00000ooooooooool01_ooool01?l00?oooooooooooooo0?ofooooo`4000000Oooool001_ooooo
00<00000ooooooooool01_ooool01?l00?oooooooooooooo0?ofooooo`4000000Oooool001_ooooo
00<00000ooooooooool01_ooool01Ol00?ooooooooooooooooooo`3o0?Gooooo0@000001ooooo`00
6oooool00`00003o0000ooooo`06ooooo`05o`00oooooooooooooooooooo0?l0m?ooool1o`000040
00000Oooool001_ooooo00<00000ooooooooool01_ooool01Ol00?ooooooooooooooooooo`3o0?Go
oooo0@000001ooooo`006oooool00`00003oooooooooo`06ooooo`05o`00oooooooooooooooooooo
0?l0mOooool10000007ooooo000Kooooo`0300000?oooooooooo00Kooooo00?o003oooooooooool0
0_ooool00ooo0?ooooooooooo`3booooo`4000000Oooool001_ooooo00<00000o`000?ooool01ooo
ool01Ol00?ooooooooooooooooooo`3o0??ooooo0Ol000010000007ooooo000Kooooo`0300000?oo
oooooooo00Oooooo00Go003oooooooooooooooooool0o`3dooooo`4000000Oooool001_ooooo00<0
0000ooooooooool01oooool01Ol00?ooooooooooooooooooo`3o0?Cooooo0@000001ooooo`006ooo
ool00`00003oooooooooo`07ooooo`03o`00oooooooooooo00;ooooo00?oo`3oooooooooool0lOoo
ool10000007ooooo000Kooooo`0300000?oooooooooo00Oooooo00?o003oooooooooool00_ooool0
0ooo0?ooooooooooo`3aooooo`4000000Oooool001_ooooo00<00000o`000?ooool01oooool00ol0
0?ooooooooooo`02ooooo`03ool0oooooooooooo0?3ooooo0Ol000010000007ooooo000Kooooo`03
00000?oooooooooo00Oooooo00?o003oooooooooool00_ooool00ooo0?ooooooooooo`3aooooo`40
00000Oooool000Sooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0ol00005ooooo`030000
0?oooooooooo00Oooooo00?o003oooooooooool00oooool00ooo0?ooooooooooo`3`ooooo`400000
0Oooool000Oooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`04ooooo`030000
0?oooooooooo00Oooooo00?o003oooooooooool00oooool00ooo0?ooooooooooo`3`ooooo`400000
0Oooool000Oooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`04ooooo`030000
0?l0003ooooo00Sooooo00?o003oooooooooool00_ooool00ooo0?ooooooooooo`3_ooooo`7o0000
0@000001ooooo`001oooool01?l0003oooooooooool00009ooooo`03o`000?oooooooooo00Cooooo
00<00000ooooooooool02?ooool00ol00?ooooooooooo`03ooooo`03ool0oooooooooooo0>oooooo
0@000001ooooo`001oooool01?l0003oooooooooool00009ooooo`03o`000?oooooooooo00Cooooo
00<00000ooooooooool02?ooool00ol00?ooooooooooo`03ooooo`03ool0oooooooooooo0>oooooo
0@000001ooooo`001oooool01?l0003oooooooooool00008ooooo`;o00001_ooool00`00003ooooo
ooooo`08ooooo`03o`00oooooooooooo00?ooooo00?oo`3oooooooooool0koooool10000007ooooo
0008ooooo`;o00002_ooool00ol0003oooooooooo`04ooooo`0300000?l0003ooooo00Sooooo00?o
003oooooooooool01?ooool00ooo0?ooooooooooo`3]ooooo`7o00000@000001ooooo`006oooool0
0`00003oooooooooo`08ooooo`03o`00oooooooooooo00Cooooo00?oo`3oooooooooool0k_ooool1
0000007ooooo000Kooooo`0300000?oooooooooo00Sooooo00?o003oooooooooool01?ooool00ooo
0?ooooooooooo`3^ooooo`4000000Oooool001_ooooo00<00000ooooooooool02?ooool00ol00?oo
ooooooooo`05ooooo`03ool0oooooooooooo0>gooooo0@000001ooooo`006oooool00`00003ooooo
ooooo`08ooooo`03o`00oooooooooooo00Gooooo00?oo`3oooooooooool0kOooool10000007ooooo
000Kooooo`0300000?l0003ooooo00Wooooo00?o003oooooooooool01?ooool00ooo0?oooooooooo
o`3/ooooo`7o00000@000001ooooo`006oooool00`00003oooooooooo`09ooooo`03o`00oooooooo
oooo00Gooooo00?oo`3oooooooooool0k?ooool10000007ooooo000Kooooo`0300000?oooooooooo
00Wooooo00?o003oooooooooool01Oooool00ooo0?ooooooooooo`3/ooooo`4000000Oooool001_o
oooo00<00000ooooooooool02Oooool00ol00?ooooooooooo`05ooooo`03ool0oooooooooooo0>co
oooo0@000001ooooo`006oooool00`00003o0000ooooo`09ooooo`03o`00oooooooooooo00Kooooo
00?oo`3oooooooooool0j_ooool1o`00004000000Oooool001_ooooo00<00000ooooooooool02Ooo
ool00ol00?ooooooooooo`06ooooo`03ool0oooooooooooo0>_ooooo0@000001ooooo`006oooool0
0`00003oooooooooo`09ooooo`03o`00oooooooooooo00Oooooo00?oo`3oooooooooool0j_ooool1
0000007ooooo000Kooooo`0300000?oooooooooo00Wooooo00?o003oooooooooool01oooool00ooo
0?ooooooooooo`3Zooooo`4000000Oooool001_ooooo00<00000ooooooooool02_ooool00ol00?oo
ooooooooo`07ooooo`03ool0oooooooooooo0>Wooooo0@000001ooooo`006oooool00`00003o0000
ooooo`0:ooooo`03o`00oooooooooooo00Oooooo00?oo`3oooooooooool0j?ooool1o`0000400000
0Oooool001_ooooo00<00000ooooooooool02_ooool00ol00?ooooooooooo`08ooooo`03ool0oooo
oooooooo0>Sooooo0@000001ooooo`000_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool3
o`0000?ooooo0ol00005ooooo`0300000?oooooooooo00[ooooo00?o003oooooooooool02?ooool0
0ooo0?ooooooooooo`3Xooooo`4000000Oooool00005ooooool0003oooooooooool000002Oooool0
1Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00_ooool00`00003ooooooooo
o`0:ooooo`03o`00oooooooooooo00Wooooo00?oo`3oooooooooool0ioooool10000007ooooo0000
1Ooooooo0000oooooooooooo000000Wooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`02ooooo`0300000?l0003ooooo00[ooooo00?o003oooooooooool02_ooool00ooo0?oooooooooo
o`3Uooooo`7o00000@000001ooooo`0000Goooooo`000?ooooooooooo`000009ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool00_ooool00`00003oooooooooo`0:ooooo`03o`00oooo
oooooooo00_ooooo00?oo`3oooooooooool0iOooool10000007ooooo00001Ooooooo0000oooooooo
oooo000000Wooooo00Co0000oooooooooooooooo1?l00005ooooo`0300000?oooooooooo00_ooooo
00?o003oooooooooool02_ooool00ooo0?ooooooooooo`3Uooooo`4000000Oooool00005ooooool0
003oooooooooool000002?ooool2o`0000?ooooo00?o0000ooooooooool01_ooool00`00003ooooo
ooooo`0;ooooo`03o`00oooooooooooo00_ooooo00?oo`3oooooooooool0i?ooool10000007ooooo
0002ooooo`;o00002_ooool01?l0003oooooooooooooool5o`0000Cooooo00<00000ooooooooool0
2oooool00ol00?ooooooooooo`0<ooooo`03ool0oooooooooooo01Oooooo`Ooo0?l30?l0o`?oo`3o
1Oooool10000007ooooo000Kooooo`0300000?l0003ooooo00_ooooo00?o003oooooooooool03Ooo
ool00ooo0?ooooooooooo`0Cooooo`?oo`3oboooool1o`00004000000Oooool001_ooooo00<00000
ooooooooool02oooool00ol00?ooooooooooo`0>ooooo`03ool0oooooooooooo013ooooo0_oo0?o?
ooooo`4000000Oooool001_ooooo00<00000ooooooooool02oooool00ol00?ooooooooooo`0?oooo
o`03ool0oooooooooooo00gooooo0_oo0?oAooooo`4000000Oooool001_ooooo00<00000oooooooo
ool02oooool00ol00?ooooooooooo`0@ooooo`?oo`3o2_ooool2ool0om?ooooo0@000001ooooo`00
6oooool00`00003o0000ooooo`0;ooooo`03o`00oooooooooooo01?ooooo2_oo0?oDooooo`7o0000
0@000001ooooo`006oooool00`00003oooooooooo`0<ooooo`03o`00oooooooooooo0?7ooooo0@00
0001ooooo`006oooool00`00003oooooooooo`0<ooooo`03o`00oooooooooooo0?7ooooo0@000001
ooooo`006oooool00`00003oooooooooo`0<ooooo`03o`00oooooooooooo0?7ooooo0@000001oooo
o`006oooool00`00003o0000ooooo`0<ooooo`03o`00oooooooooooo0?3ooooo0Ol000010000007o
oooo000Kooooo`0300000?oooooooooo00cooooo00?o003oooooooooool0lOooool10000007ooooo
000Kooooo`0300000?oooooooooo00cooooo00?o003oooooooooool0lOooool10000007ooooo000K
ooooo`0300000?oooooooooo00cooooo00?o003oooooooooool0lOooool10000007ooooo000Koooo
o`0300000?oooooooooo00cooooo00?o003oooooooooool0lOooool10000007ooooo000Kooooo`03
00000?l0003ooooo00gooooo00?o003oooooooooool0koooool1o`00004000000Oooool001_ooooo
00<00000ooooooooool03Oooool00ol00?ooooooooooo`3`ooooo`4000000Oooool000Sooooo0_l0
0004ooooo`04o`000?ooooooooooooooo`Go00001?ooool00`00003oooooooooo`0=ooooo`03o`00
oooooooooooo0?3ooooo0@000001ooooo`001oooool01?l0003oooooooooool00008ooooo`04o`00
0?ooooooooooo`0000Cooooo00<00000ooooooooool03Oooool00ol00?ooooooooooo`3`ooooo`40
00000Oooool000Oooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`04ooooo`03
00000?l0003ooooo00gooooo00?o003oooooooooool0koooool1o`00004000000Oooool000Oooooo
00Co0000oooooooooooo00002_ooool00ol0003oooooooooo`03ooooo`0300000?oooooooooo00go
oooo00?o003oooooooooool0l?ooool10000007ooooo0007ooooo`04o`000?ooooooooooo`0000Oo
oooo00Go0000ooooooooooooooooo`000004ooooo`0300000?oooooooooo00kooooo00?o003ooooo
ooooool0koooool10000007ooooo0007ooooo`04o`000?ooooooooooo`0000Oooooo00Go0000oooo
ooooooooooooo`000004ooooo`0300000?oooooooooo00kooooo00?o003oooooooooool0koooool1
0000007ooooo0008ooooo`;o00002Oooool3o`0000Gooooo00<00000ooooooooool03_ooool00ol0
0?ooooooooooo`3_ooooo`4000000Oooool001_ooooo00<00000o`000?ooool03_ooool00ol00?oo
ooooooooo`3^ooooo`7o00000@000001ooooo`006oooool00`00003oooooooooo`0>ooooo`03o`00
oooooooooooo0>oooooo0@000001ooooo`006oooool00`00003oooooooooo`0?ooooo`03o`00oooo
oooooooo0>kooooo0@000001ooooo`006oooool00`00003oooooooooo`0?ooooo`03o`00oooooooo
oooo0>kooooo0@000001ooooo`006oooool00`00003o0000ooooo`0?ooooo`03o`00oooooooooooo
0>gooooo0Ol000010000007ooooo000Kooooo`0300000?oooooooooo00oooooo00?o003ooooooooo
ool0k_ooool10000007ooooo000Kooooo`0300000?oooooooooo00oooooo00?o003oooooooooool0
k_ooool10000007ooooo000Kooooo`0300000?oooooooooo013ooooo00?o003oooooooooool0kOoo
ool10000007ooooo000Kooooo`0300000?oooooooooo013ooooo00?o003oooooooooool0kOooool1
0000007ooooo000Kooooo`0300000?l0003ooooo013ooooo00?o003oooooooooool0k?ooool1o`00
004000000Oooool001_ooooo00<00000ooooooooool04?ooool00ol00?ooooooooooo`3]ooooo`40
00000Oooool001_ooooo00<00000ooooooooool04?ooool00ol00?ooooooooooo`3]ooooo`400000
0Oooool001_ooooo00<00000ooooooooool04?ooool00ol00?ooooooooooo`3]ooooo`4000000Ooo
ool001_ooooo00<00000o`000?ooool04Oooool00ol00?ooooooooooo`3[ooooo`7o00000@000001
ooooo`006oooool00`00003oooooooooo`0Aooooo`03o`00oooooooooooo0>cooooo0@000001oooo
o`000_ooool2o`0000Cooooo00Co0000oooooooooooooooo1Ol00002ooooo`?o00001Oooool00`00
003oooooooooo`0Aooooo`03o`00oooooooooooo0>cooooo0@000001ooooo`0000Goooooo`000?oo
ooooooooo`000008ooooo`06o`000?ooooooooooo`000?oooooo00000oooool00ol0003ooooooooo
o`02ooooo`0300000?oooooooooo017ooooo00?o003oooooooooool0k?ooool10000007ooooo0000
1Ooooooo0000oooooooooooo000000Wooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`02ooooo`0300000?l0003ooooo017ooooo00?o003oooooooooool0joooool1o`00004000000Ooo
ool00005ooooool0003oooooooooool000002_ooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00;ooooo00<00000ooooooooool04Oooool00ol00?ooooooooooo`3/ooooo`4000000Ooo
ool00005ooooool0003oooooooooool000001oooool01_l0003ooooooooooooooooo0000ooooo`Co
00001Oooool00`00003oooooooooo`0Booooo`03o`00oooooooooooo0>_ooooo0@000001ooooo`00
00Goooooo`000?ooooooooooo`000007ooooo`07o`000?ooooooooooooooool0003oooooo`000008
ooooo`0300000?oooooooooo01;ooooo00?o003oooooooooool0joooool10000007ooooo0002oooo
o`;o00002Oooool3o`0000;ooooo1Ol00004ooooo`0300000?oooooooooo01;ooooo00?o003ooooo
ooooool0joooool10000007ooooo000Kooooo`0300000?l0003ooooo01;ooooo00?o003ooooooooo
ool0j_ooool1o`00004000000Oooool001_ooooo00<00000ooooooooool04_ooool00ol00?oooooo
ooooo`3[ooooo`4000000Oooool001_ooooo00<00000ooooooooool04oooool00ol00?oooooooooo
o`3Zooooo`4000000Oooool001_ooooo00<00000ooooooooool04oooool00ol00?ooooooooooo`0O
ooooo`go003o__ooool10000007ooooo000Kooooo`0300000?l0003ooooo01?ooooo00?o003ooooo
ooooool07_ooool2o`00o`cooooo1?l00?niooooo`7o00000@000001ooooo`006oooool00`00003o
ooooooooo`0Dooooo`03o`00oooooooooooo01cooooo00?o003oooooooooool04?ooool6o`00oeSo
ooooDOl00?l20000o`04o`00o`000?l0003o0000o`Gooooo0@000001ooooo`006oooool00`00003o
ooooooooo`0Dooooo`03o`00oooooooooooo01cooooo00?o003oooooooooool05_oooomHo`00oeco
oooo0@000001ooooo`006oooool00`00003oooooooooo`0Dooooo`03o`00oooooooooooo01_ooooo
00?o003oooooooooool0boooool10000007ooooo000Kooooo`0300000?oooooooooo01Gooooo00?o
003oooooooooool06Oooool00ol00?ooooooooooo`3<ooooo`4000000Oooool001_ooooo00<00000
o`000?ooool05Oooool00ol00?ooooooooooo`0Iooooo`03o`00oooooooooooo0<_ooooo0Ol00001
0000007ooooo000Kooooo`0300000?oooooooooo01Gooooo00?o003oooooooooool06?ooool00ol0
0?ooooooooooo`3=ooooo`4000000Oooool001_ooooo00<00000ooooooooool05Oooool00ol00?oo
ooooooooo`0Hooooo`03o`00oooooooooooo0<gooooo0@000001ooooo`006oooool00`00003ooooo
ooooo`0Fooooo`03o`00oooooooooooo01Oooooo00?o003oooooooooool0cOooool10000007ooooo
000Kooooo`0300000?l0003ooooo01Kooooo00?o003oooooooooool05_ooool00ol00?oooooooooo
o`3=ooooo`7o00000@000001ooooo`006oooool00`00003oooooooooo`0Fooooo`03o`00oooooooo
oooo01Gooooo00?o003oooooooooool0coooool10000007ooooo0008ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`?o00001Oooool00`00003oooooooooo`0Fooooo`03o`00oooooooooooo
01Cooooo00?o003oooooooooool0d?ooool10000007ooooo0007ooooo`04o`000?ooooooooooo`00
00Oooooo00Go0000ooooooooooooooooo`000004ooooo`0300000?oooooooooo01Oooooo00?o003o
ooooooooool04oooool00ol00?ooooooooooo`3@ooooo`4000000Oooool000Oooooo00Co0000oooo
oooooooo00002oooool00ol0003oooooooooo`02ooooo`0300000?oooooooooo01Sooooo00?o003o
ooooooooool04Oooool00ol00?ooooooooooo`3Aooooo`4000000Oooool000Oooooo00Co0000oooo
oooooooo00002Oooool2o`0000Gooooo00<00000o`000?ooool06?ooool00ol00?ooooooooooo`0@
ooooo`03o`00oooooooooooo0=7ooooo0Ol000010000007ooooo0007ooooo`04o`000?oooooooooo
o`0000_ooooo00?o0000ooooooooool00_ooool00`00003oooooooooo`0Iooooo`03o`00oooooooo
oooo00kooooo00?o003oooooooooool0doooool10000007ooooo0007ooooo`04o`000?oooooooooo
o`0000Oooooo00Go0000ooooooooooooooooo`000004ooooo`0300000?oooooooooo01Wooooo00?o
003oooooooooool03_ooool00ol00?ooooooooooo`3Cooooo`4000000Oooool000Sooooo0_l00009
ooooo`?o00001Oooool00`00003oooooooooo`0Jooooo`03o`00oooooooooooo00cooooo00?o003o
ooooooooool0e?ooool10000007ooooo000Kooooo`0300000?l0003ooooo01_ooooo00?o003ooooo
ooooool02_ooool00ol00?ooooooooooo`3Dooooo`7o00000@000001ooooo`006oooool00`00003o
ooooooooo`0Kooooo`03o`00oooooooooooo00[ooooo00?o003oooooooooool0eOooool10000007o
oooo000Kooooo`0300000?oooooooooo01cooooo00?o003oooooooooool01oooool2o`00omSooooo
0@000001ooooo`006oooool00`00003oooooooooo`0Mooooo`03o`00oooooooooooo00Gooooo00?o
003oooooooooool0f?ooool10000007ooooo000Kooooo`0300000?l0003ooooo01kooooo0ol00?l0
1?oooooo003oo`00ool00?oJooooo`7o00000@000001ooooo`006oooool00`00003oooooooooo`0P
ooooo`;o003og_ooool10000007ooooo000Kooooo`0300000?oooooooooo0?oooooo0Oooool10000
007ooooo000Kooooo`0300000?oooooooooo00?ooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`0:ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`0:ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`0:ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02_ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02_ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02_ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02_ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00?ooooo0@000001ooooo`006ooooooo000000D00000
0Oooool00?oooooo8Oooool00?oooooo8Oooool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.35193, -0.0658981, \
0.0404426, 0.00225818}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Applied torques", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(gtorques = 
        PlotMultipleTrajectory[torques, {0, tmax}, actualStyle, 
          imgSize];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 288 177.994 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0952381 0.0571498 0.0160805 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.21429 -0.0125 -3 -9 ]
[.21429 -0.0125 3 0 ]
[.40476 -0.0125 -3 -9 ]
[.40476 -0.0125 3 0 ]
[.59524 -0.0125 -3 -9 ]
[.59524 -0.0125 3 0 ]
[.78571 -0.0125 -3 -9 ]
[.78571 -0.0125 3 0 ]
[.97619 -0.0125 -6 -9 ]
[.97619 -0.0125 6 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .05715 -6 -4.5 ]
[-0.0125 .05715 0 4.5 ]
[-0.0125 .13755 -6 -4.5 ]
[-0.0125 .13755 0 4.5 ]
[-0.0125 .21795 -12 -4.5 ]
[-0.0125 .21795 0 4.5 ]
[-0.0125 .29836 -12 -4.5 ]
[-0.0125 .29836 0 4.5 ]
[-0.0125 .37876 -12 -4.5 ]
[-0.0125 .37876 0 4.5 ]
[-0.0125 .45916 -12 -4.5 ]
[-0.0125 .45916 0 4.5 ]
[-0.0125 .53956 -12 -4.5 ]
[-0.0125 .53956 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.21429 0 m
.21429 .00625 L
s
[(2)] .21429 -0.0125 0 1 Mshowa
.40476 0 m
.40476 .00625 L
s
[(4)] .40476 -0.0125 0 1 Mshowa
.59524 0 m
.59524 .00625 L
s
[(6)] .59524 -0.0125 0 1 Mshowa
.78571 0 m
.78571 .00625 L
s
[(8)] .78571 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(10)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.5 0 m
.5 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .05715 m
.00625 .05715 L
s
[(0)] -0.0125 .05715 1 0 Mshowa
0 .13755 m
.00625 .13755 L
s
[(5)] -0.0125 .13755 1 0 Mshowa
0 .21795 m
.00625 .21795 L
s
[(10)] -0.0125 .21795 1 0 Mshowa
0 .29836 m
.00625 .29836 L
s
[(15)] -0.0125 .29836 1 0 Mshowa
0 .37876 m
.00625 .37876 L
s
[(20)] -0.0125 .37876 1 0 Mshowa
0 .45916 m
.00625 .45916 L
s
[(25)] -0.0125 .45916 1 0 Mshowa
0 .53956 m
.00625 .53956 L
s
[(30)] -0.0125 .53956 1 0 Mshowa
.125 Mabswid
0 .07323 m
.00375 .07323 L
s
0 .08931 m
.00375 .08931 L
s
0 .10539 m
.00375 .10539 L
s
0 .12147 m
.00375 .12147 L
s
0 .15363 m
.00375 .15363 L
s
0 .16971 m
.00375 .16971 L
s
0 .18579 m
.00375 .18579 L
s
0 .20187 m
.00375 .20187 L
s
0 .23404 m
.00375 .23404 L
s
0 .25012 m
.00375 .25012 L
s
0 .2662 m
.00375 .2662 L
s
0 .28228 m
.00375 .28228 L
s
0 .31444 m
.00375 .31444 L
s
0 .33052 m
.00375 .33052 L
s
0 .3466 m
.00375 .3466 L
s
0 .36268 m
.00375 .36268 L
s
0 .39484 m
.00375 .39484 L
s
0 .41092 m
.00375 .41092 L
s
0 .427 m
.00375 .427 L
s
0 .44308 m
.00375 .44308 L
s
0 .47524 m
.00375 .47524 L
s
0 .49132 m
.00375 .49132 L
s
0 .5074 m
.00375 .5074 L
s
0 .52348 m
.00375 .52348 L
s
0 .55565 m
.00375 .55565 L
s
0 .57173 m
.00375 .57173 L
s
0 .58781 m
.00375 .58781 L
s
0 .60389 m
.00375 .60389 L
s
0 .04107 m
.00375 .04107 L
s
0 .02499 m
.00375 .02499 L
s
0 .00891 m
.00375 .00891 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.21429 .61178 m
.21429 .61803 L
s
.40476 .61178 m
.40476 .61803 L
s
.59524 .61178 m
.59524 .61803 L
s
.78571 .61178 m
.78571 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.07143 .61428 m
.07143 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.16667 .61428 m
.16667 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.5 .61428 m
.5 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .05715 m
1 .05715 L
s
.99375 .13755 m
1 .13755 L
s
.99375 .21795 m
1 .21795 L
s
.99375 .29836 m
1 .29836 L
s
.99375 .37876 m
1 .37876 L
s
.99375 .45916 m
1 .45916 L
s
.99375 .53956 m
1 .53956 L
s
.125 Mabswid
.99625 .07323 m
1 .07323 L
s
.99625 .08931 m
1 .08931 L
s
.99625 .10539 m
1 .10539 L
s
.99625 .12147 m
1 .12147 L
s
.99625 .15363 m
1 .15363 L
s
.99625 .16971 m
1 .16971 L
s
.99625 .18579 m
1 .18579 L
s
.99625 .20187 m
1 .20187 L
s
.99625 .23404 m
1 .23404 L
s
.99625 .25012 m
1 .25012 L
s
.99625 .2662 m
1 .2662 L
s
.99625 .28228 m
1 .28228 L
s
.99625 .31444 m
1 .31444 L
s
.99625 .33052 m
1 .33052 L
s
.99625 .3466 m
1 .3466 L
s
.99625 .36268 m
1 .36268 L
s
.99625 .39484 m
1 .39484 L
s
.99625 .41092 m
1 .41092 L
s
.99625 .427 m
1 .427 L
s
.99625 .44308 m
1 .44308 L
s
.99625 .47524 m
1 .47524 L
s
.99625 .49132 m
1 .49132 L
s
.99625 .5074 m
1 .5074 L
s
.99625 .52348 m
1 .52348 L
s
.99625 .55565 m
1 .55565 L
s
.99625 .57173 m
1 .57173 L
s
.99625 .58781 m
1 .58781 L
s
.99625 .60389 m
1 .60389 L
s
.99625 .04107 m
1 .04107 L
s
.99625 .02499 m
1 .02499 L
s
.99625 .00891 m
1 .00891 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
1 0 0 r
.5 Mabswid
.02381 .05715 m
.02605 .05952 L
.02846 .06152 L
.03072 .06297 L
.03279 .06399 L
.03521 .06489 L
.03784 .06559 L
.04032 .06606 L
.04262 .06639 L
.0472 .06687 L
.05224 .06731 L
.06244 .06822 L
.06782 .0687 L
.07293 .06907 L
.07574 .06924 L
.07723 .06931 L
.07881 .06937 L
.08028 .06942 L
.08161 .06945 L
.08225 .06946 L
.08294 .06947 L
.0842 .06948 L
.08547 .06948 L
.08666 .06947 L
.08796 .06945 L
.08869 .06944 L
.08936 .06942 L
.0907 .06938 L
.09216 .06933 L
.09481 .0692 L
.09989 .06885 L
.10458 .06843 L
.12483 .06617 L
.13475 .06517 L
.13909 .0648 L
.14385 .06446 L
.14812 .06421 L
.1527 .06399 L
.15752 .06382 L
.16201 .06371 L
.16455 .06366 L
.16729 .06361 L
.16971 .06358 L
.17227 .06356 L
.17712 .06352 L
.1816 .0635 L
.18644 .06347 L
.19103 .06344 L
.19633 .06339 L
.20117 .06333 L
Mistroke
.20572 .06326 L
.20993 .06318 L
.21948 .06295 L
.23008 .06261 L
.24005 .06222 L
.25867 .06141 L
.29727 .05966 L
.33832 .05792 L
.37786 .05625 L
.41589 .05464 L
.45636 .05298 L
.49533 .05149 L
.53674 .05003 L
.57664 .04875 L
.61502 .04766 L
.63444 .04716 L
.65585 .04664 L
.66562 .04642 L
.67614 .04619 L
.68099 .04609 L
.68608 .04598 L
.68818 .04594 L
.68933 .04592 L
.69042 .04588 L
.69171 .04669 L
.69288 .0474 L
.69517 .04867 L
.69982 .05076 L
.70245 .05168 L
.70492 .05238 L
.70752 .05296 L
.70989 .05337 L
.7112 .05355 L
.71263 .05371 L
.71397 .05383 L
.71522 .05392 L
.71629 .05399 L
.71743 .05404 L
.7187 .05408 L
.71987 .0541 L
.72102 .05411 L
.72228 .05411 L
.72359 .0541 L
.72481 .05407 L
.72597 .05405 L
.72706 .05401 L
.7291 .05394 L
.73369 .05374 L
.74339 .05327 L
.74871 .05305 L
Mistroke
.75368 .05289 L
.75791 .05278 L
.76246 .0527 L
.76505 .05266 L
.76751 .05263 L
.76971 .05262 L
.77212 .0526 L
.77345 .0526 L
.77468 .05259 L
.77602 .05259 L
.7767 .05259 L
.77744 .05259 L
.77869 .05258 L
.77939 .05258 L
.78003 .05258 L
.78128 .05258 L
.78245 .05258 L
.78358 .05259 L
.78478 .05259 L
.78694 .05259 L
.79181 .0526 L
.80191 .05263 L
.80721 .05264 L
.81279 .05265 L
.81765 .05266 L
.8204 .05266 L
.82296 .05266 L
.82527 .05267 L
.82746 .05267 L
.82867 .05267 L
.82998 .05267 L
.83117 .05267 L
.83227 .05267 L
.83343 .05267 L
.83448 .05267 L
.83564 .05267 L
.8369 .05267 L
.8381 .05267 L
.83941 .05267 L
.84064 .05267 L
.84179 .05267 L
.84402 .05267 L
.84604 .05267 L
.85056 .05267 L
.85995 .05267 L
.86508 .05267 L
.86986 .05266 L
.87504 .05266 L
.87802 .05266 L
.87931 .05266 L
Mistroke
.88072 .05266 L
.88201 .05266 L
.8832 .05266 L
.88441 .05266 L
.88554 .05266 L
.88673 .05266 L
.88804 .05266 L
.88933 .05266 L
.89003 .05266 L
.8907 .05266 L
.89135 .05266 L
.89205 .05266 L
.89327 .05266 L
.89451 .05266 L
.89566 .05266 L
.89691 .05266 L
.89762 .05266 L
.89829 .05266 L
.89899 .05266 L
.89963 .05266 L
.90037 .05266 L
.90107 .05266 L
.90226 .05266 L
.90352 .05266 L
.9046 .05266 L
.9058 .05266 L
.90707 .05266 L
.90845 .05266 L
.91094 .05266 L
.92053 .05266 L
.92521 .05266 L
.92946 .05266 L
.9306 .05266 L
.93184 .05266 L
.933 .05266 L
.93408 .05266 L
.93522 .05266 L
.93648 .05266 L
.93779 .05266 L
.93902 .05266 L
.93967 .05266 L
.94037 .05266 L
.94166 .05266 L
.94291 .05266 L
.94407 .05266 L
.94949 .05266 L
.97058 .05266 L
.97619 .05266 L
Mfstroke
0 0 1 r
.02381 .05715 m
.04262 .27864 L
.0522 .4105 L
.05746 .47002 L
.06244 .51559 L
.06506 .53528 L
.0679 .55344 L
.07286 .57764 L
.0753 .58631 L
.0779 .59336 L
.08011 .59777 L
.08137 .59967 L
.08255 .60106 L
.08391 .60224 L
.08468 .60271 L
.08539 .60302 L
.0867 .60331 L
.08743 .60332 L
.0881 .60323 L
.08935 .60284 L
.09006 .60249 L
.09071 .6021 L
.09317 .59998 L
.09453 .59842 L
.09581 .59673 L
.09872 .5922 L
.10458 .58075 L
.12322 .53724 L
.13305 .51687 L
.14349 .50026 L
.14873 .49409 L
.15152 .4914 L
.15452 .48893 L
.15727 .48706 L
.15979 .48564 L
.16235 .48447 L
.16473 .48362 L
.16699 .48302 L
.16826 .48276 L
.16941 .48256 L
.17067 .4824 L
.17139 .48233 L
.17204 .48228 L
.17325 .48222 L
.17453 .4822 L
.17575 .48222 L
.17707 .48228 L
.17831 .48238 L
.17944 .48249 L
.18222 .48288 L
Mistroke
.18485 .48337 L
.19446 .48586 L
.20469 .48908 L
.21428 .49201 L
.22309 .49432 L
.23341 .49637 L
.23883 .49716 L
.24159 .49748 L
.24455 .49777 L
.24715 .49799 L
.24953 .49815 L
.25194 .49828 L
.25417 .49838 L
.25677 .49846 L
.25814 .4985 L
.2596 .49852 L
.26086 .49854 L
.26204 .49855 L
.26333 .49856 L
.26406 .49856 L
.26473 .49856 L
.26601 .49855 L
.26722 .49854 L
.26831 .49853 L
.26948 .49851 L
.27213 .49847 L
.27456 .49841 L
.28369 .49813 L
.2944 .49774 L
.30441 .49739 L
.31424 .49711 L
.32321 .49692 L
.32832 .49685 L
.33317 .49679 L
.33583 .49677 L
.33829 .49676 L
.34099 .49674 L
.34235 .49674 L
.34386 .49673 L
.34507 .49673 L
.3462 .49673 L
.34744 .49673 L
.34814 .49673 L
.34877 .49673 L
.35004 .49673 L
.35076 .49673 L
.35143 .49673 L
.35265 .49673 L
.35394 .49673 L
.35621 .49674 L
Mistroke
.35834 .49674 L
.36318 .49676 L
.38403 .49685 L
.389 .49687 L
.39442 .49689 L
.3992 .49691 L
.40182 .49691 L
.40429 .49692 L
.40672 .49692 L
.40799 .49693 L
.40934 .49693 L
.41047 .49693 L
.41172 .49693 L
.41285 .49693 L
.4139 .49693 L
.41506 .49693 L
.41616 .49693 L
.41714 .49693 L
.41821 .49693 L
.41937 .49693 L
.42061 .49693 L
.42166 .49693 L
.42282 .49693 L
.42517 .49693 L
.42775 .49692 L
.43293 .49691 L
.43772 .4969 L
.44218 .49689 L
.46305 .49681 L
.50211 .49661 L
.54363 .49634 L
.58363 .49603 L
.62212 .49572 L
.64162 .49557 L
.65196 .4955 L
.66306 .49544 L
.6737 .49538 L
.68343 .49534 L
.68811 .49533 L
.69058 .49532 L
.6932 .49532 L
.69573 .49531 L
.69802 .49531 L
.70248 .49531 L
.70482 .4953 L
.70738 .4953 L
.71254 .4953 L
.71732 .4953 L
.72178 .4953 L
.72449 .4953 L
Mistroke
.72582 .4953 L
.72705 .4953 L
.72814 .4953 L
.72935 .4953 L
.73056 .4953 L
.73185 .4953 L
.73256 .4953 L
.73324 .4953 L
.73452 .4953 L
.73524 .4953 L
.7359 .4953 L
.73663 .4953 L
.73741 .4953 L
.73881 .4953 L
.74014 .4953 L
.74262 .4953 L
.74727 .4953 L
.75237 .4953 L
.76269 .4953 L
.78116 .49531 L
.79135 .49531 L
.80066 .49531 L
.8058 .49531 L
.8087 .49531 L
.81139 .49531 L
.81374 .49531 L
.81633 .49531 L
.81763 .49531 L
.81831 .49531 L
.81903 .49531 L
.81966 .49531 L
.82035 .49531 L
.82157 .49531 L
.82289 .49531 L
.8241 .49531 L
.82541 .49531 L
.82615 .49531 L
.82685 .49531 L
.82802 .49531 L
.82927 .49531 L
.8305 .49531 L
.83119 .49531 L
.83184 .49531 L
.83305 .49531 L
.83434 .49531 L
.83558 .49531 L
.83671 .49531 L
.84117 .49531 L
.86212 .49531 L
.87149 .49531 L
Mistroke
.88139 .49531 L
.88657 .49531 L
.88799 .49531 L
.88877 .49531 L
.88951 .49531 L
.89019 .49531 L
.89094 .49531 L
.89225 .49531 L
.89473 .49531 L
.89603 .49531 L
.89668 .49531 L
.89739 .49531 L
.8987 .49531 L
.89991 .49531 L
.90103 .49531 L
.90222 .49531 L
.90344 .49531 L
.90476 .49531 L
.90601 .49531 L
.90716 .49531 L
.9082 .49531 L
.90934 .49531 L
.91164 .49531 L
.91423 .49531 L
.91701 .49531 L
.91836 .49531 L
.91963 .49531 L
.92078 .49531 L
.92203 .49531 L
.92332 .49531 L
.92454 .49531 L
.92523 .49531 L
.92587 .49531 L
.92658 .49531 L
.92731 .49531 L
.92804 .49531 L
.92872 .49531 L
.92951 .49531 L
.93023 .49531 L
.93153 .49531 L
.93226 .49531 L
.93294 .49531 L
.93405 .49531 L
.93528 .49531 L
.93656 .49531 L
.93776 .49531 L
.93844 .49531 L
.93909 .49531 L
.94031 .49531 L
.94304 .49531 L
Mistroke
.97619 .49531 L
Mfstroke
1 0 1 r
.02381 .05715 m
.02605 .13582 L
.02846 .19523 L
.03053 .23687 L
.03279 .27356 L
.03543 .30708 L
.03793 .33087 L
.04037 .34798 L
.04262 .35942 L
.04376 .36386 L
.04498 .36774 L
.04604 .37043 L
.0472 .37273 L
.0479 .37383 L
.04855 .37467 L
.04983 .37585 L
.051 .37643 L
.05224 .3766 L
.05293 .37652 L
.05357 .37635 L
.05426 .37605 L
.055 .37564 L
.05623 .37471 L
.05759 .37343 L
.06244 .36723 L
.08278 .33695 L
.09418 .32403 L
.10458 .31513 L
.1094 .31191 L
.11452 .30913 L
.11719 .30794 L
.12014 .30683 L
.12285 .30599 L
.12534 .30536 L
.12772 .30489 L
.13022 .30452 L
.13152 .30438 L
.13273 .30427 L
.13406 .30419 L
.13481 .30415 L
.13548 .30413 L
.1367 .30411 L
.13783 .30411 L
.13907 .30412 L
.14038 .30417 L
.14152 .30422 L
.14258 .30429 L
.145 .30449 L
.14998 .30506 L
.15541 .30587 L
Mistroke
.16529 .30756 L
.17484 .30914 L
.18383 .31038 L
.1888 .31093 L
.19417 .31141 L
.19905 .31175 L
.20354 .31198 L
.20606 .31208 L
.20841 .31216 L
.21099 .31222 L
.21244 .31225 L
.21375 .31227 L
.21513 .31229 L
.21641 .3123 L
.21707 .3123 L
.2178 .31231 L
.21857 .31231 L
.21929 .31231 L
.2206 .31231 L
.22181 .31231 L
.22244 .3123 L
.22313 .3123 L
.22453 .31229 L
.22585 .31228 L
.22707 .31226 L
.22983 .31223 L
.23483 .31214 L
.24417 .31192 L
.25501 .31165 L
.26514 .31141 L
.27497 .31122 L
.28394 .31108 L
.28905 .31103 L
.2939 .31098 L
.29902 .31095 L
.30192 .31094 L
.30459 .31093 L
.30701 .31092 L
.30922 .31091 L
.31165 .31091 L
.31287 .31091 L
.31356 .31091 L
.31421 .31091 L
.31539 .31091 L
.31646 .3109 L
.31762 .3109 L
.31888 .3109 L
.31998 .31091 L
.32115 .31091 L
.32214 .31091 L
Mistroke
.32324 .31091 L
.32591 .31091 L
.32836 .31091 L
.33387 .31092 L
.34383 .31094 L
.38356 .31102 L
.4036 .31105 L
.42177 .31108 L
.44146 .31112 L
.46244 .31119 L
.50252 .31135 L
.54109 .31155 L
.58212 .31177 L
.62162 .31198 L
.6415 .31207 L
.65961 .31213 L
.66964 .31216 L
.67425 .31217 L
.67914 .31218 L
.68161 .31218 L
.68435 .31218 L
.68683 .31219 L
.68794 .31219 L
.68916 .31219 L
.6905 .31219 L
.69122 .31219 L
.6919 .31219 L
.69311 .31219 L
.69441 .31219 L
.69514 .31219 L
.69593 .31219 L
.69668 .31219 L
.69737 .31219 L
.70006 .31219 L
.70246 .31219 L
.70364 .31219 L
.70474 .31219 L
.70599 .31219 L
.70666 .31219 L
.70736 .31219 L
.70862 .31219 L
.70976 .31219 L
.71093 .31219 L
.71201 .31219 L
.71319 .31219 L
.71444 .31219 L
.71884 .31219 L
.72881 .31219 L
.73429 .3122 L
.73699 .3122 L
Mistroke
.73948 .3122 L
.74188 .3122 L
.74442 .3122 L
.74698 .3122 L
.74768 .3122 L
.74843 .3122 L
.74914 .3122 L
.74979 .3122 L
.7509 .3122 L
.75209 .3122 L
.75339 .3122 L
.7546 .3122 L
.75584 .3122 L
.757 .3122 L
.75804 .3122 L
.75917 .3122 L
.76873 .3122 L
.77753 .3122 L
.78222 .3122 L
.78726 .3122 L
.79634 .3122 L
.80189 .3122 L
.80715 .3122 L
.817 .3122 L
.8369 .3122 L
.84122 .3122 L
.84235 .3122 L
.84357 .3122 L
.84472 .3122 L
.84576 .3122 L
.84699 .3122 L
.84814 .3122 L
.84939 .3122 L
.8501 .3122 L
.85075 .3122 L
.85311 .3122 L
.85426 .3122 L
.85532 .3122 L
.85639 .3122 L
.85754 .3122 L
.8588 .3122 L
.85999 .3122 L
.86122 .3122 L
.86236 .3122 L
.86361 .3122 L
.86431 .3122 L
.86494 .3122 L
.8661 .3122 L
.86718 .3122 L
.86835 .3122 L
Mistroke
.86959 .3122 L
.87397 .3122 L
.87872 .3122 L
.88133 .3122 L
.8838 .3122 L
.8852 .3122 L
.88599 .3122 L
.88672 .3122 L
.888 .3122 L
.88866 .3122 L
.88937 .3122 L
.89066 .3122 L
.89139 .3122 L
.89206 .3122 L
.89328 .3122 L
.89457 .3122 L
.89581 .3122 L
.89651 .3122 L
.89715 .3122 L
.89833 .3122 L
.89959 .3122 L
.90071 .3122 L
.90175 .3122 L
.9029 .3122 L
.90414 .3122 L
.90545 .3122 L
.90619 .3122 L
.90688 .3122 L
.90813 .3122 L
.90878 .3122 L
.90947 .3122 L
.91067 .3122 L
.91179 .3122 L
.91244 .3122 L
.91312 .3122 L
.91434 .3122 L
.91549 .3122 L
.91654 .3122 L
.91774 .3122 L
.91889 .3122 L
.91994 .3122 L
.92091 .3122 L
.92198 .3122 L
.92311 .3122 L
.92425 .3122 L
.92548 .3122 L
.92653 .3122 L
.92769 .3122 L
.92839 .3122 L
.92903 .3122 L
Mistroke
.93032 .3122 L
.93148 .3122 L
.93272 .3122 L
.93533 .3122 L
.93808 .3122 L
.94043 .3122 L
.94107 .3122 L
.94176 .3122 L
.94301 .3122 L
.94375 .3122 L
.94444 .3122 L
.9452 .3122 L
.946 .3122 L
.94669 .3122 L
.94735 .3122 L
.94883 .3122 L
.95414 .3122 L
.97431 .3122 L
.97619 .3122 L
Mfstroke
0 1 0 r
.02381 .05715 m
.02499 .01472 L
.02605 .03061 L
.02729 .03634 L
.02846 .04193 L
.03053 .04938 L
.0317 .05245 L
.03279 .05464 L
.03396 .05641 L
.03506 .05755 L
.03626 .0583 L
.03756 .05861 L
.03821 .05859 L
.03891 .05845 L
.0402 .05792 L
.04145 .0571 L
.04262 .05613 L
.04765 .05032 L
.05221 .04402 L
.06244 .03071 L
.06787 .02522 L
.07287 .02133 L
.07835 .01826 L
.08111 .01714 L
.08414 .0162 L
.08681 .0156 L
.08924 .01522 L
.09059 .01506 L
.0913 .015 L
.09205 .01494 L
.09271 .0149 L
.09343 .01486 L
.09471 .01482 L
.09599 .01481 L
.09716 .01482 L
.09845 .01486 L
.09909 .01488 L
.0998 .01492 L
.10228 .01509 L
.10458 .01531 L
.10963 .01595 L
.1151 .01681 L
.12504 .0186 L
.14362 .02176 L
.15309 .02294 L
.15847 .02345 L
.1635 .0238 L
.16622 .02396 L
.1688 .02407 L
.17115 .02416 L
Mistroke
.17365 .02423 L
.17507 .02426 L
.17635 .02429 L
.17764 .0243 L
.17886 .02432 L
.17955 .02432 L
.18028 .02433 L
.18159 .02433 L
.18232 .02433 L
.183 .02433 L
.18375 .02433 L
.18454 .02433 L
.18577 .02432 L
.18646 .02432 L
.18712 .02431 L
.18985 .02428 L
.1922 .02424 L
.19474 .02419 L
.20582 .02392 L
.21545 .02365 L
.22596 .02338 L
.23637 .02316 L
.24604 .02302 L
.25139 .02297 L
.25625 .02293 L
.25894 .02291 L
.26186 .0229 L
.26461 .02289 L
.26592 .02289 L
.26715 .02289 L
.26832 .02288 L
.2696 .02288 L
.27076 .02288 L
.27184 .02288 L
.273 .02288 L
.27428 .02288 L
.27492 .02288 L
.27561 .02288 L
.27687 .02288 L
.27797 .02288 L
.27917 .02288 L
.2816 .02288 L
.28595 .02289 L
.29665 .02292 L
.3066 .02295 L
.31711 .02297 L
.32227 .02298 L
.32704 .02299 L
.33128 .023 L
.33595 .023 L
Mistroke
.34064 .02301 L
.34321 .02301 L
.34562 .02301 L
.34802 .02301 L
.35059 .02301 L
.35206 .02301 L
.3534 .02301 L
.35474 .02301 L
.35601 .02301 L
.35721 .02301 L
.35848 .02301 L
.35956 .02301 L
.36075 .02301 L
.36203 .02301 L
.36274 .02301 L
.3634 .02301 L
.3647 .02301 L
.36588 .02301 L
.37476 .02301 L
.37974 .02301 L
.38441 .02301 L
.38894 .02301 L
.39018 .02301 L
.39148 .02301 L
.3926 .02301 L
.39382 .02301 L
.39452 .02301 L
.39518 .02301 L
.39642 .02301 L
.39711 .02301 L
.39775 .02301 L
.39845 .02301 L
.3992 .02301 L
.40043 .02301 L
.40158 .02301 L
.40294 .02301 L
.4042 .02301 L
.4055 .02301 L
.40692 .02301 L
.40949 .02301 L
.41433 .02301 L
.4196 .02301 L
.42521 .02301 L
.43553 .02302 L
.44668 .02303 L
.46687 .02305 L
.50794 .02311 L
.5475 .02317 L
.58554 .02324 L
.62604 .0233 L
Mistroke
.64473 .02333 L
.65458 .02334 L
.66502 .02335 L
.67029 .02335 L
.67609 .02336 L
.68137 .02336 L
.68374 .02336 L
.68632 .02336 L
.68891 .02336 L
.69011 .02336 L
.69138 .02336 L
.69264 .02336 L
.69379 .02337 L
.69493 .02337 L
.69602 .02337 L
.69729 .02337 L
.69793 .02337 L
.69863 .02337 L
.69987 .02337 L
.70103 .02337 L
.70385 .02336 L
.70509 .02336 L
.70645 .02336 L
.70771 .02336 L
.70888 .02336 L
.71012 .02336 L
.71082 .02336 L
.71146 .02336 L
.71212 .02336 L
.71283 .02336 L
.71411 .02336 L
.71544 .02336 L
.7169 .02336 L
.7268 .02337 L
.7292 .02337 L
.7318 .02337 L
.73425 .02337 L
.73542 .02337 L
.73652 .02337 L
.7375 .02337 L
.73856 .02337 L
.73973 .02337 L
.74081 .02337 L
.74192 .02337 L
.74314 .02337 L
.74429 .02337 L
.74535 .02337 L
.74664 .02337 L
.74729 .02337 L
.748 .02337 L
Mistroke
.75044 .02337 L
.75594 .02337 L
.7659 .02337 L
.77547 .02337 L
.78452 .02336 L
.78942 .02336 L
.79475 .02336 L
.80406 .02336 L
.80678 .02336 L
.80811 .02336 L
.80934 .02336 L
.81062 .02336 L
.81129 .02336 L
.812 .02336 L
.81272 .02336 L
.8134 .02336 L
.81414 .02336 L
.81493 .02336 L
.81624 .02336 L
.81697 .02336 L
.81765 .02336 L
.81889 .02336 L
.8202 .02336 L
.82131 .02336 L
.82249 .02336 L
.82379 .02336 L
.82498 .02336 L
.82752 .02336 L
.82993 .02336 L
.83444 .02336 L
.83676 .02336 L
.83798 .02336 L
.83926 .02336 L
.83995 .02336 L
.84069 .02336 L
.84201 .02336 L
.84265 .02336 L
.84334 .02336 L
.84459 .02336 L
.84582 .02336 L
.84695 .02336 L
.8482 .02336 L
.84952 .02336 L
.85068 .02336 L
.85193 .02336 L
.85311 .02336 L
.85418 .02336 L
.8552 .02336 L
.85631 .02336 L
.85736 .02336 L
Mistroke
.85834 .02336 L
.85948 .02336 L
.86055 .02336 L
.86179 .02336 L
.86291 .02336 L
.86407 .02336 L
.86532 .02336 L
.8665 .02336 L
.86757 .02336 L
.86883 .02336 L
.86954 .02336 L
.87019 .02336 L
.8714 .02336 L
.87268 .02336 L
.8739 .02336 L
.87521 .02336 L
.87645 .02336 L
.87758 .02336 L
.87884 .02336 L
.87956 .02336 L
.88022 .02336 L
.88092 .02336 L
.88156 .02336 L
.8823 .02336 L
.883 .02336 L
.88406 .02336 L
.88524 .02336 L
.88646 .02336 L
.88761 .02336 L
.88868 .02336 L
.88966 .02336 L
.89186 .02336 L
.893 .02336 L
.89422 .02336 L
.89492 .02336 L
.89555 .02336 L
.89679 .02336 L
.89795 .02336 L
.8992 .02336 L
.90147 .02336 L
.90633 .02336 L
.91093 .02336 L
.91347 .02336 L
.91417 .02336 L
.91491 .02336 L
.91554 .02336 L
.91623 .02336 L
.91743 .02336 L
.91873 .02336 L
.91995 .02336 L
Mistroke
.92106 .02336 L
.92238 .02336 L
.92378 .02336 L
.92459 .02336 L
.92532 .02336 L
.92606 .02336 L
.92676 .02336 L
.92742 .02336 L
.92811 .02336 L
.92936 .02336 L
.93217 .02336 L
.94253 .02336 L
.95857 .02336 L
.96297 .02336 L
.96539 .02336 L
.96658 .02336 L
.96768 .02336 L
.96871 .02336 L
.96981 .02336 L
.97091 .02336 L
.97213 .02336 L
.97314 .02336 L
.97422 .02336 L
.97619 .02336 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`005`3oool2o`0002l0oooo1Ol0000_0?ooo`?o0000;`3oool3o`000300oooo0ol0000/0?oo
o`?o0000103oool2o`0000L0oooo000F0?ooo`04o`00003oool0ooooo`0002l0oooo00Co00000?oo
o`3ooooo0000<03oool00ol00000oooo0?ooo`0]0?ooo`05o`00003oool0oooo0?ooool00000;P3o
ool01Ol00000oooo0?ooo`3ooooo000002`0oooo00?o00000?ooo`3oool00P3oool01?l00000oooo
0?ooool000060?ooo`005P3oool01?l00000oooo0?ooool0000`0?ooo`03o`00003oool0oooo02d0
oooo1Ol0000^0?ooo`05o`00003oool0oooo0?ooool00000;P3oool01Ol00000oooo0?ooo`3ooooo
000002`0oooo00?o00000?ooo`3oool00P3oool01?l00000oooo0?ooool000060?ooo`005P3oool0
1?l00000oooo0?ooool0000a0?ooo`03o`00003oool0oooo02`0oooo00Co00000?ooo`3ooooo0000
;`3oool4o`000300oooo0ol0000]0?ooo`03o`00003oool0oooo0080oooo00Co00000?ooo`3ooooo
00001P3oool001H0oooo00Co00000?ooo`3ooooo0000;P3oool01Ol00000oooo0?ooo`3ooooo0000
02h0oooo00?o00000?ooool00000;`3oool00ol00000oooo0?ooo`0`0?ooo`05o`00003oool0oooo
0?ooool00000;03oool00ol00000oooo0?ooo`020?ooo`04o`00003oool0ooooo`0000H0oooo000F
0?ooo`04o`00003oool0ooooo`0002h0oooo00Go00000?ooo`3oool0ooooo`00000_0?ooo`;o0000
<03oool00ol00000oooo0?ooo`0_0?ooo`05o`00003oool0oooo0?ooool00000:`3oool2o`0000@0
oooo00Co00000?ooo`3ooooo00001P3oool001L0oooo0_l0000`0?ooo`?o0000<@3oool00ol00000
oooo0?ooo`0_0?ooo`;o0000<03oool3o`0002d0oooo00?o00000?ooo`3oool00`3oool2o`0000L0
oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo000A0?ooool0
00003@0000030?ooo`004@3oool00`000000oooo0?ooo`040?ooo`03o`00003oool0oooo00T0oooo
00?o00000?ooo`3oool02P3oool00ol00000oooo0?ooo`0:0?ooo`03o`00003oool0oooo00X0oooo
00<000000?ooo`3oool02@3oool00ol00000oooo0?ooo`0:0?ooo`03o`00003oool0oooo00X0oooo
00?o00000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`03o`00003oool0oooo00X0oooo
00?o00000?ooo`3oool02@3oool00ol00000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo
00?o00000?ooo`3oool02@3oool00ol00000oooo0?ooo`0:0?ooo`03o`00003oool0oooo00X0oooo
00<000000?ooo`3oool02@3oool00ol00000oooo0?ooo`0:0?ooo`03o`00003oool0oooo00X0oooo
00?o00000?ooo`3oool02@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0040oooo
000A0?ooo`0300000?l00000oooo00@0oooo00?o00000?ooo`3oool0<03oool00`000000oooo0?oo
o`0_0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0<03oool00`000000oooo0?oo
o`0_0?ooo`030000003oool0oooo00<0oooo00?o00000000003oool00P3oool00140oooo00<00000
0?ooo`3oool0o`3oool90?ooo`030000003oool0oooo0040oooo000A0?ooo`030000003oool0oooo
00@0oooo00?o0?l00?ooo`3oool0303oool<o`3o0>T0oooo00<000000?ooo`3oool00@3oool00140
oooo00<000000?ooo`3oool0103oool00ol0o`00oooo0?ooo`0;0?ooo`03o`3o003oool0oooo00X0
oooo1?l0o`3U0?ooo`030000003oool0oooo0040oooo000A0?ooo`030000003oool0oooo00@0oooo
00?o0?l00?ooo`3oool02P3oool00ol0o`00oooo0?ooo`0?0?ooom?o0?l01`00o`05o`3o00H0oooo
00<000000?ooo`3oool00@3oool00140oooo00<00000o`00003oool0103oool00ol0o`00oooo0?oo
o`080?ooo`;o0?l0mP3oool00ol0000000000?ooo`020?ooo`004@3oool00`000000oooo0?ooo`04
0?ooo`03o`3o003oool0oooo00L0oooo00?o0?l00?ooo`3oool0m`3oool00`000000oooo0?ooo`01
0?ooo`004@3oool00`000000oooo0?ooo`040?ooo`03o`3o003oool0oooo00L0oooo00?o0?l00?oo
o`3oool0m`3oool00`000000oooo0?ooo`010?ooo`004@3oool00`000000oooo0?ooo`040?ooo`;o
0?l01`3oool00ol0o`00oooo0?ooo`3h0?ooo`030000003oool0oooo0040oooo000A0?ooo`030000
0?l00000oooo00@0oooo0_l0o`060?ooo`03o`3o003oool0oooo0?P0oooo00?o00000000003oool0
0P3oool00140oooo00<000000?ooo`3oool0103oool2o`3o00D0oooo00?o0?l00?ooo`3oool0UP3o
oolCool00540oooo00<000000?ooo`3oool00@3oool000X0oooo0_l000050?ooo`030000003oool0
oooo00@0oooo00?o0?l00?ooool0o`000`3oool00ol0o`00oooo0?ooo`1j0?oooagoo`004P3oool3
ool004l0oooo00<000000?ooo`3oool00@3oool000T0oooo00Co00000?ooo`3ooooo0000103oool0
0`000000oooo0?ooo`040?ooo`03o`3o003ooooo0?l000<0oooo00?o0?l00?ooo`3oool0FP3ooolP
ool00380oooo=ooo00030?l000Ooo`001P3o0002ool000H0oooo00<000000?ooo`3oool00@3oool0
00T0oooo00Co00000?ooo`3ooooo0000103oool3000000@0oooo00?o0?l00?ooool0o`000ol0o`19
0?oooaCoo`00W`3oool3000000<0oooo00090?ooo`04o`00003oool0ooooo`0000@0oooo00<00000
0?ooo`3oool0103oool01?oo0?ooo`000?ooool0o`0d0?oooaOoo`00]@3oool00`000000oooo0?oo
o`010?ooo`002@3oool01?l00000oooo0?ooool000040?ooo`030000003oool0oooo00@0oooo00?o
o`3o0?oooooo00000_oo000G0?oooacoo`00c03oool00`000000oooo0?ooo`010?ooo`002@3oool0
1?l00000oooo0?ooool000040?ooo`030000003oool0oooo00@0oooo00?oo`3o0?ooo`3oool00P3o
oolGool00>P0oooo00<000000?ooo`3oool00@3oool000X0oooo0_l000050?ooo`0300000?l00000
oooo00@0oooo00?oo`3o0?ooo`3oool0o`3oool10?ooo`03o`0000000000oooo0080oooo000A0?oo
o`030000003oool0oooo00@0oooo00?oo`3o0?ooo`3oool0o`3oool20?ooo`030000003oool0oooo
0040oooo000A0?ooo`030000003oool0oooo00@0oooo00?oo`3o0?ooo`3oool0o`3oool20?ooo`03
0000003oool0oooo0040oooo000A0?ooo`030000003oool0oooo00@0oooo00?oo`3oo`00o`3oool0
o`3oool20?ooo`030000003oool0oooo0040oooo000A0?ooo`030000003oool0oooo00@0oooo00?o
o`3oo`00o`3oool0o`3oool20?ooo`030000003oool0oooo0040oooo000A0?ooo`0300000?l00000
oooo00@0oooo00?oo`3oo`00o`3oool0o`3oool10?ooo`03o`0000000000oooo0080oooo000A0?oo
o`030000003oool0oooo00@0oooo00?oo`3oo`00o`3oool0o`3oool20?ooo`030000003oool0oooo
0040oooo000A0?ooo`030000003oool0oooo00@0oooo00?oo`3oo`00o`3oool0o`3oool20?ooo`03
0000003oool0oooo0040oooo000A0?ooo`030000003oool0oooo00@0oooo00?oo`3oo`00o`3oool0
o`3oool20?ooo`030000003oool0oooo0040oooo000A0?ooo`0300000?l00000oooo00@0oooo00?o
o`3oo`00o`3oool0o`3oool10?ooo`03o`0000000000oooo0080oooo000A0?ooo`030000003oool0
oooo00@0oooo00?oo`3oo`00o`3oool0o`3oool20?ooo`030000003oool0oooo0040oooo000A0?oo
o`030000003oool0oooo00@0oooo00?oo`3oo`00o`3oool0o`3oool20?ooo`030000003oool0oooo
0040oooo000A0?ooo`030000003oool0oooo00@0oooo00?oo`3oo`00o`3oool0o`3oool20?ooo`03
0000003oool0oooo0040oooo000A0?ooo`0300000?l00000oooo00@0oooo00?oo`3oo`00o`3oool0
o`3oool10?ooo`03o`0000000000oooo0080oooo000A0?ooo`030000003oool0oooo00@0oooo00?o
o`3oo`00o`3oool0o`3oool20?ooo`030000003oool0oooo0040oooo00090?ooo`?o00001@3oool0
0`000000oooo0?ooo`040?ooo`03ool0o`3ooooo003o0?l0oooo0P3oool00`000000oooo0?ooo`01
0?ooo`00203oool01Ol00000oooo0?ooo`3ooooo000000@0oooo00<000000?ooo`3oool0103oool0
0ooo0?l0ooooo`00o`3o0?ooo`80oooo00<000000?ooo`3oool00@3oool000`0oooo00?o00000?oo
o`3oool00P3oool00`000000oooo0?ooo`040?ooo`03ool0o`3ooooo003o0?l0oooo0P3oool00`00
0000oooo0?ooo`010?ooo`00303oool00ol00000oooo0?ooo`020?ooo`<00000103oool00ooo0?l0
ooooo`00o`3o0?ooo`<000000`3oool000P0oooo1?l000050?ooo`030000003oool0oooo00@0oooo
00?oo`3o0?ooool00?l0o`3oool20?ooo`030000003oool0oooo0040oooo00080?ooo`03o`00003o
ool0oooo00H0oooo00<000000?ooo`3oool0103oool00ooo0?l0ooooo`00o`3o0?ooo`80oooo00<0
00000?ooo`3oool00@3oool000P0oooo1Ol000040?ooo`030000003oool0oooo00@0oooo00?oo`3o
0?ooool00?l0o`3oool20?ooo`030000003oool0oooo0040oooo000A0?ooo`0300000?l00000oooo
00@0oooo00?oo`3o0?ooool00?l0o`3oool10?ooo`03o`0000000000oooo0080oooo000A0?ooo`03
0000003oool0oooo00@0oooo00?oo`3o0?ooool00?l0o`3oool20?ooo`030000003oool0oooo0040
oooo000A0?ooo`030000003oool0oooo00@0oooo00?oo`3o0?ooool00?l0o`3oool20?ooo`030000
003oool0oooo0040oooo000A0?ooo`030000003oool0oooo00@0oooo00?oo`3o0?ooool00?l0o`3o
ool20?ooo`030000003oool0oooo0040oooo000A0?ooo`0300000?l00000oooo00D0oooo00?oo`3o
0?ooool00?l0o`3oool00ol0000000000?ooo`020?ooo`004@3oool00`000000oooo0?ooo`050?oo
o`03ool0o`3ooooo003o0?l0oooo0@3oool00`000000oooo0?ooo`010?ooo`004@3oool00`000000
oooo0?ooo`050?ooo`03ool0o`3ooooo003o0?l0oooo0@3oool00`000000oooo0?ooo`010?ooo`00
4@3oool00`000000oooo0?ooo`050?ooo`03ool0o`3ooooo003o0?l0oooo0@3oool00`000000oooo
0?ooo`010?ooo`004@3oool00`00003o00000?ooo`050?ooo`03ool0o`3ooooo003o0?l0oooo00?o
00000000003oool00P3oool00140oooo00<000000?ooo`3oool01@3oool00ooo0?l0ooooo`00o`3o
0?ooo`40oooo00<000000?ooo`3oool00@3oool00140oooo00<000000?ooo`3oool01@3oool00ooo
0?l0ooooo`00o`3o0?ooo`40oooo00<000000?ooo`3oool00@3oool00140oooo00<000000?ooo`3o
ool01@3oool00ooo0?l0ooooo`00o`3o0?ooo`40oooo00<000000?ooo`3oool00@3oool00140oooo
00<000000?ooo`3oool01@3oool00ooo0?l0ooooo`00o`3o0?ooo`40oooo00<000000?ooo`3oool0
0@3oool00140oooo00<00000o`00003oool01@3oool00ooo0?l0ooooo`00o`3o0?ooo`03o`000000
0000oooo0080oooo00030?ooo`?o0000103oool2o`0000D0oooo00<000000?ooo`3oool01@3oool0
0ooo0?l0ooooo`00o`3o0?ooo`40oooo00<000000?ooo`3oool00@3oool000@0oooo00?o00000?oo
o`3oool00P3oool01?l00000oooo0?ooool000040?ooo`030000003oool0oooo00D0oooo00?oo`3o
0?ooool00?l0o`3oool10?ooo`030000003oool0oooo0040oooo00040?ooo`03o`00003oool0oooo
0080oooo00Co00000?ooo`3ooooo0000103oool00`000000oooo0?ooo`050?ooo`04ool0o`3oool0
ooooo`00ool0oooo00<000000?ooo`3oool00@3oool000@0oooo00?o00000?ooo`3oool00P3oool0
1?l00000oooo0?ooool000040?ooo`<000001P3oool00ooo0?l0ooooo`00o`3m0?ooo`<000000`3o
ool000@0oooo00?o00000?ooo`3oool00P3oool01?l00000oooo0?ooool000040?ooo`030000003o
ool0oooo00H0oooo00?oo`3o0?ooool00?l0o`3oool00`000000oooo0?ooo`010?ooo`000`3oool2
o`0000@0oooo00Co00000?ooo`3ooooo0000103oool00`000000oooo0?ooo`060?ooo`03ool0o`3o
oooo003o0?l0oooo00<000000?ooo`3oool00@3oool000@0oooo00?o00000?ooo`3oool00`3oool2
o`0000D0oooo00<000000?ooo`3oool01P3oool00ooo0?l0ooooo`00o`3o0?ooo`030000003oool0
oooo0040oooo000A0?ooo`0300000?l00000oooo00H0oooo00?oo`3o0?ooool00?l0oP3oool00ol0
000000000?ooo`020?ooo`004@3oool00`000000oooo0?ooo`060?ooo`03ool0o`3ooooo003o0?l0
oooo00<000000?ooo`3oool00@3oool00140oooo00<000000?ooo`3oool01P3oool00ooo0?l0oooo
o`00o`3o0?ooo`030000003oool0oooo0040oooo000A0?ooo`030000003oool0oooo00H0oooo00?o
o`3o0?ooool00?l0o`3oool00`000000oooo0?ooo`010?ooo`004@3oool00`000000oooo0?ooo`06
0?ooo`03ool0o`3ooooo003o0?l0oooo00<000000?ooo`3oool00@3oool00140oooo00<00000o`00
003oool01P3oool00ooo0?l0ooooo`00o`3n0?ooo`03o`0000000000oooo0080oooo000A0?ooo`03
0000003oool0oooo00H0oooo00?oo`3o0?ooool00?l0o`3oool00`000000oooo0?ooo`010?ooo`00
4@3oool00`000000oooo0?ooo`060?ooo`04ool0o`3oool0ooooo`00ooh0oooo00<000000?ooo`3o
ool00@3oool00140oooo00<000000?ooo`3oool01P3oool01?oo0?l0oooo0?ooool00?on0?ooo`03
0000003oool0oooo0040oooo000A0?ooo`0300000?l00000oooo00H0oooo00Coo`3o0?ooo`3ooooo
003oo@3oool00ol0000000000?ooo`020?ooo`004@3oool00`000000oooo0?ooo`060?ooo`04ool0
o`3oool0ooooo`00ooh0oooo00<000000?ooo`3oool00@3oool00140oooo00<000000?ooo`3oool0
1P3oool01?oo0?l0oooo0?ooool00?on0?ooo`030000003oool0oooo0040oooo000A0?ooo`030000
003oool0oooo00H0oooo00Coo`3o0?ooo`3ooooo003ooP3oool00`000000oooo0?ooo`010?ooo`00
4@3oool00`00003o00000?ooo`060?ooo`04ool0o`3oool0ooooo`00ood0oooo00?o00000000003o
ool00P3oool00140oooo00<000000?ooo`3oool01P3oool01?oo0?l0oooo0?ooool00?on0?ooo`03
0000003oool0oooo0040oooo00030?ooo`?o00000`3oool3o`0000D0oooo00<000000?ooo`3oool0
1P3oool01?oo0?l0oooo0?ooool00?on0?ooo`030000003oool0oooo0040oooo00040?ooo`05o`00
003oool0oooo0?ooool000000`3oool00ol00000oooo0?ooo`020?ooo`030000003oool0oooo00L0
oooo00?oo`3o0?ooool00?l0oP3oool00`000000oooo0?ooo`010?ooo`00103oool00ol00000oooo
0?ooo`050?ooo`03o`00003oool0oooo0080oooo0`0000070?ooo`03ool0o`3ooooo003o0?`0oooo
0`0000030?ooo`00103oool00ol00000oooo0?ooo`050?ooo`03o`00003oool0oooo0080oooo00<0
00000?ooo`3oool01`3oool00ooo0?l0ooooo`00o`3n0?ooo`030000003oool0oooo0040oooo0004
0?ooo`04o`00003oool0oooo0?ooo`Co00001@3oool00`000000oooo0?ooo`070?ooo`03ool0o`3o
oooo003o01@0oooo2ooo0?oO0?ooo`030000003oool0oooo0040oooo00030?ooo`;o00000`3oool0
0ol00000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00?oo`3o0?ooool00?l04P3oool2
ool0o`/0oooo1Ooo0?oJ0?ooo`030000003oool0oooo0040oooo00040?ooo`04o`00003oool0oooo
0?ooo`Go0000103oool00`000000oooo0?ooo`070?ooo`03ool0o`3ooooo003o0100oooo0_oo0?lB
0?ooolcoo`3o1`3o0?l00ooo0?l0oooo0?ooo`040?ooo`030000003oool0oooo0040oooo000A0?oo
o`0300000?l00000oooo00L0oooo00Coo`3o0?ooo`3ooooo003o3P3oool00ooo0?l0oooo0?ooo`3[
0?ooo`03o`0000000000oooo0080oooo000A0?ooo`030000003oool0oooo00L0oooo00Coo`3o0?oo
o`3ooooo003o3@3oool00ooo0?l0oooo0?ooo`3]0?ooo`030000003oool0oooo0040oooo000A0?oo
o`030000003oool0oooo00P0oooo00?oo`3o0?ooool00?l0303oool00ooo0?l0oooo0?ooo`3^0?oo
o`030000003oool0oooo0040oooo000A0?ooo`030000003oool0oooo00P0oooo00?oo`3o0?ooool0
0?l02`3oool00ooo0?l0oooo0?ooo`3_0?ooo`030000003oool0oooo0040oooo000A0?ooo`030000
0?l00000oooo00P0oooo00?oo`3o0?ooool00?l02P3oool00ooo0?l0oooo0?ooo`3_0?ooo`03o`00
00000000oooo0080oooo000A0?ooo`030000003oool0oooo00P0oooo00?oo`3o0?ooool00?l02@3o
ool00ooo0?l0oooo0?ooo`3a0?ooo`030000003oool0oooo0040oooo000A0?ooo`030000003oool0
oooo00P0oooo00?oo`3o0?ooool00?l0203oool00ooo0?l0oooo0?ooo`3b0?ooo`030000003oool0
oooo0040oooo000A0?ooo`030000003oool0oooo00P0oooo00?oo`3o0?ooool00?l0203oool00ooo
0?l0oooo0?ooo`3b0?ooo`030000003oool0oooo0040oooo000A0?ooo`0300000?l00000oooo00P0
oooo00?oo`3o0?ooool00?l01`3oool00ooo0?l0oooo0?ooo`3b0?ooo`03o`0000000000oooo0080
oooo000A0?ooo`030000003oool0oooo00P0oooo00?oo`3o0?ooool00?l01P3oool00ooo0?l0oooo
0?ooo`3d0?ooo`030000003oool0oooo0040oooo000A0?ooo`030000003oool0oooo00P0oooo00?o
o`3o0?ooool00?l01P3oool00ooo0?l0oooo0?ooo`3d0?ooo`030000003oool0oooo0040oooo000A
0?ooo`030000003oool0oooo00T0oooo00?oo`3oo`00o`3oool0103oool00ooo0?l0oooo0?ooo`3e
0?ooo`030000003oool0oooo0040oooo000A0?ooo`0300000?l00000oooo00T0oooo00?oo`3oo`00
o`3oool0103oool00ooo0?l0oooo0?ooo`3d0?ooo`03o`0000000000oooo0080oooo000A0?ooo`03
0000003oool0oooo00T0oooo00?oo`3oo`00o`3oool00`3oool00ooo0?l0oooo0?ooo`3f0?ooo`03
0000003oool0oooo0040oooo00020?ooo`Go00000`3oool2o`0000D0oooo00<000000?ooo`3oool0
2@3oool00ooo0?oo003o0?ooo`020?ooo`03ool0o`3oool0oooo0?L0oooo00<000000?ooo`3oool0
0@3oool000<0oooo00Co00000?ooo`3ooooo00000P3oool01?l00000oooo0?ooool000040?ooo`03
0000003oool0oooo00X0oooo00Coo`3o0?ooo`3oooooo`3onP3oool00`000000oooo0?ooo`010?oo
o`00103oool00ol00000oooo0?ooo`020?ooo`04o`00003oool0ooooo`0000@0oooo00<000000?oo
o`3oool02P3oool3ool0oo/0oooo00<000000?ooo`3oool00@3oool000D0oooo00Go00000?ooo`3o
ool0ooooo`0000020?ooo`03o`00003oool0oooo0080oooo0`00000;0?ooo`03o`00o`3oool0oooo
0?P0oooo0`0000030?ooo`000P3oool01Ol00000oooo0?ooo`3ooooo00000080oooo00Co00000?oo
o`3ooooo0000103oool00`000000oooo0?ooo`0;0?ooo`03o`00o`3oool0oooo0?X0oooo00<00000
0?ooo`3oool00@3oool00080oooo00Go00000?ooo`3oool0ooooo`0000020?ooo`04o`00003oool0
ooooo`0000@0oooo00<000000?ooo`3oool02`3oool00ol00?l0oooo0?ooo`3j0?ooo`030000003o
ool0oooo0040oooo00030?ooo`?o0000103oool2o`0000D0oooo00<000000?ooo`3oool02`3oool0
0ol00?l0oooo0?ooo`3j0?ooo`030000003oool0oooo0040oooo000A0?ooo`0300000?l00000oooo
00/0oooo00?o003o0?ooo`3oool0n@3oool00ol0000000000?ooo`020?ooo`004@3oool00`000000
oooo0?ooo`0;0?ooo`03o`00o`3oool0oooo0?X0oooo00<000000?ooo`3oool00@3oool00140oooo
00<000000?ooo`3oool02`3oool00ol00?l0oooo0?ooo`3j0?ooo`030000003oool0oooo0040oooo
000A0?ooo`030000003oool0oooo00/0oooo00?o003o0?ooo`3oool0nP3oool00`000000oooo0?oo
o`010?ooo`004@3oool00`00003o00000?ooo`0;0?ooo`03o`00o`3oool0oooo0?T0oooo00?o0000
0000003oool00P3oool00140oooo00<000000?ooo`3oool02`3oool00ol00?l0oooo0?ooo`3j0?oo
o`030000003oool0oooo0040oooo000A0?ooo`030000003oool0oooo00/0oooo00?o003o0?ooo`3o
ool0nP3oool00`000000oooo0?ooo`010?ooo`004@3oool00`000000oooo0?ooo`0;0?ooo`03o`00
o`3oool0oooo0?X0oooo00<000000?ooo`3oool00@3oool00140oooo00<000000?ooo`3oool02`3o
ool00ol00?l0oooo0?ooo`3j0?ooo`030000003oool0oooo0040oooo000A0?ooo`0300000?l00000
oooo00`0oooo00?o003o0?ooo`3oool0n03oool00ol0000000000?ooo`020?ooo`004@3oool00`00
0000oooo0?ooo`0<0?ooo`03o`00o`3oool0oooo0?T0oooo00<000000?ooo`3oool00@3oool00140
oooo00<000000?ooo`3oool0303oool00ol00?l0oooo0?ooo`3i0?ooo`030000003oool0oooo0040
oooo000A0?ooo`030000003oool0oooo00`0oooo00?o003o0?ooo`3oool0n@3oool00`000000oooo
0?ooo`010?ooo`004@3oool00`00003o00000?ooo`0<0?ooo`03o`00o`3oool0oooo0?P0oooo00?o
00000000003oool00P3oool00140oooo00<000000?ooo`3oool0303oool00ol00?l0oooo0?ooo`3i
0?ooo`030000003oool0oooo0040oooo00020?ooo`Go00000P3oool3o`0000D0oooo00<000000?oo
o`3oool0303oool00ol00?l0oooo0?ooo`3i0?ooo`030000003oool0oooo0040oooo00030?ooo`06
o`00003oool0ooooo`00003ooooo00000`3oool00ol00000oooo0?ooo`020?ooo`030000003oool0
oooo00`0oooo00?o003o0?ooo`3oool0n@3oool00`000000oooo0?ooo`010?ooo`00103oool00ol0
0000oooo0?ooo`050?ooo`03o`00003oool0oooo0080oooo0`00000=0?ooo`03o`00o`3oool0oooo
0?H0oooo0`0000030?ooo`001@3oool00ol00000oooo0?ooo`040?ooo`03o`00003oool0oooo0080
oooo00<000000?ooo`3oool03@3oool00ol00?l0oooo0?ooo`3h0?ooo`030000003oool0oooo0040
oooo00020?ooo`06o`00003oool0oooo0?ooool00000oooo1?l000050?ooo`030000003oool0oooo
00d0oooo00?o003o0?ooo`3oool0n03oool00`000000oooo0?ooo`010?ooo`000P3oool01ol00000
oooo0?ooo`3ooooo00000?ooool00000203oool00`000000oooo0?ooo`0=0?ooo`03o`00o`3oool0
oooo0?P0oooo00<000000?ooo`3oool00@3oool000<0oooo0ol000020?ooo`Go0000103oool00`00
003o00000?ooo`0=0?ooo`03o`00o`3oool0oooo0?L0oooo00?o00000000003oool00P3oool00140
oooo00<000000?ooo`3oool03@3oool00ol00?l0oooo0?ooo`3h0?ooo`030000003oool0oooo0040
oooo000A0?ooo`030000003oool0oooo00d0oooo00?o003o0?ooo`3oool06P3oool4o`00omX0oooo
00<000000?ooo`3oool00@3oool00140oooo00<000000?ooo`3oool03@3oool00ol00?l0oooo0?oo
o`0H0?ooo`?o003o0`3oool6o`00om@0oooo00<000000?ooo`3oool00@3oool00140oooo00<00000
0?ooo`3oool03@3oool00ol00?l0oooo0?ooo`0G0?ooo`03o`00o`3oool0oooo00X0oooo0ol00?oA
0?ooo`030000003oool0oooo0040oooo000A0?ooo`0300000?l00000oooo00d0oooo00?o003o0?oo
o`3oool05@3oool2o`00oa00oooo0_l00?o>0?ooo`03o`0000000000oooo0080oooo000A0?ooo`03
0000003oool0oooo00h0oooo00?o003o0?ooo`3oool0503oool00ol00?l0oooo0?ooo`0A0?ooo`Oo
003o2`3ooon^o`00o`T0003o1P3oool00`000000oooo0?ooo`010?ooo`004@3oool00`000000oooo
0?ooo`0>0?ooo`03o`00o`3oool0oooo01<0oooo00?o003o0?ooo`3oool0603oool<o`00okd0oooo
00<000000?ooo`3oool00@3oool00140oooo00<000000?ooo`3oool03P3oool00ol00?l0oooo0?oo
o`0C0?ooo`03o`00o`3oool0oooo0>40oooo00<000000?ooo`3oool00@3oool00140oooo00<00000
o`00003oool03P3oool00ol00?l0oooo0?ooo`0B0?ooo`03o`00o`3oool0oooo0>40oooo00?o0000
0000003oool00P3oool00140oooo00<000000?ooo`3oool03P3oool00ol00?l0oooo0?ooo`0A0?oo
o`03o`00o`3oool0oooo0><0oooo00<000000?ooo`3oool00@3oool00140oooo00<000000?ooo`3o
ool03P3oool00ol00?l0oooo0?ooo`0A0?ooo`03o`00o`3oool0oooo0><0oooo00<000000?ooo`3o
ool00@3oool00140oooo00<000000?ooo`3oool03P3oool00ol00?l0oooo0?ooo`0@0?ooo`03o`00
o`3oool0oooo0>@0oooo00<000000?ooo`3oool00@3oool00140oooo00<00000o`00003oool03P3o
ool00ol00?l0oooo0?ooo`0@0?ooo`03o`00o`3oool0oooo0><0oooo00?o00000000003oool00P3o
ool00140oooo00<000000?ooo`3oool03`3oool00ol00?l0oooo0?ooo`0>0?ooo`03o`00o`3oool0
oooo0>D0oooo00<000000?ooo`3oool00@3oool000<0oooo0ol000040?ooo`;o00001@3oool00`00
0000oooo0?ooo`0?0?ooo`03o`00o`3oool0oooo00d0oooo00?o003o0?ooo`3oool0iP3oool00`00
0000oooo0?ooo`010?ooo`000P3oool01Ol00000oooo0?ooo`3ooooo00000080oooo00Co00000?oo
o`3ooooo0000103oool00`000000oooo0?ooo`0?0?ooo`03o`00o`3oool0oooo00d0oooo00?o003o
0?ooo`3oool0iP3oool00`000000oooo0?ooo`010?ooo`001P3oool01?l00000oooo0?ooool00002
0?ooo`03o`00003oool0oooo0080oooo00<000000?ooo`3oool03`3oool00ol00?l0oooo0?ooo`0<
0?ooo`03o`00o`3oool0oooo0>L0oooo00<000000?ooo`3oool00@3oool000@0oooo0_l000030?oo
o`04o`00003oool0ooooo`0000@0oooo0`00000?0?ooo`03o`00o`3oool0oooo00`0oooo00?o003o
0?ooo`3oool0i@3oool3000000<0oooo00060?ooo`04o`00003oool0ooooo`000080oooo00?o0000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`03o`00o`3oool0oooo00X0oooo00?o003o
0?ooo`3oool0j03oool00`000000oooo0?ooo`010?ooo`000P3oool01Ol00000oooo0?ooo`3ooooo
00000080oooo00Co00000?ooo`3ooooo0000103oool00`000000oooo0?ooo`0@0?ooo`03o`00o`3o
ool0oooo00X0oooo00?o003o0?ooo`3oool0j03oool00`000000oooo0?ooo`010?ooo`000`3oool3
o`0000@0oooo0_l000050?ooo`030000003oool0oooo0100oooo00?o003o0?ooo`3oool02P3oool0
0ol00?l0oooo0?ooo`3X0?ooo`030000003oool0oooo0040oooo000A0?ooo`0300000?l00000oooo
0100oooo00?o003o0?ooo`3oool02@3oool00ol00?l0oooo0?ooo`3X0?ooo`03o`0000000000oooo
0080oooo000A0?ooo`030000003oool0oooo0100oooo00?o003o0?ooo`3oool02@3oool00ol00?l0
oooo0?ooo`3Y0?ooo`030000003oool0oooo0040oooo000A0?ooo`030000003oool0oooo0100oooo
00?o003o0?ooo`3oool0203oool00ol00?l0oooo0?ooo`3Z0?ooo`030000003oool0oooo0040oooo
000A0?ooo`030000003oool0oooo0140oooo00?o003o0?ooo`3oool01`3oool00ol00?l0oooo0?oo
o`3Z0?ooo`030000003oool0oooo0040oooo000A0?ooo`0300000?l00000oooo0140oooo00?o003o
0?ooo`3oool01P3oool00ol00?l0oooo0?ooo`3Z0?ooo`03o`0000000000oooo0080oooo000A0?oo
o`030000003oool0oooo0140oooo00?o003o0?ooo`3oool01P3oool00ol00?l0oooo0?ooo`3[0?oo
o`030000003oool0oooo0040oooo000A0?ooo`030000003oool0oooo0140oooo00?o003o0?ooo`3o
ool01P3oool00ol00?l0oooo0?ooo`3[0?ooo`030000003oool0oooo0040oooo000A0?ooo`030000
003oool0oooo0140oooo00?o003o0?ooo`3oool01@3oool00ol00?l0oooo0?ooo`3/0?ooo`030000
003oool0oooo0040oooo000A0?ooo`0300000?l00000oooo0140oooo00?o003o0?ooo`3oool01@3o
ool00ol00?l0oooo0?ooo`3[0?ooo`03o`0000000000oooo0080oooo000A0?ooo`030000003oool0
oooo0140oooo00?o003o0?ooo`3oool0103oool00ol00?l0oooo0?ooo`3]0?ooo`030000003oool0
oooo0040oooo000A0?ooo`030000003oool0oooo0180oooo00?o003o0?ooo`3oool00`3oool00ol0
0?l0oooo0?ooo`3]0?ooo`030000003oool0oooo0040oooo000A0?ooo`030000003oool0oooo01<0
oooo00Go003o0?ooo`3oool0ooooo`00o`3`0?ooo`030000003oool0oooo0040oooo000A0?ooo`03
0000003oool0oooo01<0oooo1?l00?oa0?ooo`030000003oool0oooo0040oooo000A0?ooo`030000
0?l00000oooo01D0oooo00?o003o0?ooo`3oool0k`3oool00ol0000000000?ooo`020?ooo`004@3o
ool00`000000oooo0?ooo`3o0?ooo`T0oooo00<000000?ooo`3oool00@3oool00140oooo00<00000
0?ooo`3oool0103oool00ol00000oooo0?ooo`090?ooo`03o`00003oool0oooo00X0oooo00?o0000
0?ooo`3oool02P3oool00ol00000oooo0?ooo`0:0?ooo`03o`00003oool0oooo00T0oooo00?o0000
0?ooo`3oool02P3oool00ol00000oooo0?ooo`0:0?ooo`03o`00003oool0oooo00T0oooo00?o0000
0?ooo`3oool02P3oool00ol00000oooo0?ooo`0:0?ooo`03o`00003oool0oooo00T0oooo00?o0000
0?ooo`3oool02P3oool00ol00000oooo0?ooo`0:0?ooo`03o`00003oool0oooo00T0oooo00?o0000
0?ooo`3oool02P3oool00ol00000oooo0?ooo`0:0?ooo`03o`00003oool0oooo00T0oooo00?o0000
0?ooo`3oool02P3oool00ol00000oooo0?ooo`0:0?ooo`03o`00003oool0oooo00T0oooo00?o0000
0?ooo`3oool0103oool00`000000oooo0?ooo`010?ooo`004@3ooooo000000d000000`3oool00001
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.956927, -6.4329, \
0.0394232, 0.233487}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Animation", "Section"],

Cell[CellGroupData[{

Cell["Robot types", "Subsection"],

Cell[BoxData[
    \("Available robots:\n\n\t1 = two-link planar robot with point mass \
assumption.\n\t2 = two-link planar robot with cylindrical link assumption.\n\t\
3 = four-link robot with point mass assumption."\)], "Print",
  CellFrame->True,
  Background->GrayLevel[0.849989],
  CellTags->"Info3215543836-4508301"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Animation parameters", "Subsection"],

Cell[BoxData[{
    \(\(dim\  = \ 3;\)\), "\n", 
    \(\(nFrames\  = \ 150;\)\), "\n", 
    \(\(robotType\  = \ 3;\)\)}], "Input",
  CellFrame->True,
  Background->GrayLevel[0.849989]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Animation", "Subsection"],

Cell[CellGroupData[{

Cell["\<\
Animate just actual trajectory (result not shown to minimize file \
size)\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(AnimateTrajectory[DH[\([robotType]\)], actual, dim, nFrames, 
        tmax];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Animate both desired and actual trajectory (result not shown to \
minimize file size)\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(AnimateTwoTrajectories[DH[\([robotType]\)], desiredOut, \ actual, 
        dim, nFrames, tmax];\)\)], "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Macintosh",
ScreenRectangle->{{0, 1280}, {0, 828}},
WindowSize->{802, 639},
WindowMargins->{{11, Automatic}, {Automatic, 17}},
PrintingCopies->1,
PrintingPageRange->{1, 9999},
StyleDefinitions -> "Classroom.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "Experiment-specific user definitions"->{
    Cell[17718, 549, 119, 1, 39, "Section",
      CellTags->{"Experiment-specific user definitions", "Experiment"}]},
  "Experiment"->{
    Cell[17718, 549, 119, 1, 39, "Section",
      CellTags->{"Experiment-specific user definitions", "Experiment"}]},
  "Info3215543836-4508301"->{
    Cell[137804, 5540, 321, 6, 103, "Print",
      CellTags->"Info3215543836-4508301"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"Experiment-specific user definitions", 139734, 5609},
  {"Experiment", 139870, 5612},
  {"Info3215543836-4508301", 140018, 5615}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 125, 4, 62, "Title"],

Cell[CellGroupData[{
Cell[1904, 59, 36, 0, 59, "Section"],

Cell[CellGroupData[{
Cell[1965, 63, 41, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[2031, 67, 60, 0, 42, "Subsubsection"],
Cell[2094, 69, 101, 2, 43, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[2232, 76, 51, 0, 28, "Subsubsection"],
Cell[2286, 78, 179, 3, 59, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2514, 87, 47, 0, 33, "Subsection"],

Cell[CellGroupData[{
Cell[2586, 91, 43, 0, 42, "Subsubsection"],
Cell[2632, 93, 274, 5, 59, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[2943, 103, 35, 0, 42, "Subsubsection"],
Cell[2981, 105, 318, 6, 75, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[3336, 116, 56, 0, 42, "Subsubsection"],
Cell[3395, 118, 442, 7, 107, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[3874, 130, 104, 3, 42, "Subsubsection"],
Cell[3981, 135, 310, 5, 59, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[4328, 145, 38, 0, 42, "Subsubsection"],
Cell[4369, 147, 66, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[4472, 153, 65, 0, 42, "Subsubsection"],
Cell[4540, 155, 234, 4, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[4811, 164, 71, 0, 42, "Subsubsection"],
Cell[4885, 166, 244, 4, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[5166, 175, 65, 0, 42, "Subsubsection"],
Cell[5234, 177, 322, 5, 59, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[5605, 188, 67, 0, 33, "Subsection"],

Cell[CellGroupData[{
Cell[5697, 192, 67, 0, 42, "Subsubsection"],
Cell[5767, 194, 370, 9, 59, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[6174, 208, 120, 3, 28, "Subsubsection"],
Cell[6297, 213, 148, 2, 43, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[6482, 220, 103, 3, 28, "Subsubsection"],
Cell[6588, 225, 74, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[6699, 231, 68, 0, 28, "Subsubsection"],
Cell[6770, 233, 147, 3, 59, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[6954, 241, 134, 5, 28, "Subsubsection"],
Cell[7091, 248, 594, 10, 123, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[7722, 263, 108, 3, 28, "Subsubsection"],
Cell[7833, 268, 82, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[7952, 274, 71, 0, 28, "Subsubsection"],
Cell[8026, 276, 159, 3, 59, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[8222, 284, 85, 1, 28, "Subsubsection"],
Cell[8310, 287, 81, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[8428, 293, 85, 1, 28, "Subsubsection"],
Cell[8516, 296, 105, 2, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[8658, 303, 71, 0, 28, "Subsubsection"],
Cell[8732, 305, 330, 7, 107, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[9111, 318, 58, 0, 33, "Subsection"],
Cell[9172, 320, 475, 16, 46, "Text"],

Cell[CellGroupData[{
Cell[9672, 340, 76, 0, 42, "Subsubsection"],
Cell[9751, 342, 315, 5, 43, "Input"],
Cell[10069, 349, 103, 2, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[10209, 356, 96, 3, 28, "Subsubsection"],
Cell[10308, 361, 146, 2, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[10491, 368, 96, 3, 28, "Subsubsection"],
Cell[10590, 373, 81, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[10708, 379, 348, 9, 28, "Subsubsection"],
Cell[11059, 390, 340, 6, 107, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[11436, 401, 334, 8, 28, "Subsubsection"],
Cell[11773, 411, 79, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[11889, 417, 215, 6, 28, "Subsubsection"],
Cell[12107, 425, 79, 1, 27, "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[12235, 432, 41, 0, 33, "Subsection"],
Cell[12279, 434, 354, 6, 91, "Input"],
Cell[12636, 442, 269, 5, 59, "Input"],
Cell[12908, 449, 669, 12, 139, "Input"],
Cell[13580, 463, 873, 15, 155, "Input"],
Cell[14456, 480, 1065, 18, 171, "Input"],
Cell[15524, 500, 233, 4, 43, "Input"],
Cell[15760, 506, 1378, 23, 219, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[17175, 534, 59, 0, 33, "Subsection"],
Cell[17237, 536, 432, 7, 127, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[17718, 549, 119, 1, 39, "Section",
  CellTags->{"Experiment-specific user definitions", "Experiment"}],

Cell[CellGroupData[{
Cell[17862, 554, 52, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[17939, 558, 38, 0, 43, "Subsubsection"],
Cell[17980, 560, 159, 4, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18176, 569, 58, 0, 43, "Subsubsection"],
Cell[18237, 571, 110, 2, 47, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[18396, 579, 46, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[18467, 583, 42, 0, 43, "Subsubsection"],

Cell[CellGroupData[{
Cell[18534, 587, 338, 7, 79, "Input"],
Cell[18875, 596, 36606, 1618, 186, 20505, 1416, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[55530, 2220, 35, 0, 43, "Subsubsection"],

Cell[CellGroupData[{
Cell[55590, 2224, 255, 6, 63, "Input"],
Cell[55848, 2232, 37942, 1482, 186, 17552, 1227, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[93839, 3720, 40, 0, 43, "Subsubsection"],

Cell[CellGroupData[{
Cell[93904, 3724, 133, 3, 47, "Input"],
Cell[94040, 3729, 43602, 1797, 186, 22045, 1527, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[137715, 5534, 28, 0, 59, "Section"],

Cell[CellGroupData[{
Cell[137768, 5538, 33, 0, 49, "Subsection"],
Cell[137804, 5540, 321, 6, 103, "Print",
  CellTags->"Info3215543836-4508301"]
}, Open  ]],

Cell[CellGroupData[{
Cell[138162, 5551, 42, 0, 49, "Subsection"],
Cell[138207, 5553, 183, 5, 79, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[138427, 5563, 31, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[138483, 5567, 106, 3, 43, "Subsubsection"],
Cell[138592, 5572, 113, 2, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[138742, 5579, 118, 3, 43, "Subsubsection"],
Cell[138863, 5584, 132, 2, 47, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

